@@ -1,12 +1,12 @@
-/* $Header: /cvsroot/nco/nco/src/nco++/ncap2.cc,v 1.39 2007/05/24 14:50:20 hmb Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco++/ncap2.cc,v 1.100 2009/06/26 16:25:55 hmb Exp $ */
 
 /* ncap2 -- netCDF arithmetic processor */
 
 /* Purpose: Compute user-defined derived fields using forward algebraic notation applied to netCDF files */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   
+   License: GNU General Public License (GPL) Version 3
    The full license text is at http://www.gnu.org/copyleft/gpl.html 
    and in the file nco/doc/LICENSE in the NCO source distribution.
    
@@ -15,14 +15,14 @@
    libraries and to distribute the resulting executables under the terms 
    of the GPL, but in addition obeying the extra stipulations of the 
    HDF, netCDF, OPeNDAP, and UDUnits licenses.
-
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
    See the GNU General Public License for more details.
    
-   The original author of this software, Charlie Zender, wants to improve it
-   with the help of your suggestions, improvements, bug-reports, and patches.
+   The original author of this software, Charlie Zender, seeks to improve
+   it with your suggestions, contributions, bug-reports, and patches.
    Please contact the NCO project at http://nco.sf.net or write to
    Charlie Zender
    Department of Earth System Science
@@ -31,65 +31,74 @@
 
 /* Usage:
    ncap2 -O -D 1 -S ~/nco/data/ncap2.in ~/nco/data/in.nc ~/foo.nc
+   ncap2 -O -D 1 -v -S ~/nco/data/ncap2_tst.nco ~/nco/data/in.nc ~/foo.nc
    ncap2 -O -D 1 -s two=one+two ~/nco/data/in.nc ~/foo.nc
    ncap2 -O -v -s 'foo=exp(0.61)' ~/nco/data/in.nc ~/foo.nc;ncks ~/foo.nc
    ncap2 -O -v -D 1 -s "one_dmn_rec_var(0)=one_dmn_rec_var(0)+1" ~/nco/data/in.nc ~/foo.nc
    ncap2 -O -v -D 1 -s "three_dmn_rec_var(0,,)=three_dmn_rec_var(0,,)+1" ~/nco/data/in.nc ~/foo.nc */
 
-/* temporary define --not for release */
-//#define HAVE_CONFIG_H 1
-
 #ifdef HAVE_CONFIG_H
-#include "config.h" /* Autotools tokens */
+# include "config.h" /* Autotools tokens */
 #endif /* !HAVE_CONFIG_H */
 
-/* Standard C headers */
+// Standard C++ headers
+#include <string>
+
+// Standard C headers
 #include <assert.h>  /* assert() debugging macro */
+#include <math.h> /* sin cos cos sin 3.14159 */
 #include <stdio.h> /* stderr, FILE, NULL, etc. */
 #include <stdlib.h> /* atof, atoi, malloc, getopt */
 #include <string.h> /* strcmp. . . */
 #include <sys/stat.h> /* stat() */
 #include <time.h> /* machine time */
 #include <unistd.h> /* all sorts of POSIX stuff */
-#include <string>
-#include <math.h> /* sin cos cos sin 3.14159 */
 
 /* GNU getopt() is independent system header on FREEBSD, LINUX, LINUXALPHA, LINUXAMD, LINUXARM, WIN32
    AT&T getopt() is in unistd.h or stdlib.h on AIX, CRAY, NECSX, SUNMP, SUN4SOL2
-   fxm: I'm not sure what ALPHA and SGI do */
+   fxm: Unsure what ALPHA and SGI do */
 #ifndef HAVE_GETOPT_LONG
-#include "nco_getopt.h"
+# include "nco_getopt.h"
 #else /* !NEED_GETOPT_LONG */ 
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif /* !HAVE_GETOPT_H */ 
+# ifdef HAVE_GETOPT_H
+#  include <getopt.h>
+# endif /* !HAVE_GETOPT_H */ 
 #endif /* HAVE_GETOPT_LONG */
 
 /* 3rd party vendors */
+#ifdef ENABLE_GSL
+#include <gsl/gsl_errno.h>
+#include <gsl/gsl_rng.h>
+
+#endif // !ENABLE_GSL
 #include <netcdf.h> /* netCDF definitions and C library */
 #include "nco_netcdf.h"  /* NCO wrappers for libnetcdf.a */
 
 /* Personal headers */
 /* #define MAIN_PROGRAM_FILE MUST precede #include libnco.h */
 #define MAIN_PROGRAM_FILE
-#include "ncap2.hh" /* netCDF arithmetic processor-specific definitions (symbol table, ...) */
 #include "libnco++.hh" /* netCDF Operator (NCO) C++ library */
-#include "libnco.h"    /* netCDF Operator (NCO) library */
-#include "sdo_utl.hh"  /* error messages etc */
-
+#include "libnco.h" /* netCDF Operator (NCO) library */
 
 /* Global variables */
 size_t ncap_ncl_dpt_crr=0UL; /* [nbr] Depth of current #include file (incremented in ncap_lex.l) */
 size_t *ncap_ln_nbr_crr; /* [cnt] Line number (incremented in ncap_lex.l) */
 char **ncap_fl_spt_glb=NULL_CEWI; /* [fl] Script file */
+#ifdef ENABLE_GSL
+int ncap_gsl_mode_prec=0; /* Precision for GSL functions with mode_t argument (Airy, hypergeometric) */ 
+#endif // !ENABLE_GSL
+
+/* Forward Declaration */
+void pop_fmc_vtr(std::vector<fmc_cls> &fmc_vtr, vtl_cls *vfnc);
+void ram_vars_add(prs_cls *prs_arg);
 
 int 
 main(int argc,char **argv)
 {
   const char fnc_nm[]="main"; 
   FILE *yyin; /* file handle used to check file existance */
-  int parse_antlr(prs_sct*,char*,char*);
-
+  int parse_antlr(std::vector<prs_cls> &prs_vtr ,char*,char*);
+  
   /* fxm TODO nco652 */
   double rnd_nbr(double);
   
@@ -125,79 +134,43 @@ main(int argc,char **argv)
 #define NCAP_SPT_NBR_MAX 100
   char *spt_arg[NCAP_SPT_NBR_MAX]; /* fxm: Arbitrary size, should be dynamic */
   char *spt_arg_cat=NULL_CEWI; /* [sng] User-specified script */
-
-  const char * const CVS_Id="$Id: ncap2.cc,v 1.39 2007/05/24 14:50:20 hmb Exp $"; 
-  const char * const CVS_Revision="$Revision: 1.39 $";
-  const char * const opt_sht_lst="4ACcD:Ffhl:n:Oo:p:Rrs:S:vx-:"; /* [sng] Single letter command line options */
-
+  
+  const char * const att_nm_tmp="eulaVlliF_"; /* name used for netCDF4 name hack */
+  const char * const CVS_Id="$Id: ncap2.cc,v 1.100 2009/06/26 16:25:55 hmb Exp $"; 
+  const char * const CVS_Revision="$Revision: 1.100 $";
+  const char * const opt_sht_lst="34ACcD:FfhL:l:n:Oo:p:Rrs:S:t:vx-:"; /* [sng] Single letter command line options */
+  
   dmn_sct **dmn_in=NULL_CEWI;  /* [lst] Dimensions in input file */
   dmn_sct *dmn_new;
   dmn_sct *dmn_item;
-
-  // template lists
-  NcapVector<sym_sct*> sym_vtr;
+  
+  // Template lists
   NcapVector<dmn_sct*> dmn_in_vtr;  
   NcapVector<dmn_sct*> dmn_out_vtr;  
   
-  // Holder for attributtes and vectors
+  // Holder for attributes and vectors
   NcapVarVector var_vtr;
   
-  // Holder for attributtes and vectors in FIRST PARSE
+  // Holder for attributes and vectors in first parse
   NcapVarVector int_vtr;
-
-
+  
+  // Method/function holder
+  std::vector<fmc_cls> fmc_vtr;
+  
+  // Holder for prs_cls, NB: used for OpenMP
+  std::vector<prs_cls> prs_vtr;
+  
   extern char *optarg;
   extern int optind;
-
-  /* Math float prototypes required by AIX, Solaris, but not by Linux, IRIX */
-  /* Basic math: acos, asin, atan, cos, exp, fabs, log, log10, sin, sqrt, tan */
   
-  /* GNU g++ barfs at these float declartions -- remove if g++ used */
-#ifndef __GNUG__
-  extern float acosf(float);
-  extern float asinf(float);
-  extern float atanf(float);
-  extern float cosf(float);
-  extern float expf(float);
-  extern float fabsf(float); /* 20040629: Only AIX may need this */
-  extern float logf(float);
-  extern float log10f(float);
-  extern float rnd_nbrf(float);
-  extern float sinf(float);
-  extern float sqrtf(float);
-  extern float tanf(float);
-  
-  /* Advanced math: erf, erfc, gamma */
-  extern float erff(float);
-  extern float erfcf(float);
-  extern float gammaf(float);
-
-  /* Hyperbolic trigonometric: acosh, asinh, atanh, cosh, sinh, tanh */
-  extern float acoshf(float);
-  extern float asinhf(float);
-  extern float atanhf(float);
-  extern float coshf(float);
-  extern float sinhf(float);
-  extern float tanhf(float);
-  
-  /* Basic Rounding: ceil, floor */
-  extern float ceilf(float);
-  extern float floorf(float);
-  
-  /* Advanced Rounding: nearbyint, rint, round, trunc */
-  extern float nearbyintf(float);
-  extern float rintf(float);
-  extern float roundf(float);
-  extern float truncf(float);
-#endif
-
   int abb_arg_nbr=0;
+  int dfl_lvl=0; /* [enm] Deflate level */
   int fl_nbr=0;
   int fl_in_fmt; /* [enm] Input file format */
-  int fl_out_fmt=NC_FORMAT_CLASSIC; /* [enm] Output file format */
+  int fl_out_fmt=NCO_FORMAT_UNDEFINED; /* [enm] Output file format */
   int fll_md_old; /* [enm] Old fill mode */
-  int idx;
   int in_id;  
+  int idx;
   int jdx;
   int lmt_nbr=0; /* Option d. NB: lmt_nbr gets incremented */
   int nbr_dmn_ass=int_CEWI;/* Number of dimensions in temporary list */
@@ -212,8 +185,7 @@ main(int argc,char **argv)
   int out_id;  
   int rcd=NC_NOERR; /* [rcd] Return code */
   int var_id;
-  
-  int sym_tbl_nbr; /* [nbr] Size of symbol table */
+  int thr_nbr=int_CEWI; /* [nbr] Thread number Option t */
   
   lmt_sct **lmt=NULL_CEWI;
   
@@ -230,10 +202,8 @@ main(int argc,char **argv)
   var_sct **var_out;
   var_sct **var_prc;
   var_sct **var_prc_out;
-
-  aed_sct att_item; //Used to convert atts in vector to normal form  
-
-  prs_sct prs_arg; /* [sct] Global information required in parser routines */
+  
+  aed_sct att_item; // Used to convert atts in vector to normal form  
   
   static struct option opt_lng[]=
     { /* Structure ordered by short option key if possible */
@@ -244,10 +214,13 @@ main(int argc,char **argv)
       {"drt",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
       {"dirty",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
       {"mmr_drt",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
+      {"version",no_argument,0,0},
+      {"vrs",no_argument,0,0},
       /* Long options with argument, no short option counterpart */
       {"fl_fmt",required_argument,0,0},
       {"file_format",required_argument,0,0},
       /* Long options with short counterparts */
+      {"3",no_argument,0,'3'},
       {"4",no_argument,0,'4'},
       {"64bit",no_argument,0,'4'},
       {"netcdf4",no_argument,0,'4'},
@@ -265,6 +238,8 @@ main(int argc,char **argv)
       {"ftn",no_argument,0,'F'},
       {"history",no_argument,0,'h'},
       {"hst",no_argument,0,'h'},
+      {"dfl_lvl",required_argument,0,'L'}, /* [enm] Deflate level */
+      {"deflate",required_argument,0,'L'}, /* [enm] Deflate level */
       {"local",required_argument,0,'l'},
       {"lcl",required_argument,0,'l'},
       {"nintap",required_argument,0,'n'},
@@ -278,13 +253,13 @@ main(int argc,char **argv)
       {"revision",no_argument,0,'r'},
       {"file",required_argument,0,'S'},
       {"script-file",required_argument,0,'S'},
+      {"signal",no_argument,0,'z'},
       {"fl_spt",required_argument,0,'S'},
       {"spt",required_argument,0,'s'},
       {"script",required_argument,0,'s'},
+      {"thr_nbr",required_argument,0,'t'},
       {"units",no_argument,0,'u'},
       {"variable",no_argument,0,'v'},
-      {"version",no_argument,0,'r'},
-      {"vrs",no_argument,0,'r'},
       {"exclude",no_argument,0,'x'},
       {"xcl",no_argument,0,'x'},
       {"help",no_argument,0,'?'},
@@ -305,18 +280,25 @@ main(int argc,char **argv)
     /* NB: access to opt_crr is only valid when long_opt is detected */
     if(opt == EOF) break; /* Parse positional arguments once getopt_long() returns EOF */
     opt_crr=(char *)strdup(opt_lng[opt_idx].name);
-
+    
     /* Process long options without short option counterparts */
     if(opt == 0){
       if(!strcmp(opt_crr,"cln") || !strcmp(opt_crr,"mmr_cln") || !strcmp(opt_crr,"clean")) flg_cln=True; /* [flg] Clean memory prior to exit */
       if(!strcmp(opt_crr,"drt") || !strcmp(opt_crr,"mmr_drt") || !strcmp(opt_crr,"dirty")) flg_cln=False; /* [flg] Clean memory prior to exit */
       if(!strcmp(opt_crr,"fl_fmt") || !strcmp(opt_crr,"file_format")) rcd=nco_create_mode_prs(optarg,&fl_out_fmt);
+      if(!strcmp(opt_crr,"vrs") || !strcmp(opt_crr,"version")){
+	(void)nco_vrs_prn(CVS_Id,CVS_Revision);
+	nco_exit(EXIT_SUCCESS);
+      } /* endif "vrs" */
     } /* opt != 0 */
     /* Process short options */
     switch(opt){
     case 0: /* Long options have already been processed, return */
       break;
-    case '4': /* [flg] Catch-all to prescribe output storage format */
+    case '3': /* Request netCDF3 output storage format */
+      fl_out_fmt=NC_FORMAT_CLASSIC;
+      break;
+    case '4': /* Catch-all to prescribe output storage format */
       if(!strcmp(opt_crr,"64bit")) fl_out_fmt=NC_FORMAT_64BIT; else fl_out_fmt=NC_FORMAT_NETCDF4; 
       break;
     case 'A': /* Toggle FORCE_APPEND */
@@ -340,6 +322,9 @@ main(int argc,char **argv)
     case 'h': /* Toggle appending to history global attribute */
       HISTORY_APPEND=!HISTORY_APPEND;
       break;
+    case 'L': /* [enm] Deflate level. Default is 0. */
+      dfl_lvl=(int)strtol(optarg,(char **)NULL,10);
+      break;
     case 'l': /* Local path prefix for files retrieved from remote file system */
       fl_pth_lcl=(char *)strdup(optarg);
       break;
@@ -349,7 +334,7 @@ main(int argc,char **argv)
       fl_lst_abb=nco_lst_prs_2D(optarg,",",&abb_arg_nbr);
       if(abb_arg_nbr < 1 || abb_arg_nbr > 3)
 	err_prn(fnc_nm, "Incorrect abbreviation for file list\n");
-
+      
       break;
     case 'O': /* Toggle FORCE_OVERWRITE */
       FORCE_OVERWRITE=!FORCE_OVERWRITE;
@@ -364,8 +349,10 @@ main(int argc,char **argv)
       REMOVE_REMOTE_FILES_AFTER_PROCESSING=!REMOVE_REMOTE_FILES_AFTER_PROCESSING;
       break;
     case 'r': /* Print CVS program information and copyright notice */
-      (void)copyright_prn(CVS_Id,CVS_Revision);
+      (void)nco_vrs_prn(CVS_Id,CVS_Revision);
       (void)nco_lbr_vrs_prn();
+      (void)nco_cpy_prn();
+      (void)nco_cnf_prn();
       nco_exit(EXIT_SUCCESS);
       break;
     case 's': /* Copy command script for later processing */
@@ -377,6 +364,9 @@ main(int argc,char **argv)
     case 'S': /* Read command script from file rather than from command line */
       fl_spt_usr=(char *)strdup(optarg);
       break;
+    case 't': /* Thread number */
+      thr_nbr=(int)strtol(optarg,(char **)NULL,10);
+      break;
     case 'v': /* Variables to extract/exclude */
       PROCESS_ALL_VARS=False;
       nbr_xtr=0;
@@ -384,10 +374,11 @@ main(int argc,char **argv)
     case 'x': /* Exclude rather than extract variables specified with -v */
       EXCLUDE_INPUT_LIST=True;
       /*
-      if(EXCLUDE_INPUT_LIST) 
+	if(EXCLUDE_INPUT_LIST) 
         err_prn(fnc_nm,std::string(prg_nm_get())+ " does not currently implement -x option\n");
       */
       break;
+      
     case '?': /* Print proper usage */
       (void)nco_usg_prn();
       nco_exit(EXIT_SUCCESS);
@@ -419,89 +410,92 @@ main(int argc,char **argv)
     } /* end else */
   } /* end if */    
   
-  /* Create function table */
-  sym_tbl_nbr= /* fxm: Make this dynamic */
-    +12 /* Basic math: acos, asin, atan, cos, exp, fabs, log, log10, rnd_nbr, sin, sqrt, tan */
-    +1 /* Basic math synonyms: ln */
-    +6 /* Hyperbolic trigonometric: acosh, asinh, atanh, cosh, sinh, tanh */
-    +2 /* Basic Rounding: ceil, floor */
-    +4 /* Advanced Rounding: nearbyint, rint, round, trunc */
-    +3 /* Advanced math: erf, erfc, gamma */
-    ;
-  /* Basic math: acos, asin, atan, cos, exp, log, log10, rnd_nbr, sin, sqrt, tan */
+  /* create function/method vector */
 
-  sym_vtr.push_back(ncap_sym_init("acos",acos,acosf));  
-  sym_vtr.push_back(ncap_sym_init("asin",asin,asinf));
-  sym_vtr.push_back(ncap_sym_init("atan",atan,atanf));
-  sym_vtr.push_back(ncap_sym_init("cos",cos,cosf));  
-  sym_vtr.push_back(ncap_sym_init("exp",exp,expf));
-  sym_vtr.push_back(ncap_sym_init("fabs",fabs,fabsf));
-  sym_vtr.push_back(ncap_sym_init("log",log,logf));
-  sym_vtr.push_back(ncap_sym_init("log10",log10,log10f));
-  //sym_vtr.push_back(ncap_sym_init("rnd_nbr",rnd_nbr,rnd_nbrf));
-  sym_vtr.push_back(ncap_sym_init("sin",sin,sinf));
-  sym_vtr.push_back(ncap_sym_init("sqrt",sqrt,sqrtf));
-  sym_vtr.push_back(ncap_sym_init("tan",tan,tanf));
+  // Conversion functions
+  cnv_cls cnv_obj(true);
+  // Aggregate functions
+  agg_cls agg_obj(true);
+  // Utility Functions 
+  utl_cls utl_obj(true);
+  // Maths Functions
+  mth_cls mth_obj(true);
+  // Maths2 Functions
+  mth2_cls mth2_obj(true);
+  // Basic Functions
+  bsc_cls bsc_obj(true);
+  //PDQ functions
+  pdq_cls pdq_obj(true);
+  //Mask functions
+  msk_cls msk_obj(true);
+  //Pack functions
+  pck_cls pck_obj(true); 
+  //Sort functions
+  srt_cls srt_obj(true); 
+  //Unary functions
+  unr_cls unr_obj(true); 
+  //array functions
+  arr_cls arr_obj(true); 
 
-  /* Basic math synonyms: ln */
-  sym_vtr.push_back(ncap_sym_init("ln",log,logf)); /* ln() is synonym for log() */
   
-  /* Basic Rounding: ceil, fl<oor */
-  sym_vtr.push_back(ncap_sym_init("ceil",ceil,ceilf)); /* Round up to nearest integer */
-  sym_vtr.push_back(ncap_sym_init("floor",floor,floorf)); /* Round down to nearest integer */
+  //populate vector
+  (void)pop_fmc_vtr(fmc_vtr,&cnv_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&agg_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&utl_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&mth_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&mth2_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&bsc_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&pdq_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&msk_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&pck_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&srt_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&unr_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&arr_obj);
   
-  /* fxm: Change whole function symbol table section to autotools format #if HAVE_ERF ... */
 
-  /* Advanced math: erf, erfc, gamma
-     LINUX*, MACOSX*, and SUN* provide these functions with C89
-     20020122 and 20020422: AIX, CRAY, SGI*, WIN32 do not define erff(), erfcf(), gammaf() with C89
-     20050610: C99 mandates support for erf(), erfc(), tgamma()
-     Eventually users without C99 will forego ncap */
-#if defined(LINUX) || defined(LINUXAMD64)  || defined(MACOSX)
-  sym_vtr.push_back(ncap_sym_init("erf",erf,erff));
-  sym_vtr.push_back(ncap_sym_init("erfc",erfc,erfcf));
-  sym_vtr.push_back(ncap_sym_init("gamma",tgamma,tgammaf));
-#endif /* !LINUX */
-
-  /* Hyperbolic trigonometric: acosh, asinh, atanh, cosh, sinh, tanh
-     20020703: AIX, SGI*, WIN32 do not define acoshf, asinhf, atanhf
-     20050610: C99 mandates support for acosh(), asinh(), atanh(), cosh(), sinh(), tanh()
-     Eventually users without C99 will forego ncap */
-#if defined(LINUX) || defined(LINUXAMD64)
-  sym_vtr.push_back(ncap_sym_init("acosh",acosh,acoshf));
-  sym_vtr.push_back(ncap_sym_init("asinh",asinh,asinhf));
-  sym_vtr.push_back(ncap_sym_init("atanh",atanh,atanhf));
-  sym_vtr.push_back(ncap_sym_init("cosh",cosh,coshf));
-  sym_vtr.push_back(ncap_sym_init("sinh",sinh,sinhf));
-  sym_vtr.push_back(ncap_sym_init("tanh",tanh,tanhf));
-#endif /* !LINUX */
+  // GSL functions
+#ifdef ENABLE_GSL
+  char *str_ptr;
   
-  /* 20020703: AIX, MACOSX, SGI*, WIN32 do not define rintf
-     Only LINUX* supplies all of these and I do not care about them enough
-     to activate them on LINUX* but not on MACOSX* and SUN* */
- /* Advanced Rounding: nearbyint, rint, round, trunc */
-  /* Advanced Rounding: nearbyint, round, trunc */
-  /* sym_vtr.push(ncap_sym_init("nearbyint",nearbyint,nearbyintf)); *//* Round to integer value in floating point format using current rounding direction, do not raise inexact exceptions */
-  /* sym_vtr.push(ncap_sym_init("round",round,roundf)); *//* Round to nearest integer away from zero */
-  /* sym_vtr.push(ncap_sym_init("trunc",trunc,truncf)); *//* Round to nearest integer not larger in absolute value */
-  /* sym_vtr.push(ncap_sym_init("rint",rint,rintf)); *//* Round to integer value in floating point format using current rounding direction, raise inexact exceptions */
+  gsl_cls gsl_obj(true); 
+  gsl2_cls gsl2_obj(true); 
+  gsl_stt2_cls gsl_stt2_obj(true);
+  gsl_spl_cls gsl_spl_obj(true);
+ 
+  (void)pop_fmc_vtr(fmc_vtr,&gsl_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&gsl2_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&gsl_stt2_obj);
+  (void)pop_fmc_vtr(fmc_vtr,&gsl_spl_obj);
+
+  
+  /* Set GSL error handler */
+  gsl_set_error_handler_off(); 
+  
+  /* initialize global from environment variable */  
+  if((str_ptr=getenv("GSL_PREC_MODE")) != NULL) ncap_gsl_mode_prec=(int)strtol(str_ptr,(char **)NULL,10);
+  
+  if(ncap_gsl_mode_prec<0 || ncap_gsl_mode_prec>2) ncap_gsl_mode_prec=0;
+
+  /* create a generator chosen by the 
+    environment variables GSL_RNG_TYPE ,GSL_RNG_SEED */
+     
+ gsl_rng_env_setup();
 
 
-  /* add delete function for RAM variables --Temporary location */
-  /* function args are not used */
-    sym_vtr.push_back(ncap_sym_init("delete",tanh,tanhf));
-
-  /* now sort sym_vtr */
-  sym_vtr.sort(); 
-   
+#endif // !ENABLE_GSL
+  
+  //Sort Vector 
+  std::sort(fmc_vtr.begin(),fmc_vtr.end());
+  
   if(PRN_FNC_TBL){
-    /* ncap TODO #43: alphabetize this list */ 
-    (void)fprintf(stdout,"Maths functions available in %s:\n",prg_nm_get());
-    (void)fprintf(stdout,"Name\tFloat\tDouble\n"); 
-    for(idx=0;idx<sym_vtr.size();idx++)
-      (void)fprintf(stdout,"%s\t%c\t%c\n",sym_vtr[idx]->nm, (sym_vtr[idx]->fnc_flt ? 'y' : 'n'),(sym_vtr[idx]->fnc_dbl ? 'y' : 'n'));
+    (void)fprintf(stdout,"Methods available in %s:\n",prg_nm_get());
+    for(idx=0;idx<(signed int)fmc_vtr.size();idx++)
+      std::cout<< fmc_vtr[idx].fnm()<<"()"<<std::endl; 
     nco_exit(EXIT_SUCCESS);
-  } /* end if PRN_FNC_TBL */
+  }
+  
+  /* Initialize thread information */
+  thr_nbr=nco_openmp_ini(thr_nbr);
   
   /* Process positional arguments and fill in filenames */
   fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&fl_nbr,&fl_out,&FL_LST_IN_FROM_STDIN);
@@ -522,7 +516,7 @@ main(int argc,char **argv)
   
   //dmn_in=(dmn_sct **)nco_malloc(nbr_dmn_in*sizeof(dmn_sct *));
   for(idx=0;idx<nbr_dmn_in;idx++) dmn_in_vtr.push_back(nco_dmn_fll(in_id,dmn_lst[idx].id,dmn_lst[idx].nm));
-
+  
   //sort vector
   dmn_in_vtr.sort();
   dmn_in=&dmn_in_vtr[0];
@@ -530,9 +524,9 @@ main(int argc,char **argv)
   /* Merge hyperslab limit information into dimension structures */
   if(lmt_nbr > 0) (void)nco_dmn_lmt_mrg(dmn_in,nbr_dmn_in,lmt,lmt_nbr);
   
-  /* Make output and input files consanguinous fxm: TODO nco836 */
-  if(fl_in_fmt == NC_FORMAT_NETCDF4) fl_out_fmt=NC_FORMAT_NETCDF4;
-
+  /* Make output and input files consanguinous */
+  if(fl_out_fmt == NCO_FORMAT_UNDEFINED) fl_out_fmt=fl_in_fmt;
+  
   /* Open output file */
   fl_out_tmp=nco_fl_out_open(fl_out,FORCE_APPEND,FORCE_OVERWRITE,fl_out_fmt,&out_id);
   
@@ -542,48 +536,76 @@ main(int argc,char **argv)
   /* Catenate time-stamped command line to "history" global attribute */
   if(HISTORY_APPEND) (void)nco_hst_att_cat(out_id,cmd_ln);
   
+  if(thr_nbr > 0 && HISTORY_APPEND) (void)nco_thr_att_cat(out_id,thr_nbr);
+  
   (void)nco_enddef(out_id);
   
-  /* Set arguments for  script execution */
+  /* Set arguments for script execution NB: all these are used as references */
+  prs_cls prs_arg(dmn_in_vtr,dmn_out_vtr,fmc_vtr,var_vtr,int_vtr);
+  
   prs_arg.fl_in=fl_in; /* [sng] Input data file */
   prs_arg.in_id=in_id; /* [id] Input data file ID */
   prs_arg.fl_out=fl_out; /* [sng] Output data file */
   prs_arg.out_id=out_id; /* [id] Output data file ID */
+  //rcd=nco_open(fl_out_tmp, NC_NOWRITE|NC_SHARE,&prs_arg.r_out_id); /* Read Output file */
+  rcd=nco_open(fl_out_tmp, NC_NOWRITE,&prs_arg.r_out_id); /* Read Output file */
   
-  prs_arg.ptr_dmn_in_vtr=&dmn_in_vtr;
-  prs_arg.ptr_dmn_out_vtr=&dmn_out_vtr;
-  prs_arg.ptr_sym_vtr=&sym_vtr;
-  prs_arg.ptr_var_vtr=&var_vtr;
-  prs_arg.ptr_int_vtr=&int_vtr;
-  prs_arg.ntl_scn=False;   //[flg] Initial scan of script */
   prs_arg.FORTRAN_IDX_CNV=FORTRAN_IDX_CNV;
   prs_arg.ATT_PROPAGATE=ATT_PROPAGATE;      
   prs_arg.ATT_INHERIT=ATT_INHERIT;
-  prs_arg.NCAP_MPI_SORT=EXCLUDE_INPUT_LIST;
-
+  prs_arg.NCAP_MPI_SORT=(thr_nbr>1 ? true:false);
+  
+  prs_arg.dfl_lvl=dfl_lvl; /* [enm] Deflate level */
+  
+#ifdef NCO_NETCDF4_AND_FILLVALUE
+  prs_arg.NCAP4_FILL = (fl_out_fmt==NC_FORMAT_NETCDF4 || fl_out_fmt==NC_FORMAT_NETCDF4_CLASSIC);
+# else
+  prs_arg.NCAP4_FILL=false;
+#endif
+  prs_arg.ntl_scn=false;
+  (void)ram_vars_add(&prs_arg);
+  
+  prs_vtr.push_back(prs_arg); 
+  
+  for(idx=1;idx<thr_nbr;idx++){
+    prs_cls prs_tmp(prs_arg);
+    
+    // Open files for each thread
+    rcd=nco_open(fl_in,NC_NOWRITE,&prs_tmp.in_id);
+    
+    // Handle to read output only
+    //rcd=nco_open(fl_out_tmp, NC_NOWRITE|NC_SHARE,&prs_tmp.r_out_id);
+    rcd=nco_open(fl_out_tmp, NC_NOWRITE,&prs_tmp.r_out_id);
+    
+    // only one handle for reading & writing 
+    prs_tmp.out_id=out_id;
+    
+    prs_vtr.push_back(prs_tmp);
+  } /* end loop over threads */
+  
   if(fl_spt_usr == NULL_CEWI){
     /* No script file specified, look for command-line scripts */
     if(nbr_spt == 0)
       err_prn(fnc_nm,"No script file or command line scripts specified\nHINT Use, e.g., -s \"foo=bar\"\n");
-      
-     /* Print all command-line scripts */
-     if(dbg_lvl_get() > 0){
-       for(idx=0;idx<nbr_spt;idx++) 
-         (void)fprintf(stderr,"spt_arg[%d] = %s\n",idx,spt_arg[idx]);
-     } /* endif debug */
-      
-     /* Parse command line scripts */
-     fl_spt_usr=(char *)strdup("Command-line script");
-     }else{ /* ...endif command-line scripts, begin script file... */
-      /* Open script file for reading */
-      if((yyin=fopen(fl_spt_usr,"r")) == NULL_CEWI)
-        err_prn(fnc_nm,"Unable to open script file "+std::string(fl_spt_usr));
-      fclose(yyin); 
-    } /* end else script file */
     
+    /* Print all command-line scripts */
+    if(dbg_lvl_get() > 0){
+      for(idx=0;idx<nbr_spt;idx++) 
+	(void)fprintf(stderr,"spt_arg[%d] = %s\n",idx,spt_arg[idx]);
+    } /* endif debug */
+    
+    /* Parse command line scripts */
+    fl_spt_usr=(char *)strdup("Command-line script");
+  }else{ /* ...endif command-line scripts, begin script file... */
+    /* Open script file for reading */
+    if((yyin=fopen(fl_spt_usr,"r")) == NULL_CEWI)
+      err_prn(fnc_nm,"Unable to open script file "+std::string(fl_spt_usr));
+    fclose(yyin); 
+  } /* end else script file */
+  
   /* Invoke ANTLR parser */
-  rcd=parse_antlr(&prs_arg,fl_spt_usr,spt_arg_cat);
-    
+  rcd=parse_antlr(prs_vtr,fl_spt_usr,spt_arg_cat);
+  
   /* Get number of variables in output file */
   rcd=nco_inq(out_id,(int *)NULL,&nbr_var_fl,(int *)NULL,(int*)NULL);
   
@@ -624,7 +646,7 @@ main(int argc,char **argv)
   
   /* Free current list of all dimensions in input file */
   dmn_lst=nco_nm_id_lst_free(dmn_lst,nbr_dmn_in);
-
+  
   /* Make list of dimensions of variables in xtr_lst */
   if(nbr_xtr > 0) dmn_lst=nco_dmn_lst_ass_var(in_id,xtr_lst,nbr_xtr,&nbr_dmn_ass);
   
@@ -638,17 +660,16 @@ main(int argc,char **argv)
     (void)nco_dmn_dfn(fl_out,out_id,&dmn_new,1);
     (void)nco_dmn_xrf(dmn_new,dmn_item);
     (void)dmn_out_vtr.push_back(dmn_new);
-   }  /* end loop over idx */
+  }  /* end loop over idx */
   
   /* Free current list of all dimensions in input file */
   dmn_lst=nco_nm_id_lst_free(dmn_lst,nbr_dmn_ass);
-
+  
   /* Dimensions for manually specified extracted variables are now defined in output file
      Add coordinate variables to extraction list
      If EXTRACT_ALL_COORDINATES then write associated dimension to output */
-
-
- if(EXTRACT_ASSOCIATED_COORDINATES){
+  
+  if(EXTRACT_ASSOCIATED_COORDINATES){
     for(idx=0;idx<dmn_in_vtr.size();idx++){
       if(!dmn_in_vtr[idx]->is_crd_dmn) continue;
       
@@ -665,7 +686,7 @@ main(int argc,char **argv)
       if(dmn_in_vtr[idx]->xrf){
 	for(jdx=0;jdx<nbr_xtr;jdx++)
 	  if(!strcmp(xtr_lst[jdx].nm,dmn_in_vtr[idx]->nm)) break;
-
+	
 	if(jdx != nbr_xtr) continue;
 	/* If coordinate is not on list then add it to extraction list */
 	xtr_lst=(nm_id_sct *)nco_realloc(xtr_lst,(nbr_xtr+1)*sizeof(nm_id_sct));
@@ -674,13 +695,13 @@ main(int argc,char **argv)
       } /* endif */
     } /* end loop over idx */	      
   } /* end if */ 
- 
+  
   /* Is this an CCM/CCSM/CF-format history tape? */
   CNV_CCM_CCSM_CF=nco_cnv_ccm_ccsm_cf_inq(in_id);
-
+  
   /* Add coordinates defined by CF convention */
   if(CNV_CCM_CCSM_CF && (EXTRACT_ALL_COORDINATES || EXTRACT_ASSOCIATED_COORDINATES)) xtr_lst=nco_cnv_cf_crd_add(in_id,xtr_lst,&nbr_xtr);
-
+  
   /* Subtract list A again (it may contain re-defined coordinates) */
   if(nbr_xtr > 0) xtr_lst=nco_var_lst_sub(xtr_lst,&nbr_xtr,xtr_lst_a,nbr_lst_a);
   
@@ -697,51 +718,68 @@ main(int argc,char **argv)
     (void)nco_xrf_dmn(var_out[idx]);
   } /* end loop over idx */
   
-
   /* NB: ncap is not well-suited for nco_var_lst_dvd() */
   /* Divide variable lists into lists of fixed variables and variables to be processed */
   (void)nco_var_lst_dvd(var,var_out,nbr_xtr,CNV_CCM_CCSM_CF,nco_pck_plc_nil,nco_pck_map_nil,(dmn_sct **)NULL,(int)0,&var_fix,&var_fix_out,&nbr_var_fix,&var_prc,&var_prc_out,&nbr_var_prc);
   
   /* csz: Why not call this with var_fix? */
   /* Define non-processed vars */
-  (void)nco_var_dfn(in_id,fl_out,out_id,var_out,nbr_xtr,(dmn_sct **)NULL,(int)0,nco_pck_plc_nil,nco_pck_map_nil);
+  (void)nco_var_dfn(in_id,fl_out,out_id,var_out,nbr_xtr,(dmn_sct **)NULL,(int)0,nco_pck_plc_nil,nco_pck_map_nil,dfl_lvl);
   
-
   /* Write out new attributes possibly overwriting old ones */
   for(idx=0;idx<var_vtr.size();idx++){
-
-    // Check if attrribute
-    if( var_vtr[idx]->xpr_typ != ncap_att) continue;
-  
-    // Skip misssing values for now !!!
-    if(var_vtr[idx]->getAtt() =="missing_value") continue;
-        
- 
+    /* write misssing value contained inside var */
+    if(var_vtr[idx]->xpr_typ == ncap_var) {
+      var_sct *var_ref;
+      
+      var_ref=var_vtr[idx]->var;
+      rcd=nco_inq_varid_flg(out_id,var_ref->nm,&var_id);
+      
+      /* skip RAM vars and orphaned vars */
+      if(rcd!=NC_NOERR|| !var_ref->has_mss_val)
+        continue;  
+      
+      /* Do netdf4 name hack */
+      if(prs_arg.NCAP4_FILL){
+        (void)nco_put_att(out_id,var_id,att_nm_tmp,var_ref->type,1,var_ref->mss_val.vp);
+        (void)nco_rename_att(out_id,var_id,att_nm_tmp,nco_mss_val_sng_get());
+      }else{
+        (void)nco_put_att(out_id,var_id,nco_mss_val_sng_get(),var_ref->type,1,var_ref->mss_val.vp);
+      }
+      continue;
+    }
+    
+    
+    /* Skip misssing values for now !!! */
+    if(var_vtr[idx]->getAtt() == nco_mss_val_sng_get()) 
+      continue;     
+    
     att_item.att_nm=strdup(var_vtr[idx]->getAtt().c_str());
     att_item.var_nm=strdup(var_vtr[idx]->getVar().c_str());
     att_item.sz=var_vtr[idx]->var->sz;
     att_item.type=var_vtr[idx]->var->type;
     att_item.val=var_vtr[idx]->var->val;
     att_item.mode=aed_overwrite;
-
+    
     if(!strcmp(att_item.var_nm,"global")) 
-       var_id=NC_GLOBAL;
+      var_id=NC_GLOBAL;
     else {
-       rcd=nco_inq_varid_flg(out_id,att_item.var_nm,&var_id);
-       if(rcd != NC_NOERR)  continue;
+      rcd=nco_inq_varid_flg(out_id,att_item.var_nm,&var_id);
+      if(rcd != NC_NOERR)  goto cln_up;
     }
     // Check size;
-      if(att_item.sz > NC_MAX_ATTRS ){ 
-	(void)fprintf(stdout,"%s: Attribute %s size %ld excceeds maximium %d\n",prg_nm_get(),att_item.att_nm,att_item.sz, NC_MAX_ATTRS );
-	continue;
-      }
+    if(att_item.sz > NC_MAX_ATTRS ){ 
+      (void)fprintf(stdout,"%s: Attribute %s size %ld excceeds maximium %d\n",prg_nm_get(),att_item.att_nm,att_item.sz, NC_MAX_ATTRS );
+      goto cln_up;
+    }
     /* NB: These attributes should probably be written prior to last data mode */
-
-
+    
+    
     (void)nco_aed_prc(out_id,var_id,att_item);
-     att_item.var_nm=(char*)nco_free(att_item.var_nm);
-     att_item.att_nm=(char*)nco_free(att_item.att_nm);
-   }/* end for */
+    
+  cln_up: att_item.var_nm=(char*)nco_free(att_item.var_nm);
+    att_item.att_nm=(char*)nco_free(att_item.att_nm);
+  }/* end for */
   
   /* Turn off default filling behavior to enhance efficiency */
   rcd=nco_set_fill(out_id,NC_NOFILL,&fll_md_old);
@@ -755,6 +793,12 @@ main(int argc,char **argv)
   /* Close input netCDF file */
   rcd=nco_close(in_id);
   
+  /* Close all files in threads --except main thread */
+  for( idx=1; idx<thr_nbr; idx++){
+    rcd=nco_close(prs_vtr[idx].in_id);
+    rcd=nco_close(prs_vtr[idx].r_out_id);
+  }
+  
   /* Remove local copy of file */
   if(FILE_RETRIEVED_FROM_REMOTE_LOCATION && REMOVE_REMOTE_FILES_AFTER_PROCESSING) (void)nco_fl_rm(fl_in);
   
@@ -764,14 +808,6 @@ main(int argc,char **argv)
   /* Clean memory unless dirty memory allowed */
   if(flg_cln){
     /* ncap-specific memory */
-    int sz;
-    sz=sym_vtr.size();
-    for(idx=0; idx<sym_vtr.size() ;idx++){
-      sym_vtr[idx]->nm=(char*)nco_free(sym_vtr[idx]->nm);
-      (void)nco_free(sym_vtr[idx]);
-    } 
-    
-    
     if(fl_spt_usr != NULL_CEWI) fl_spt_usr=(char *)nco_free(fl_spt_usr);
     
     /* Free extraction lists */ 
@@ -797,24 +833,36 @@ main(int argc,char **argv)
     /* Free limits */
     for(idx=0;idx<lmt_nbr;idx++) lmt_arg[idx]=(char *)nco_free(lmt_arg[idx]);
     if(lmt_nbr > 0) lmt=nco_lmt_lst_free(lmt,lmt_nbr);
-    
     /* Free dimension vectors */
     if(dmn_in_vtr.size() > 0) { 
-      for(idx=0; idx < dmn_in_vtr.size(); idx++)
+      for(idx=0;idx<dmn_in_vtr.size();idx++)
 	(void)nco_dmn_free(dmn_in_vtr[idx]);
     }
-    
     if(dmn_out_vtr.size() > 0) { 
-      for(idx=0; idx < dmn_out_vtr.size(); idx++)
+      for(idx=0;idx< dmn_out_vtr.size();idx++)
 	(void)nco_dmn_free(dmn_out_vtr[idx]);
     }
     
     /* Free var_vtr */
     if(var_vtr.size() > 0) { 
       for(idx=0; idx < var_vtr.size(); idx++)
-            delete var_vtr[idx];
+	delete var_vtr[idx];
     }  
     
+    /* Clear vectors */
+    
+    fmc_vtr.clear();
+    cnv_obj.fmc_vtr.clear();
+    agg_obj.fmc_vtr.clear();
+    utl_obj.fmc_vtr.clear();
+    mth_obj.fmc_vtr.clear();
+    mth2_obj.fmc_vtr.clear();
+    bsc_obj.fmc_vtr.clear();
+    pdq_obj.fmc_vtr.clear();
+    msk_obj.fmc_vtr.clear();
+    pck_obj.fmc_vtr.clear();     
+
+
     /* Free variable lists */
     if(nbr_xtr > 0) var=nco_var_lst_free(var,nbr_xtr);
     if(nbr_xtr > 0) var_out=nco_var_lst_free(var_out,nbr_xtr);
@@ -827,3 +875,57 @@ main(int argc,char **argv)
   nco_exit_gracefully();
   return EXIT_SUCCESS;
 } /* end main() */
+
+// Copy vector elements
+void 
+pop_fmc_vtr
+(std::vector<fmc_cls> &fmc_vtr, 
+ vtl_cls *vfnc)
+{
+  // De-reference
+  std::vector<fmc_cls> &lcl_vtr=*vfnc->lst_vtr();
+  std::copy(lcl_vtr.begin(),lcl_vtr.end(),inserter(fmc_vtr,fmc_vtr.end())  );
+}
+
+// Add some useful constants as RAM variables 
+void
+ram_vars_add
+(prs_cls *prs_arg)
+{
+  var_sct *var1;
+  
+  var1=ncap_sclr_var_mk(std::string("__BYTE"),nco_int(NC_BYTE));
+  prs_arg->ncap_var_write(var1,true);
+  
+  var1=ncap_sclr_var_mk(std::string("__CHAR"),nco_int(NC_CHAR));
+  prs_arg->ncap_var_write(var1,true);
+  
+  var1=ncap_sclr_var_mk(std::string("__SHORT"),nco_int(NC_SHORT));
+  prs_arg->ncap_var_write(var1,true);
+  
+  var1=ncap_sclr_var_mk(std::string("__INT"),nco_int(NC_INT));
+  prs_arg->ncap_var_write(var1,true);
+  
+  var1=ncap_sclr_var_mk(std::string("__FLOAT"),nco_int(NC_FLOAT));
+  prs_arg->ncap_var_write(var1,true);
+  
+  var1=ncap_sclr_var_mk(std::string("__DOUBLE"),nco_int(NC_DOUBLE));
+  prs_arg->ncap_var_write(var1,true);
+  
+#ifdef ENABLE_NETCDF4
+  var1=ncap_sclr_var_mk(std::string("__UBYTE"),nco_int(NC_UBYTE));
+  prs_arg->ncap_var_write(var1,true); 
+  
+  var1=ncap_sclr_var_mk(std::string("__USHORT"),nco_int(NC_USHORT));
+  prs_arg->ncap_var_write(var1,true);
+  
+  var1=ncap_sclr_var_mk(std::string("__UINT"),nco_int(NC_UINT));
+  prs_arg->ncap_var_write(var1,true);
+  
+  var1=ncap_sclr_var_mk(std::string("__INT64"),nco_int(NC_INT64));
+  prs_arg->ncap_var_write(var1,true);
+  
+  var1=ncap_sclr_var_mk(std::string("__UINT64"),nco_int(NC_UINT64));
+  prs_arg->ncap_var_write(var1,true);
+#endif // !ENABLE_NETCDF4
+} // end ram_vars_add()
