@@ -2,7 +2,7 @@
 #define INC_ncoParser_hpp_
 
 #include <antlr/config.hpp>
-/* $ANTLR 2.7.6 (20070220): "ncoGrammer.g" -> "ncoParser.hpp"$ */
+/* $ANTLR 2.7.7 (20090306): "ncoGrammer.g" -> "ncoParser.hpp"$ */
 #include <antlr/TokenStream.hpp>
 #include <antlr/TokenBuffer.hpp>
 #include "ncoParserTokenTypes.hpp"
@@ -10,28 +10,55 @@
 
 #line 1 "ncoGrammer.g"
 
-    #include <math.h>
-    #include <malloc.h>
+/* $Header: /cvsroot/nco/nco/src/nco++/ncoParser.hpp,v 1.76 2009/05/07 04:41:15 zender Exp $ */
+
+/* Purpose: ANTLR Grammar and support files for ncap2 */
+
+/* Copyright (C) 2005--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
+
+    // C Standard Headers
     #include <assert.h>
     #include <ctype.h>
+    #include <malloc.h>
+    #include <math.h>
+    #if !(defined __xlC__) && !(defined SGIMP64) // C++ compilers that do not allow stdint.h
+    #include <stdint.h> // Required by g++ for LLONG_MAX, ULLONG_MAX, by icpc for int64_t    
+    #endif // C++ compilers that do not allow stdint.h
+
+    // C++ Standard Headers 
+    #include <algorithm>
     #include <iostream>
     #include <sstream>
+    #include <fstream>
     #include <string>
-    #include <algorithm>
-    #include "ncap2.hh"
+  
+    // Custom Headers
+    #include "prs_cls.hh"
+    #include "ncap2_utl.hh"
+    #include "fmc_cls.hh"
     #include "NcapVar.hh"
     #include "NcapVarVector.hh"
     #include "sdo_utl.hh" // SDO stand-alone utilities: dbg/err/wrn_prn()
     #include "NcapVector.hh"
+    #include "antlr/TokenStreamSelector.hpp"
+    #include "ncoParser.hpp"
+    #include "Invoke.hh"
+
     ANTLR_USING_NAMESPACE(std);
     ANTLR_USING_NAMESPACE(antlr);
-    
 
-#line 31 "ncoParser.hpp"
+#line 53 "ncoParser.hpp"
 class CUSTOM_API ncoParser : public ANTLR_USE_NAMESPACE(antlr)LLkParser, public ncoParserTokenTypes
 {
-#line 1 "ncoGrammer.g"
-#line 35 "ncoParser.hpp"
+#line 81 "ncoGrammer.g"
+
+
+public:
+   std::vector<std::string> inc_vtr;
+
+#line 57 "ncoParser.hpp"
 public:
 	void initializeASTFactory( ANTLR_USE_NAMESPACE(antlr)ASTFactory& factory );
 protected:
@@ -58,35 +85,32 @@ public:
 	}
 	public: void program();
 	public: void statement();
-	public: void block();
-	public: void if_stmt();
-	public: void assign_statement();
-	public: void def_dim();
 	public: void expr();
-	public: void hyper_slb();
+	public: void for_stmt();
+	public: void block();
+	public: void lmt();
 	public: void lmt_list();
 	public: void dmn_list();
-	public: void arg_list();
 	public: void dmn_arg_list();
+	public: void value_list();
+	public: void arg_list();
+	public: void call_ref();
 	public: void func_arg();
-	public: void func_exp();
-	public: void primary_exp();
-	public: void meth();
-	public: void prop_exp();
+	public: void hyper_slb();
 	public: void meth_exp();
+	public: void primary_exp();
 	public: void unaryleft_exp();
 	public: void unary_exp();
 	public: void pow_exp();
 	public: void mexpr();
 	public: void add_expr();
+	public: void frel_expr();
 	public: void rel_expr();
 	public: void eq_expr();
 	public: void lmul_expr();
 	public: void lor_expr();
 	public: void cond_expr();
 	public: void ass_expr();
-	public: void value_list();
-	public: void lmt();
 	public: void imaginary_token();
 public:
 	ANTLR_USE_NAMESPACE(antlr)RefAST getAST()
@@ -99,10 +123,10 @@ protected:
 private:
 	static const char* tokenNames[];
 #ifndef NO_STATIC_CONSTS
-	static const int NUM_TOKENS = 103;
+	static const int NUM_TOKENS = 113;
 #else
 	enum {
-		NUM_TOKENS = 103
+		NUM_TOKENS = 113
 	};
 #endif
 	
@@ -114,10 +138,6 @@ private:
 	static const ANTLR_USE_NAMESPACE(antlr)BitSet _tokenSet_2;
 	static const unsigned long _tokenSet_3_data_[];
 	static const ANTLR_USE_NAMESPACE(antlr)BitSet _tokenSet_3;
-	static const unsigned long _tokenSet_4_data_[];
-	static const ANTLR_USE_NAMESPACE(antlr)BitSet _tokenSet_4;
-	static const unsigned long _tokenSet_5_data_[];
-	static const ANTLR_USE_NAMESPACE(antlr)BitSet _tokenSet_5;
 };
 
 #endif /*INC_ncoParser_hpp_*/
