@@ -3,53 +3,86 @@
 
 #include <antlr/config.hpp>
 #include "ncoParserTokenTypes.hpp"
-/* $ANTLR 2.7.6 (20070220): "ncoGrammer.g" -> "ncoTree.hpp"$ */
+/* $ANTLR 2.7.7 (20090306): "ncoGrammer.g" -> "ncoTree.hpp"$ */
 #include <antlr/TreeParser.hpp>
 
 #line 1 "ncoGrammer.g"
 
-    #include <math.h>
-    #include <malloc.h>
+/* $Header: /cvsroot/nco/nco/src/nco++/ncoTree.hpp,v 1.89 2009/07/07 15:40:39 zender Exp $ */
+
+/* Purpose: ANTLR Grammar and support files for ncap2 */
+
+/* Copyright (C) 2005--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
+
+    // C Standard Headers
     #include <assert.h>
     #include <ctype.h>
+    #include <malloc.h>
+    #include <math.h>
+    #if !(defined __xlC__) && !(defined SGIMP64) // C++ compilers that do not allow stdint.h
+    #include <stdint.h> // Required by g++ for LLONG_MAX, ULLONG_MAX, by icpc for int64_t    
+    #endif // C++ compilers that do not allow stdint.h
+
+    // C++ Standard Headers 
+    #include <algorithm>
     #include <iostream>
     #include <sstream>
+    #include <fstream>
     #include <string>
-    #include <algorithm>
-    #include "ncap2.hh"
+  
+    // Custom Headers
+    #include "prs_cls.hh"
+    #include "ncap2_utl.hh"
+    #include "fmc_cls.hh"
     #include "NcapVar.hh"
     #include "NcapVarVector.hh"
     #include "sdo_utl.hh" // SDO stand-alone utilities: dbg/err/wrn_prn()
     #include "NcapVector.hh"
+    #include "antlr/TokenStreamSelector.hpp"
+    #include "ncoParser.hpp"
+    #include "Invoke.hh"
+
     ANTLR_USING_NAMESPACE(std);
     ANTLR_USING_NAMESPACE(antlr);
-    
 
-#line 29 "ncoTree.hpp"
+#line 51 "ncoTree.hpp"
 class CUSTOM_API ncoTree : public ANTLR_USE_NAMESPACE(antlr)TreeParser, public ncoParserTokenTypes
 {
-#line 506 "ncoGrammer.g"
+#line 645 "ncoGrammer.g"
 
 
 private:
-    prs_sct *prs_arg;
+    //prs_cls *prs_arg;
     bool bcst;
     var_sct* var_cst;
 public:
-    void setTable(prs_sct *prs_in){
-        prs_arg=prs_in;
+    prs_cls *prs_arg;
+    ASTFactory myFactory;
+
+
+
+
+     //Structure to hold AST pointers to indices in hyperslabs -only temporary 
+     typedef struct{
+        ANTLR_USE_NAMESPACE(antlr)RefAST ind[3];
+     } ast_lmt_sct;   
+
+    void setTable(prs_cls *prs_in){
+        prs_arg=prs_in; 
+
     }
-    // Customized Constructor
-    ncoTree(prs_sct *prs_in){
+   // Customized Constructor
+    ncoTree(prs_cls *prs_in){
         prs_arg=prs_in;
         // default is NO. Casting variable set to true 
         // causes casting in function out(). var_cst must 
         // then be defined 
-        bcst=false;  
+        bcst=false; 
         ncoTree();
     }
 
-
 int 
 lmt_init(
 RefAST aRef, 
@@ -140,7 +173,6 @@ int nbr_dmn;
 int idx;
 int jdx;
 long lcl_ind[3];
-char *buff;
 
 var_sct *var_out;
 lmt_sct *lmt_ptr;
@@ -190,42 +222,39 @@ nbr_dmn=lmt_init(lmt,ast_lmt_vtr);
      lmt_ptr->min_sng=NULL;
      lmt_ptr->max_sng=NULL;
      lmt_ptr->srd_sng=NULL;
+     lmt_ptr->is_usr_spc_min=False;
+     lmt_ptr->is_usr_spc_max=False;
+
      /* rec_skp_nsh_spf is used for record dimension in multi-file operators */
      lmt_ptr->rec_skp_nsh_spf=0L; /* Number of records skipped in initial superfluous files */
     
     
     /* Fill in structure */
-    if( lcl_ind[0] >= 0) {
-          buff=(char*)nco_malloc(20*sizeof(char));
-          (void)sprintf(buff, "%ld",lcl_ind[0]);
-           lmt_ptr->min_sng=buff; 
+    if( lcl_ind[0] >= 0){ 
+           lmt_ptr->is_usr_spc_min=True;
+           lmt_ptr->srt=lcl_ind[0]; 
     }    
 
+
     /* Fill in structure */
     if( lcl_ind[1] >= 0) {
-          buff=(char*)nco_malloc(20*sizeof(char));
-          (void)sprintf(buff, "%ld",lcl_ind[1]);
-           lmt_ptr->max_sng=buff;
+           lmt_ptr->is_usr_spc_max=True;
+           lmt_ptr->end=lcl_ind[1];
     }    
 
     /* Fill in structure */
     if( lcl_ind[2] > 0) {
-          buff=(char*)nco_malloc(20*sizeof(char));
-          (void)sprintf(buff, "%ld",lcl_ind[2]);
-           lmt_ptr->srd_sng=buff;
+           lmt_ptr->srd_sng=strdup("~fill_in");
+           lmt_ptr->srd=lcl_ind[2];
     }    
 
     /* need to deal with situation where only start is defined -- ie picking only a single value */
     if(lcl_ind[0] >=0 && lcl_ind[1]==-2){
-          buff=(char*)nco_malloc(20*sizeof(char));
-          (void)sprintf(buff, "%ld",lcl_ind[0]);
-          lmt_ptr->max_sng=buff; 
+          lmt_ptr->is_usr_spc_max=True;
+          lmt_ptr->end=lcl_ind[0]; 
     }    
 
 
-    if(lmt_ptr->max_sng == NULL) lmt_ptr->is_usr_spc_max=False; else lmt_ptr->is_usr_spc_max=True;
-    if(lmt_ptr->min_sng == NULL) lmt_ptr->is_usr_spc_min=False; else lmt_ptr->is_usr_spc_min=True;
-
     lmt_vtr.push_back(lmt_ptr);
 
 
@@ -236,7 +265,7 @@ nbr_dmn=lmt_init(lmt,ast_lmt_vtr);
 } /* end lmt_mk */
 
 
-
+/* Legacy run -- will remove soon
 public:
     void run(RefAST tr){
         while(tr) {
@@ -244,46 +273,63 @@ public:
           tr=tr->getNextSibling();   
         }
     }
+*/
 
 public:
     void run_dbl(RefAST tr,int icnt){
      int idx=0;
      RefAST ntr=tr;
-      
+
+     extern int      
+     ncap_omp_exe(
+     std::vector< std::vector<RefAST> > &all_ast_vtr,
+     ncoTree** wlk_ptr,
+     int nbr_wlk);
+
+     if(tr== ANTLR_USE_NAMESPACE(antlr)nullAST)
+        err_prn("run_dbl"," REPORTS given a null AST Refrence\n");
+            
      //small list dont bother with double parsing     
      if(icnt <4) goto small;
 
      //Initial scan
      prs_arg->ntl_scn=True;
      while(idx++ < icnt){
-       (void)statements(ntr);   
+         (void)statements(ntr);   
        ntr=ntr->getNextSibling();   
      }
 
      //Define variables in output
     (void)nco_redef(prs_arg->out_id);  
-    (void)ncap_def_ntl_scn(prs_arg);
+    (void)prs_arg->ncap_def_ntl_scn();
     (void)nco_enddef(prs_arg->out_id);  
 
+    // see if below does anything ? 
+    (void)nco_sync(prs_arg->out_id); 
+
 
     // Sort expressions - MPI preparation
     if(prs_arg->NCAP_MPI_SORT){  
        prs_arg->ntl_scn=False;
       // nb A vector of vectors
       std::vector< std::vector<RefAST> > all_ast_vtr;
+      ncoTree **wlk_vtr=(ncoTree**)NULL; 
 
-      // Populate  vector 
-      (void)ncap_mpi_srt(tr,icnt,all_ast_vtr);
-
-      // Evaluate expressions
+      // Populate and sort  vector 
+      (void)ncap_mpi_srt(tr,icnt,all_ast_vtr,prs_arg);
+       
+      // Evaluate expressions (execute)
+      (void)ncap_omp_exe(all_ast_vtr,wlk_vtr,0);  
+      
+      /*  
       for(unsigned vtr_idx=0 ; vtr_idx<all_ast_vtr.size(); vtr_idx++)
         for(unsigned jdx=0 ; jdx<all_ast_vtr[vtr_idx].size(); jdx++)
 	     (void)statements(all_ast_vtr[vtr_idx][jdx]);
-        
+      */
+      
     goto end;
     } //end if
 
-   
 
 
 small: 
@@ -299,12 +345,13 @@ end: ;
    }
 
 public:
-    void run_exe(RefAST tr){
+    int run_exe(RefAST tr, int nbr_dpt){
     // number of statements in block
     int nbr_stmt=0;
     int idx;
     int icnt=0;
-    int gtyp;
+    int ntyp;
+    int iret=0;
     
     RefAST etr=ANTLR_USE_NAMESPACE(antlr)nullAST;
     RefAST ntr;
@@ -315,46 +362,62 @@ public:
             
      
     
-    if(nbr_stmt <4){
+    if(nbr_stmt <4 || nbr_dpt>0 ){
         prs_arg->ntl_scn=False;
         ntr=tr;
-        do (void)statements(ntr);   
-        while(ntr=ntr->getNextSibling());   
+        do{ 
+          iret=statements(ntr);
+          // break if jump statement   
+          if(iret==BREAK || iret==CONTINUE) 
+           break; 
+        } while(ntr=ntr->getNextSibling());   
         goto exit;
     }
   
     ntr=tr;
 
     for(idx=0 ; idx < nbr_stmt; idx++){
-      gtyp=ntr->getType();
-      // we have hit an IF or a code block
-      if(gtyp==BLOCK || gtyp==IF ||gtyp==DEFDIM ) {
+      ntyp=ntr->getType();
+      // we have hit an IF or a basic block
+      if(ntyp==BLOCK || ntyp==IF ||ntyp==DEFDIM || ntyp==WHILE ||ntyp==FOR || ntyp==FEXPR ||ntyp==WHERE) {
+      //  if(ntyp != EXPR ){ 
         if(icnt>0) 
          (void)run_dbl(etr,icnt);
         icnt=0;
         etr=ANTLR_USE_NAMESPACE(antlr)nullAST;; 
         prs_arg->ntl_scn=False;
-        (void)statements(ntr);      
-       }
-
-       if(gtyp==EXPR || gtyp== NULL_NODE) 
+        iret=statements(ntr);      
+      }else{
         if(icnt++==0) etr=ntr;
-        
-       
-      ntr=ntr->getNextSibling();
+       }        
+
+     ntr=ntr->getNextSibling();
       
     } // end for
-    if(icnt >0)
-       (void)run_dbl(etr,icnt);      
 
+    if(icnt >0){
+       iret=0;
+       (void)run_dbl(etr,icnt);      
+    }
       
-exit: ;     
+exit: return iret;     
             
 
     } // end run_exe
 
-
-#line 33 "ncoTree.hpp"
+public:
+RefAST nco_dupList(RefAST tr){
+      RefAST otr;  
+      // nb astFactory is protected- must call from within class
+      otr=astFactory->dupList(tr);      
+      otr->setNextSibling( ANTLR_USE_NAMESPACE(antlr)nullAST ) ;
+      /*  
+      if(otr->getNextSibling()!= ANTLR_USE_NAMESPACE(antlr)ASTNULL )     
+        err_prn("nco_dupList", "NON NULL AST SIBLING\n");
+      */ 
+      return otr; 
+     }
+#line 55 "ncoTree.hpp"
 public:
 	ncoTree();
 	static void initializeASTFactory( ANTLR_USE_NAMESPACE(antlr)ASTFactory& factory );
@@ -372,15 +435,24 @@ public:
 		return ncoTree::tokenNames;
 	}
 	public: int  lmt_peek(ANTLR_USE_NAMESPACE(antlr)RefAST _t);
-	public: void statements(ANTLR_USE_NAMESPACE(antlr)RefAST _t);
+	public: int  statements(ANTLR_USE_NAMESPACE(antlr)RefAST _t);
+	public: var_sct * out(ANTLR_USE_NAMESPACE(antlr)RefAST _t);
 	public: var_sct * assign_ntl(ANTLR_USE_NAMESPACE(antlr)RefAST _t,
 		bool bram
 	);
 	public: var_sct * assign(ANTLR_USE_NAMESPACE(antlr)RefAST _t,
 		bool bram
 	);
-	public: var_sct * out(ANTLR_USE_NAMESPACE(antlr)RefAST _t);
 	public: var_sct * out_asn(ANTLR_USE_NAMESPACE(antlr)RefAST _t);
+	public: var_sct * value_list(ANTLR_USE_NAMESPACE(antlr)RefAST _t);
+	public: bool  where_assign(ANTLR_USE_NAMESPACE(antlr)RefAST _t,
+		var_sct *var_msk
+	);
+	public: var_sct * var_lmt_one(ANTLR_USE_NAMESPACE(antlr)RefAST _t);
+	public: var_sct * var_lmt_one_lhs(ANTLR_USE_NAMESPACE(antlr)RefAST _t,
+		bool bram
+	);
+	public: var_sct * var_lmt(ANTLR_USE_NAMESPACE(antlr)RefAST _t);
 public:
 	ANTLR_USE_NAMESPACE(antlr)RefAST getAST()
 	{
@@ -393,10 +465,10 @@ protected:
 private:
 	static const char* tokenNames[];
 #ifndef NO_STATIC_CONSTS
-	static const int NUM_TOKENS = 103;
+	static const int NUM_TOKENS = 113;
 #else
 	enum {
-		NUM_TOKENS = 103
+		NUM_TOKENS = 113
 	};
 #endif
 	
