@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/libnco.h,v 1.37 2007/02/23 21:59:27 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/libnco.h,v 1.44 2009/05/21 15:38:08 zender Exp $ */
 
-/* Purpose: netCDxF Operator (NCO) library */
+/* Purpose: netCDF Operator (NCO) library */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* libnco.h headers depend on netcdf.h and nco_netcdf.h headers */
@@ -20,10 +20,12 @@
 
 /* Library headers */
 #include "nco_att_utl.h" /* Attribute utilities */
+#include "nco_aux.h" /* Auxiliary coordinates */
 #include "nco_bnr.h" /* Binary write utilities */
 #include "nco_cln_utl.h" /* Calendar utilities */
 #include "nco_cnf_dmn.h" /* Conform dimensions */
 #include "nco_cnf_typ.h" /* Conform variable types */
+#include "nco_cnk.h" /* Chunking */
 #include "nco_cnv_arm.h" /* ARM conventions */
 #include "nco_cnv_csm.h" /* CCM/CCSM/CF conventions */
 #include "nco_ctl.h" /* Program flow control functions */
