@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncap.h,v 1.79 2007/02/23 21:59:27 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncap.h,v 1.87 2009/05/02 22:22:30 zender Exp $ */
 
 /* Purpose: netCDF arithmetic processor definitions and function prototypes for ncap.c, ncap_utl.c, ncap_lex.l, and ncap_yacc.y */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -95,6 +95,8 @@ typedef struct{ /* prs_sct */
   var_sct ***var_lst; /* list of variables to be defined in O */
   int *nbr_var;        /* [nbr] of variables in above list */ 
   int nco_op_typ; /* [enm] Operation type */
+  int dfl_lvl; /* [enm] Deflate level */
+  size_t *cnk_sz; /* [nbr] Chunk sizes */
 } prs_sct;
 
 /* Begin funtions in ncap_lex.l */
