@@ -1,12 +1,12 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncatted.c,v 1.107 2007/05/25 17:36:46 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncatted.c,v 1.123 2009/04/19 23:17:04 zender Exp $ */
 
 /* ncatted -- netCDF attribute editor */
 
 /* Purpose: Add, create, delete, or overwrite attributes in a netCDF file */
 
-/* Copyright (C) 1995--2007 Charlie Zender
+/* Copyright (C) 1995--2009 Charlie Zender
 
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   License: GNU General Public License (GPL) Version 3
    The full license text is at http://www.gnu.org/copyleft/gpl.html 
    and in the file nco/doc/LICENSE in the NCO source distribution.
    
@@ -21,8 +21,8 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
    See the GNU General Public License for more details.
    
-   The original author of this software, Charlie Zender, wants to improve it
-   with the help of your suggestions, improvements, bug-reports, and patches.
+   The original author of this software, Charlie Zender, seeks to improve
+   it with your suggestions, contributions, bug-reports, and patches.
    Please contact the NCO project at http://nco.sf.net or write to
    Charlie Zender
    Department of Earth System Science
@@ -144,8 +144,8 @@ main(int argc,char **argv)
   char *fl_pth_lcl=NULL; /* Option l */
   char *opt_crr=NULL; /* [sng] String representation of current long-option name */
 
-  const char * const CVS_Id="$Id: ncatted.c,v 1.107 2007/05/25 17:36:46 zender Exp $"; 
-  const char * const CVS_Revision="$Revision: 1.107 $";
+  const char * const CVS_Id="$Id: ncatted.c,v 1.123 2009/04/19 23:17:04 zender Exp $"; 
+  const char * const CVS_Revision="$Revision: 1.123 $";
   const char * const opt_sht_lst="Aa:D:hl:Oo:p:Rr-:";
   
 #if defined(__cplusplus) || defined(PGI_CC)
@@ -179,6 +179,8 @@ main(int argc,char **argv)
       {"drt",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
       {"dirty",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
       {"mmr_drt",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
+      {"version",no_argument,0,0},
+      {"vrs",no_argument,0,0},
       /* Long options with argument, no short option counterpart */
       {"hdr_pad",required_argument,0,0},
       {"header_pad",required_argument,0,0},
@@ -198,8 +200,6 @@ main(int argc,char **argv)
       {"path",required_argument,0,'p'},
       {"retain",no_argument,0,'R'},
       {"rtn",no_argument,0,'R'},
-      {"version",no_argument,0,'r'},
-      {"vrs",no_argument,0,'r'},
       {"help",no_argument,0,'?'},
       {0,0,0,0}
     }; /* end opt_lng */
@@ -227,6 +227,10 @@ main(int argc,char **argv)
       if(!strcmp(opt_crr,"cln") || !strcmp(opt_crr,"mmr_cln") || !strcmp(opt_crr,"clean")) flg_cln=True; /* [flg] Clean memory prior to exit */
       if(!strcmp(opt_crr,"drt") || !strcmp(opt_crr,"mmr_drt") || !strcmp(opt_crr,"dirty")) flg_cln=False; /* [flg] Clean memory prior to exit */
       if(!strcmp(opt_crr,"hdr_pad") || !strcmp(opt_crr,"header_pad")) hdr_pad=strtoul(optarg,(char **)NULL,10);
+      if(!strcmp(opt_crr,"vrs") || !strcmp(opt_crr,"version")){
+	(void)nco_vrs_prn(CVS_Id,CVS_Revision);
+	nco_exit(EXIT_SUCCESS);
+      } /* endif "vrs" */
     } /* opt != 0 */
     /* Process short options */
     switch(opt){
@@ -261,8 +265,10 @@ main(int argc,char **argv)
       REMOVE_REMOTE_FILES_AFTER_PROCESSING=!REMOVE_REMOTE_FILES_AFTER_PROCESSING;
       break;
     case 'r': /* Print CVS program information and copyright notice */
-      (void)copyright_prn(CVS_Id,CVS_Revision);
+      (void)nco_vrs_prn(CVS_Id,CVS_Revision);
       (void)nco_lbr_vrs_prn();
+      (void)nco_cpy_prn();
+      (void)nco_cnf_prn();
       nco_exit(EXIT_SUCCESS);
       break;
     case '?': /* Print proper usage */
@@ -290,6 +296,8 @@ main(int argc,char **argv)
     nco_exit(EXIT_FAILURE);
   } /* end if */ 
 
+  if(dbg_lvl_get() >= nco_dbg_var) (void)fprintf(stdout,"%s: DEBUG attribute assumed to hold missing data is named \"%s\"\n",prg_nm_get(),nco_mss_val_sng_get());
+
   /* Make uniform list of user-specified attribute edit structures */
   if(nbr_aed > 0) aed_lst=nco_prs_aed_lst(nbr_aed,aed_arg);
 
@@ -312,6 +320,7 @@ main(int argc,char **argv)
       if(rcd_lcl != -1){
         char usr_reply;
         
+	/* fxm TODO nco199: Internationalize (i18n) NCO with gettext() */
         usr_reply='z';
         while(usr_reply != 'n' && usr_reply != 'y'){
           (void)fprintf(stdout,"%s: overwrite %s (y/n)? ",prg_nm_get(),fl_out);
@@ -343,23 +352,34 @@ main(int argc,char **argv)
   ddra_info.tmr_flg=nco_tmr_rgl;
 
   for(idx=0;idx<nbr_aed;idx++){
-
-    if(aed_lst[idx].var_nm != NULL){
-
-      /* Is this a global attribute? */
-      if(!strcmp(aed_lst[idx].var_nm,"global")) aed_lst[idx].id=NC_GLOBAL; else (void)nco_inq_varid(nc_id,aed_lst[idx].var_nm,&aed_lst[idx].id);
-
-      /* Edit attribute */
-      (void)nco_aed_prc(nc_id,aed_lst[idx].id,aed_lst[idx]);
-
-    }else{ /* var_nm == NULL */
-      /* Perform operation for every variable for which it makes sense */
-      
-      /* Edit attribute for every variable */
+    /* Edit attribute for every variable */
+    if(aed_lst[idx].var_nm == NULL){
       for(idx_var=0;idx_var<nbr_var_fl;idx_var++) 
 	(void)nco_aed_prc(nc_id,idx_var,aed_lst[idx]);
+    } 
+    /* regular expression */       
+    else if(strpbrk(aed_lst[idx].var_nm,".*^$\\[]()<>+?|{}")){
+        int nbr_xtr=1;
+        nm_id_sct *xtr_lst=NULL;
 
-    } /* end else var_nm == NULL */
+        xtr_lst=nco_var_lst_mk(nc_id,nbr_var_fl,&aed_lst[idx].var_nm,False,False,&nbr_xtr);
+        /* edit list of attributes */
+        for(idx_var=0 ; idx_var<nbr_xtr ; idx_var++)
+          (void)nco_aed_prc(nc_id,xtr_lst[idx_var].id,aed_lst[idx]);
+
+          /* Free Extraction list  */
+        xtr_lst=nco_nm_id_lst_free(xtr_lst,nbr_xtr);
+    }
+    /* global variable */
+    else if(!strcmp(aed_lst[idx].var_nm,"global")) {
+       /* Edit Global attribute */
+      (void)nco_aed_prc(nc_id,NC_GLOBAL,aed_lst[idx]);
+    }else{ 
+        /* regular variable */
+        (void)nco_inq_varid(nc_id,aed_lst[idx].var_nm,&aed_lst[idx].id);
+        /* Edit attribute */
+        (void)nco_aed_prc(nc_id,aed_lst[idx].id,aed_lst[idx]);
+    }
 
   } /* end loop over idx */
   
