@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco.h,v 1.116 2007/05/22 18:26:54 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco.h,v 1.142 2009/07/12 19:42:53 zender Exp $ */
 
 /* Purpose: netCDF Operator (NCO) definitions */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -39,7 +39,6 @@
 #endif /* !__cplusplus */
 
 #ifdef __cplusplus
-
 /* Use C-bindings so C++-compiled and C-compiled libraries are compatible */
 extern "C" {
 #endif /* !__cplusplus */
@@ -108,15 +107,41 @@ extern "C" {
 #define size_t_CEWI 0UL
   
   /* netcdf.h NC_GLOBAL is, strictly, the variable ID for global attributes
-     NCO_REC_DMN_UNDEFINED is variable ID of record dimension iff record dimension is undefined
+     NCO_REC_DMN_UNDEFINED is dimension ID of record dimension iff record dimension is undefined
      Normally using -1 for this ID is fine, but token makes meaning clearer
-     NB: nc_inq() family is defined to return -1 for missing record dimensions */
+     NB: nc_inq() family returns -1 for missing record dimensions */
 #define NCO_REC_DMN_UNDEFINED -1
+
+  /* netcdf.h defines four NC_FORMAT tokens: NC_FORMAT_CLASSIC, ...
+     The values are (currently) enumerated from one to four
+     Operators need to check if fl_out_fmt has been user-specified
+     Saftest way is to compare current value of fl_out_fmt to initial value 
+     Initial value should be a number that will never be a true netCDF format */
+#define NCO_FORMAT_UNDEFINED 0
   
   /* Debugging level that quiets all non-requested informational messages
      This value is compared against user-selected dbg_lvl 
      Running operators with --quiet automatically sets dbg_lvl=NCO_DBG_QUIET */
-#define NCO_DBG_QUIET 6
+#define NCO_DBG_QUIET 0
+
+  /* NCO_MSS_VAL_SNG names attribute whose value is "skipped" by arithmetic
+     Must be either "missing_value" or "_FillValue" */
+#ifndef NCO_MSS_VAL_SNG
+  /* 20070831: TKN2SNG technique inserts quotes into string though same test code in c.c does not produce extra quotes. TODO nco905. */
+  /*# define NCO_MSS_VAL_SNG missing_value */
+  /*# define NCO_MSS_VAL_SNG _FillValue*/
+  /*char nco_mss_val_sng[]=TKN2SNG(NCO_MSS_VAL_SNG);*/ /* [sng] Missing value attribute name */
+# define NCO_USE_FILL_VALUE
+# ifdef NCO_USE_FILL_VALUE
+  /* NCO_NETCDF4_AND_FILLVALUE tells whether netCDF4 restrictions on 
+     _FillValue operations (must be defined before variable written,
+     cannot be changed after variable written) can affect output file */ 
+#  ifdef ENABLE_NETCDF4
+#   define NCO_NETCDF4_AND_FILLVALUE
+#  endif /* !ENABLE_NETCDF4 */
+# endif /* !NCO_USE_FILL_VALUE */
+# define NCO_MSS_VAL_SNG
+#endif /* NCO_MSS_VAL_SNG */
 
   /* Prototype global functions before defining them in next block */
   char *nco_mss_val_sng_get(void); /* [sng] Missing value attribute name */
@@ -136,16 +161,12 @@ extern "C" {
   
   unsigned short dbg_lvl=0; /* [enm] Debugging level */
   unsigned short dbg_lvl_get(void){return dbg_lvl;} /* [enm] Debugging level */
-  
-  /* NCO_MSS_VAL_SNG contains the name of the attribute whose value is treated as
-     "missing" during arithmetic. Choose either missing_value of _FillValue. */
-#ifndef NCO_MSS_VAL_SNG
-# define NCO_MSS_VAL_SNG missing_value
-  /* # define NCO_MSS_VAL_SNG _FillValue */
-#endif /* NCO_MSS_VAL_SNG */
 
-  /* Unidata standard string for missing values */
-  char nco_mss_val_sng[]=TKN2SNG(NCO_MSS_VAL_SNG); /* [sng] Missing value attribute name */
+# ifdef NCO_USE_FILL_VALUE
+  char nco_mss_val_sng[]="_FillValue"; /* [sng] Missing value attribute name */
+# else /* !NCO_USE_FILL_VALUE */
+  char nco_mss_val_sng[]="missing_value"; /* [sng] Missing value attribute name */
+# endif /* !NCO_USE_FILL_VALUE */
   char *nco_mss_val_sng_get(void){return nco_mss_val_sng;} /* [sng] Missing value attribute name */
 
 #else /* MAIN_PROGRAM_FILE is NOT defined, i.e., current file does not contain main() */
@@ -154,28 +175,41 @@ extern "C" {
      Variables with local file scope in all files except main.c are allocated here */
   
 #endif /* MAIN_PROGRAM_FILE is NOT defined, i.e., the current file does not contain main() */
-  
-  /* Define compatibility tokens when user does not have netCDF4 */
-#ifndef ENABLE_NETCDF4
-/* Single compatibility token new to netCDF4 netcdf.h */
+
+/* Compatibility tokens for when NCO compiled with older netcdf.h
+   It is hard to track where/when many tokens defined
+   Easiest to individually check for pre-definition of each */
+/* Three compatibility tokens new to netCDF4 netcdf.h: */
+#ifndef NC_NETCDF4
 # define NC_NETCDF4     (0x1000) /* Use netCDF-4/HDF5 format */
-/* Four compatibility tokens from netCDF3.6 netcdf.h */
-# ifndef NC_CLASSIC_MODEL
-#  define NC_CLASSIC_MODEL (0x0008) /* Enforce strict netcdf-3 rules. */
-# endif
-# ifndef NC_FORMAT_CLASSIC
-#  define NC_FORMAT_CLASSIC (1)
-# endif
-# ifndef NC_FORMAT_64BIT
-#  define NC_FORMAT_64BIT   (2)
-# endif
-# ifndef NC_FORMAT_NETCDF4
-#  define NC_FORMAT_NETCDF4 (3)
-# endif
-# ifndef NC_FORMAT_NETCDF4_CLASSIC
-#  define NC_FORMAT_NETCDF4_CLASSIC  (4) /* create netcdf-4 files, with NC_CLASSIC_MODEL. */
-# endif
-#endif /* !ENABLE_NETCDF4 */
+#endif
+#ifndef NC_CHUNKED
+# define NC_CHUNKED (0)
+#endif
+#ifndef NC_CONTIGUOUS
+# define NC_CONTIGUOUS (1)
+#endif
+/* Six compatibility tokens not all available until netCDF 3.6.1 netcdf.h
+   NC_64BIT_OFFSET is used (so far) only in nco_fl_utl.c */
+#ifndef NC_64BIT_OFFSET
+# define NC_64BIT_OFFSET (0x0200) /* Use large (64-bit) file offsets */
+#endif
+#ifndef NC_CLASSIC_MODEL
+# define NC_CLASSIC_MODEL (0x0008) /* Enforce strict netcdf-3 rules. */
+#endif
+#ifndef NC_FORMAT_CLASSIC
+# define NC_FORMAT_CLASSIC (1)
+#endif
+#ifndef NC_FORMAT_64BIT
+# define NC_FORMAT_64BIT   (2)
+#endif
+#ifndef NC_FORMAT_NETCDF4
+# define NC_FORMAT_NETCDF4 (3)
+#endif
+#ifndef NC_FORMAT_NETCDF4_CLASSIC
+# define NC_FORMAT_NETCDF4_CLASSIC  (4) /* create netcdf-4 files, with NC_CLASSIC_MODEL. */
+#endif
+/* Endif older netcdf.h */
 
   /* Define compatibility tokens when user does not have znetCDF */
 #ifndef ENABLE_ZNETCDF
@@ -216,18 +250,18 @@ extern "C" {
   
   enum nco_dbg_typ_enm{ /* [enm] Debugging levels */
     /* List in increasing levels of verbosity */
-    nco_dbg_quiet, /* Quiet all non-error messages. */
-    nco_dbg_std, /* Standard mode. Minimal, but some, messages. */
-    nco_dbg_fl, /* Filenames */
-    nco_dbg_scl, /* Scalars, other per-file information  */
-    nco_dbg_var, /* Variables, highest level per-file loop information */
-    nco_dbg_crr, /* Current task */
-    nco_dbg_sbr, /* Subroutine names on entry and exit */
-    nco_dbg_io, /* Subroutine I/O */
-    nco_dbg_vec, /* Entire vectors */
-    nco_dbg_vrb, /* Verbose, print everything possible */
-    nco_dbg_old, /* Old debugging blocks not used anymore */
-    nco_dbg_nbr /* Number of debugging types. Should be last enumerated value. */
+    nco_dbg_quiet, /* 0 [enm] Quiet all non-error messages */
+    nco_dbg_std, /* 1 [enm] Standard mode. Minimal, but some, messages */
+    nco_dbg_fl, /* 2 [enm] Filenames */
+    nco_dbg_scl, /* 3 [enm] Scalars, other per-file information  */
+    nco_dbg_var, /* 4 [enm] Variables, highest level per-file loop information */
+    nco_dbg_crr, /* 5 [enm] Current task */
+    nco_dbg_sbr, /* 6 [enm] Subroutine names on entry and exit */
+    nco_dbg_io, /* 7 [enm] Subroutine I/O */
+    nco_dbg_vec, /* 8 [enm] Entire vectors */
+    nco_dbg_vrb, /* 9 [enm] Verbose, print everything possible */
+    nco_dbg_old, /* 10 [enm] Old debugging blocks not used anymore */
+    nco_dbg_nbr /* 10 [enm] Number of debugging types (equals last enumerated value) */
   }; /* end nco_dbg_typ_enm */
 
   typedef enum { /* [enm] Memory allocation type */
@@ -318,6 +352,8 @@ extern "C" {
     long end; /* Index to end of hyperslab */
     long cnt; /* # of valid elements in this dimension (including effects of stride and wrapping) */
     long srd; /* Stride of hyperslab */
+    double origin;   /* used for ncra,ncrcat to time rebase co-ordinate var */
+    char *re_bs_sng; /* used for ncra,ncrcat to time rebase co-ordinate -holds unit attribue from first file */
   } lmt_sct;
 
   /* Container holding all limit structures indexible by dimension */
@@ -328,6 +364,7 @@ extern "C" {
     int lmt_dmn_nbr; /* [nbr] Number of lmt args */
     nco_bool BASIC_DMN; /* [flg] Limit is same as dimension in input file */
     nco_bool WRP; /* [flg] Limit is wrapped (true iff wrapping, lmt_dmn_nbr==2) */ 
+    nco_bool MSA_USR_RDR; /* [flg] Multi-slabbing algorithm leaves hyperslabs in user order */
     lmt_sct **lmt_dmn; /* [sct] List of limit structures associated with each dimension */  
   } lmt_all_sct;
   
@@ -421,6 +458,16 @@ extern "C" {
     ptr_unn val;
   } att_sct;
   
+  /* Chunking structure */
+  typedef struct{ /* cnk_sct */
+    char *nm; /* [sng] Dimension name */
+    size_t sz; /* [nbr] Chunk size */
+    nco_bool is_usr_spc_cnk; /* [flg] Chunk size was user-specified */
+  } cnk_sct;
+  
+  /* Fill actual value of dmn_sct structure in in nco_dmn_fll()
+     free() each pointer member of dmn_sct structure in nco_dmn_free()
+     deep-copy each pointer member of dmn_sct structure in nco_dmn_dpl() */
   /* Dimension structure */
   typedef struct dmn_sct_tag{ /* dmn_sct */
     char *nm; /* [sng] Dimension name */
@@ -436,11 +483,13 @@ extern "C" {
     long end; /* [idx] Index to end of hyperslab */
     long cnt; /* [nbr] Number of valid elements in this dimension (including effects of stride and wrapping) */
     long srd; /* [nbr] Stride of hyperslab */
+    size_t cnk_sz; /* [nbr] Chunk size */
     ptr_unn val; /* [sct] Buffer to hold hyperslab fxm: is this ever used? */
     struct dmn_sct_tag *xrf; /* [sct] Cross-reference to associated dimension structure (usually the structure for dimension on output) */
   } dmn_sct; /* end dmn_sct_tag */
   
   /* Initialize default value of each member of var_sct structure in var_dfl_set()
+     Fill actual value of var_sct structure in nco_var_fll()
      free() each pointer member of var_sct structure in nco_var_free()
      deep-copy each pointer member of var_sct structure in nco_var_dpl() */
   /* Variable structure */
@@ -463,6 +512,7 @@ extern "C" {
     char fmt[5]; /* [sng] Hint for printf()-style formatting */
     dmn_sct **dim; /* [sct] Pointers to full dimension structures */
     int *dmn_id; /* [id] Contiguous vector of dimension IDs */
+    size_t *cnk_sz; /* [id] Contiguous vector of chunk sizes */
     long *srt; /* [id] Contiguous vector of indices to start of hyperslab */
     long *end; /* [id] Contiguous vector of indices to end of hyperslab */
     long *cnt; /* [id] Contiguous vector of lengths of hyperslab */
