@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.c,v 1.76 2007/05/25 17:36:46 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.c,v 1.97 2009/05/25 20:48:59 zender Exp $ */
 
 /* Purpose: Attribute utilities */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_att_utl.h" /* Attribute utilities */
@@ -17,19 +17,24 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
   /* Purpose: Process single attribute edit for single variable */
   
   /* If var_id == NC_GLOBAL ( = -1) then global attribute will be edited */
-  
+
+#ifdef NCO_NETCDF4_AND_FILLVALUE
+  char att_nm_tmp[]="eulaVlliF_"; /* String of same length as "_FillValue" for name hack with netCDF4 */
+  nco_bool flg_netCDF4=False; /* [flg] File format is netCDF4 */
+#endif /* !NCO_NETCDF4_AND_FILLVALUE */
+
   char att_nm[NC_MAX_NAME];
   char var_nm[NC_MAX_NAME];
   
-  /* fxm: netCDF 2 specifies att_sz should be type int, netCDF 3 uses size_t */
+  /* fxm: netCDF2 specifies att_sz should be type int, netCDF3 uses size_t */
   int nbr_att; /* [nbr] Number of attributes */
   int rcd=NC_NOERR; /* [rcd] Return code */
   long att_sz;
-  
+     
   nc_type att_typ;
   
   void *att_val_new=NULL;
-  
+
   if(var_id == NC_GLOBAL){
     /* Get number of global attributes in file */
     (void)nco_inq(nc_id,(int *)NULL,(int *)NULL,&nbr_att,(int *)NULL);
@@ -44,19 +49,19 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
 
   /* Before changing metadata, change missing values to new missing value if warranted 
      This capability is add-on feature not implemented too cleanly or efficiently
-     If every variable has "missing_value" attribute and "missing_value" is changed
+     If every variable has "_FillValue" attribute and "_FillValue" is changed
      globally, then algorithm goes into and out of define mode for each variable,
      rather than collecting all information in first pass and replacing all data in second pass.
      This is because ncatted was originally designed to change only metadata and so was
      architected differently from other NCO operators. */
   if(
      aed.att_nm /* Linux strcmp() dumps core if attribute name is blank */
-     && strcmp(aed.att_nm,nco_mss_val_sng_get()) == 0 /* Current attribute is "missing_value" */
+     && strcmp(aed.att_nm,nco_mss_val_sng_get()) == 0 /* Current attribute is "_FillValue" */
      && var_id != NC_GLOBAL /* Current attribute is not global */
      && (aed.mode == aed_modify || aed.mode == aed_overwrite)  /* Modifying or overwriting existing value */
-     && rcd == NC_NOERR /* Only when existing missing_value attribute is modified */
-     && att_sz == 1L /* Old missing_value attribute must be of size 1 */
-     && aed.sz == 1L /* New missing_value attribute must be of size 1 */
+     && rcd == NC_NOERR /* Only when existing _FillValue attribute is modified */
+     && att_sz == 1L /* Old _FillValue attribute must be of size 1 */
+     && aed.sz == 1L /* New _FillValue attribute must be of size 1 */
      ){
 
     int *dmn_id;
@@ -114,7 +119,7 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
 
     /* Sanity check */
     if(var->has_mss_val == False){
-      (void)fprintf(stdout,"%s: ERROR \"NCO_MSS_VAL_SNG\" attribute does not exist in nco_aed_prc()\n",prg_nm_get());
+      (void)fprintf(stdout,"%s: ERROR variable %s does not have \"%s\" attribute in nco_aed_prc()\n",prg_nm_get(),var_nm,nco_mss_val_sng_get());
       nco_exit(EXIT_FAILURE);
     } /* end if */
 
@@ -177,7 +182,27 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
     (void)nco_redef(nc_id);
   } /* end if replacing missing value data */
 
-  /* Change metadata (as written, this must be done after missing_value data is replaced) */
+  /* Change metadata (as written, this must be done after _FillValue data is replaced) */
+
+#ifdef NCO_NETCDF4_AND_FILLVALUE
+  /* Bold hack which gets around problem of modifying netCDF4 "_FillValue" attributes
+     netCDF4 does not allow this by default, though netCDF3 does
+     Change attribute name to att_nm_tmp, modify value, then restore name */
+
+  /* Check if file is netCDF3 classic with netCDF4 library
+     If so, do not kludge. NB: create global variable for output file format? */
+  { /* scope for fl_fmt temporary */
+    int fl_fmt; 
+    (void)nco_inq_format(nc_id,&fl_fmt);
+    flg_netCDF4=(fl_fmt==NC_FORMAT_NETCDF4 || fl_fmt==NC_FORMAT_NETCDF4_CLASSIC);
+  } /* end scope */
+
+  if(flg_netCDF4 && !strcmp(aed.att_nm,nco_mss_val_sng_get()) && aed.mode != aed_delete){
+    if(aed.mode != aed_create) (void)nco_rename_att(nc_id,var_id,aed.att_nm,att_nm_tmp);
+    strcpy(aed.att_nm,att_nm_tmp); 
+  } /* endif libnetCDF may have netCDF4 restrictions */
+#endif /* !NCO_NETCDF4_AND_FILLVALUE */
+
   switch(aed.mode){
   case aed_append:	
     if(rcd == NC_NOERR){
@@ -225,7 +250,15 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
   default: 
     break;
   } /* end switch */
-  
+
+#ifdef NCO_NETCDF4_AND_FILLVALUE
+    if(flg_netCDF4 && !strcmp(aed.att_nm,att_nm_tmp) && aed.mode != aed_delete ){
+      (void)nco_rename_att(nc_id,var_id,att_nm_tmp,nco_mss_val_sng_get());
+      /* Restore orginal name (space already allocated)*/
+     strcpy(aed.att_nm,nco_mss_val_sng_get()); 
+    } /* !flg_netCDF4 */
+#endif /* !NCO_NETCDF4_AND_FILLVALUE */
+
 } /* end nco_aed_prc() */
 
 void 
@@ -239,7 +272,7 @@ nco_att_cpy  /* [fnc] Copy attributes from input netCDF file to output netCDF fi
   /* Purpose: Copy attributes from input netCDF file to output netCDF file
      If var_in_id == NC_GLOBAL, then copy global attributes
      Otherwise copy only indicated variable's attributes
-     If PCK_ATT_CPY is false, then copy all attributes except "scale_factor", "add_offset" */
+     When PCK_ATT_CPY is false, copy all attributes except "scale_factor", "add_offset" */
 
   char att_nm[NC_MAX_NAME];
   char var_nm[NC_MAX_NAME];
@@ -279,12 +312,12 @@ nco_att_cpy  /* [fnc] Copy attributes from input netCDF file to output netCDF fi
       } /* end if */
     } /* end if dbg */
 
-    if(PCK_ATT_CPY || strcmp(att_nm,nco_mss_val_sng_get())){
-      /* Copy all attributes except missing_value with fast library routine */
+    if(strcmp(att_nm,nco_mss_val_sng_get())){
+      /* Copy all attributes except _FillValue with fast library routine */
       (void)nco_copy_att(in_id,var_in_id,att_nm,out_id,var_out_id);
     }else{
-      /* Convert "missing_value" attribute to unpacked type then copy 
-	 This imposes NCO convention that missing_value is same type as variable,
+      /* Convert "_FillValue" attribute to unpacked type then copy 
+	 Impose NCO convention that _FillValue is same type as variable,
 	 whether variable is packed or not */
       aed_sct aed;
       
@@ -303,7 +336,7 @@ nco_att_cpy  /* [fnc] Copy attributes from input netCDF file to output netCDF fi
 	nco_exit(EXIT_FAILURE); 
       } /* end if */
       
-      /* Convert "missing_value" to unpacked type before copying */
+      /* Convert "_FillValue" to unpacked type before copying */
       aed.att_nm=att_nm; /* Name of attribute */
       if(var_out_id == NC_GLOBAL){
 	aed.var_nm=NULL;
@@ -313,19 +346,32 @@ nco_att_cpy  /* [fnc] Copy attributes from input netCDF file to output netCDF fi
       } /* end if */
       aed.id=out_id; /* Variable ID or NC_GLOBAL ( = -1) for global attribute */
       aed.sz=att_sz; /* Number of elements in attribute */
-      (void)nco_inq_vartype(out_id,var_out_id,&att_typ_out);
-      aed.type=att_typ_out; /* Type of attribute */
-      aed.val.vp=(void *)nco_malloc(nco_typ_lng(aed.type)); /* Pointer to attribute value */
-      att_lng_in=att_sz*nco_typ_lng(att_typ_in);
-      mss_tmp.vp=(void *)nco_malloc(att_lng_in);
-      (void)nco_get_att(in_id,var_in_id,att_nm,mss_tmp.vp,att_typ_in);
-      (void)nco_val_cnf_typ(att_typ_in,mss_tmp,att_typ_out,aed.val);
-      aed.mode=aed_overwrite; /* Action to perform with attribute */
+
+      /* Do not convert global attributes or PCK_ATT_CPY */  
+      if(PCK_ATT_CPY || var_out_id==NC_GLOBAL) att_typ_out=att_typ_in; else (void)nco_inq_vartype(out_id,var_out_id,&att_typ_out);
+
+      if(att_typ_out==att_typ_in){
+        aed.type=att_typ_out; /* Type of attribute */
+        aed.val.vp=(void *)nco_malloc(nco_typ_lng(aed.type)); /* Pointer to attribute value */
+        (void)nco_get_att(in_id,var_in_id,att_nm,aed.val.vp,att_typ_out);
+      }else{ /* att_typ_out!=att_typ_in */
+	/* Convert type */          
+        aed.type=att_typ_out; /* Type of attribute */
+        aed.val.vp=(void *)nco_malloc(nco_typ_lng(aed.type)); /* Pointer to attribute value */
+        att_lng_in=att_sz*nco_typ_lng(att_typ_in);
+        mss_tmp.vp=(void *)nco_malloc(att_lng_in);
+        (void)nco_get_att(in_id,var_in_id,att_nm,mss_tmp.vp,att_typ_in);
+        (void)nco_val_cnf_typ(att_typ_in,mss_tmp,att_typ_out,aed.val);
+        mss_tmp.vp=nco_free(mss_tmp.vp);
+      } /* att_typ_out!=att_typ_in */
+
+      /* Overwrite mode causes problems with netCDF4 and "_FillValue" 
+	 Use create mode instead */
+      aed.mode=aed_create;
       (void)nco_aed_prc(out_id,var_out_id,aed); 
       /* Release temporary memory */
       aed.val.vp=nco_free(aed.val.vp);
-      mss_tmp.vp=nco_free(mss_tmp.vp);
-    } /* PCK_ATT_CPY */
+    } /* endif copying _FillValue */
 
   } /* end loop over attributes */
 } /* end nco_att_cpy() */
@@ -594,20 +640,20 @@ nco_prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure lis
       /* Set type of current aed structure */
       /* Convert single letter code to type enum */
       switch(*(arg_lst[3])){
-      case 'f':	aed_lst[idx].type=NC_FLOAT; break;
-      case 'd':	aed_lst[idx].type=NC_DOUBLE; break;
+      case 'f':	aed_lst[idx].type=(nc_type)NC_FLOAT; break;
+      case 'd':	aed_lst[idx].type=(nc_type)NC_DOUBLE; break;
       case 'l':	
-      case 'i':	aed_lst[idx].type=NC_INT; break;
-      case 's':	aed_lst[idx].type=NC_SHORT; break;
-      case 'c':	aed_lst[idx].type=NC_CHAR; break;
-      case 'b':	aed_lst[idx].type=NC_BYTE; break;
+      case 'i':	aed_lst[idx].type=(nc_type)NC_INT; break;
+      case 's':	aed_lst[idx].type=(nc_type)NC_SHORT; break;
+      case 'c':	aed_lst[idx].type=(nc_type)NC_CHAR; break;
+      case 'b':	aed_lst[idx].type=(nc_type)NC_BYTE; break;
       default: 
-	if(!strcasecmp(arg_lst[3],"ub")) aed_lst[idx].type=NC_UBYTE; 
-	else if(!strcasecmp(arg_lst[3],"us")) aed_lst[idx].type=NC_USHORT; 
-	else if(!strcasecmp(arg_lst[3],"u") || !strcasecmp(arg_lst[3],"ui") || !strcasecmp(arg_lst[3],"ul")) aed_lst[idx].type=NC_UINT; 
-	else if(!strcasecmp(arg_lst[3],"ll") || !strcasecmp(arg_lst[3],"int64")) aed_lst[idx].type=NC_INT64; 
-	else if(!strcasecmp(arg_lst[3],"ull") || !strcasecmp(arg_lst[3],"uint64")) aed_lst[idx].type=NC_UINT64; 
-	else if(!strcasecmp(arg_lst[3],"sng")) aed_lst[idx].type=NC_STRING; 
+	if(!strcasecmp(arg_lst[3],"ub")) aed_lst[idx].type=(nc_type)NC_UBYTE; 
+	else if(!strcasecmp(arg_lst[3],"us")) aed_lst[idx].type=(nc_type)NC_USHORT; 
+	else if(!strcasecmp(arg_lst[3],"u") || !strcasecmp(arg_lst[3],"ui") || !strcasecmp(arg_lst[3],"ul")) aed_lst[idx].type=(nc_type)NC_UINT; 
+	else if(!strcasecmp(arg_lst[3],"ll") || !strcasecmp(arg_lst[3],"int64")) aed_lst[idx].type=(nc_type)NC_INT64; 
+	else if(!strcasecmp(arg_lst[3],"ull") || !strcasecmp(arg_lst[3],"uint64")) aed_lst[idx].type=(nc_type)NC_UINT64; 
+	else if(!strcasecmp(arg_lst[3],"sng")) aed_lst[idx].type=(nc_type)NC_STRING; 
 	else{
 	  (void)fprintf(stderr,"%s: ERROR `%s' is not a supported netCDF data type\n",prg_nm_get(),arg_lst[3]);
 	  (void)fprintf(stderr,"%s: HINT: Valid data types are `c' = char, `f' = float, `d' = double,`s' = short, 'l' = `i' = integer, `b' = byte",prg_nm_get());
@@ -825,6 +871,37 @@ nco_prs_rnm_lst /* [fnc] Set old_nm, new_nm elements of rename structure */
   return rnm_lst;
 } /* end nco_prs_rnm_lst() */
 
+void 
+nco_vrs_att_cat /* [fnc] Add NCO version global attribute */
+(const int out_id) /* I [id] netCDF output-file ID */
+{
+  /* Purpose: Write NCO version information to global metadata */
+  aed_sct vrs_sng_aed;
+  char att_nm[]="NCO";
+  char *vrs_cvs; /* [sng] Version according to CVS release tag */
+  char *vrs_sng; /* [sng] NCO version */
+  ptr_unn att_val;
+  
+  vrs_cvs=cvs_vrs_prs();
+  vrs_sng=vrs_cvs;
+
+  /* Insert thread number into value */
+  att_val.cp=(char *)strdup(vrs_sng);
+  /* Initialize nco_openmp_thread_number attribute edit structure */
+  vrs_sng_aed.att_nm=att_nm;
+  vrs_sng_aed.var_nm=NULL;
+  vrs_sng_aed.id=NC_GLOBAL;
+  vrs_sng_aed.sz=strlen(vrs_sng)+1L;
+  vrs_sng_aed.type=NC_CHAR;
+  /* Insert value into attribute structure */
+  vrs_sng_aed.val=att_val;
+  vrs_sng_aed.mode=aed_overwrite;
+  /* Write nco_openmp_thread_number attribute to disk */
+  (void)nco_aed_prc(out_id,NC_GLOBAL,vrs_sng_aed);
+  vrs_sng=(char *)nco_free(vrs_sng);
+
+} /* end nco_vrs_att_cat() */
+
 void 
 nco_thr_att_cat /* [fnc] Add threading global attribute */
 (const int out_id, /* I [id] netCDF output-file ID */
