@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.h,v 1.25 2007/02/23 21:59:28 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.h,v 1.31 2009/05/25 20:48:59 zender Exp $ */
 
 /* Purpose: Attribute utilities */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -32,6 +32,7 @@
 #include "nco_lst_utl.h" /* List utilities */
 #include "nco_mmr.h" /* Memory management */
 #include "nco_mss_val.h" /* Missing value utilities */
+#include "nco_scm.h" /* Software configuration management */
 #include "nco_sng_utl.h" /* String utilities */
 
 #ifdef __cplusplus
@@ -88,6 +89,10 @@ nco_thr_att_cat /* [fnc] Add threading global attribute */
 (const int out_id, /* I [id] netCDF output-file ID */
  const int thr_nbr); /* I [nbr] Thread number */
 
+void 
+nco_vrs_att_cat /* [fnc] Add NCO version global attribute */
+(const int out_id); /* I [id] netCDF output-file ID */
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif /* __cplusplus */
