@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.c,v 1.44 2007/05/15 23:51:59 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.c,v 1.51 2009/05/26 10:41:24 hmb Exp $ */
 
 /* Purpose: Conform variable types */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_cnf_typ.h" /* Conform variable types */
@@ -910,22 +910,145 @@ nco_scv_cnf_typ /* [fnc] Convert scalar attribute to typ_new using C implicit co
   return True;
 } /* end nco_scv_cnf_typ */
 
+nco_bool /* O [flg] Input is signed type */
+nco_typ_sgn /* [fnc] Identify signed types */
+(nc_type typ_in) /* I [enm] Type to check for signedness */
+{
+  nco_bool flg_sgn=True; /* CEWI */
+
+  switch(typ_in){
+  case NC_FLOAT: 
+  case NC_DOUBLE: 
+  case NC_INT64: 
+  case NC_INT: 
+  case NC_SHORT: 
+  case NC_BYTE: 
+  case NC_CHAR: 
+  case NC_NAT: 
+  case NC_STRING: 
+    flg_sgn=True;
+    break;       
+  case NC_UBYTE: 
+  case NC_USHORT:
+  case NC_UINT:
+  case NC_UINT64:
+    flg_sgn=False;
+    break;
+  default: nco_dfl_case_nc_type_err(); break;
+  } /* end switch */
+  return flg_sgn;
+} /* end nco_typ_sgn */
+
 nc_type /* O [enm] Highest precision of input variables */
 ncap_var_retype /* [fnc] Promote variable to higher common precision */
 (var_sct *var_1, /* I/O [sct] Variable */
  var_sct *var_2) /* I/O [sct] Variable */
 {
-  /* fxm: TODO nco616: netCDF4 breaks assumption that range/precision increases with nc_type enum */
-  /* Purpose: Convert variable, if necessary, so variables are of same type */
-  if(var_1->type == var_2->type) return var_1->type;
-  /* fxm: Unsafe assumption that netCDF types are enumerated in increasing order of precision */
-  if(var_1->type > var_2->type){
-    var_2=nco_var_cnf_typ(var_1->type,var_2);
-    return var_1->type;
-  }else{
-    var_1=nco_var_cnf_typ(var_2->type,var_1);
-    return var_2->type;
-  } /* endif */
+  /* Threads: Routine is thread safe and makes no unsafe routines */
+  /* Purpose: Perform intelligent type conversion with the netCDF3/4 types 
+     The logic is as follows
+     1) If one type is double then other is converted to double .. end
+     2) if one type float then other converted to float         .. end
+     3) if both are signed type then the "lower" type is converted 
+        to the higher type
+     4) if both are unsigned type then the "lower" type is converted 
+        to the higher type
+     5) First var signed second var unsigned
+        if the signed type can represent all values af the unsigned type 
+        Then the second var is converted to the first vars type .. end
+
+        If the types are the same then the signed var is converted to the
+        unsigned type 
+
+        If the signed type is less than the unsigned type then the signed type is 
+        converted to unsigned type
+
+        Examples:
+
+        NC_CHAR,NC_UBYTE -> result NC_UBYTE
+        NC_INT, NC_USHORT -> result NC_INT
+        NC_SHORT, NC_UINT -> result NC_UINT  
+        NC_INT64, NC_UINT -> result NC_INT64
+        NC_INT64,NC_UINT64  -> result NC_UINT64 */
+ 
+  nco_bool v1s;
+  nco_bool v2s;
+  nc_type typ_1;
+  nc_type typ_2;
+
+  typ_1=var_1->type;
+  typ_2=var_2->type;
+ 
+  /* already identical */
+  if(typ_1 == typ_2)
+    return typ_1;
+
+  /* deal with NC_DOUBLE */
+  if(typ_1 == NC_DOUBLE || typ_2 == NC_DOUBLE ){
+    var_1=nco_var_cnf_typ(NC_DOUBLE,var_1);
+    var_2=nco_var_cnf_typ(NC_DOUBLE,var_2);
+    return NC_DOUBLE;
+  }
+
+  /* deal with NC_FLOAT */
+  if(typ_1 == NC_FLOAT || typ_2 ==NC_FLOAT ){
+    var_1=nco_var_cnf_typ(NC_FLOAT,var_1);
+    var_2=nco_var_cnf_typ(NC_FLOAT,var_2);
+    return NC_FLOAT;
+  }
+
+  v1s=nco_typ_sgn(typ_1);
+  v2s=nco_typ_sgn(typ_2);
+
+  /* Both signed or both unsigned */
+  /* convert to the highest type */
+  if( v1s==v2s  ){
+    if(typ_1 > typ_2)
+      var_2=nco_var_cnf_typ(typ_1,var_2);
+     else  
+      var_1=nco_var_cnf_typ(typ_2,var_1); 
+
+    return typ_1;
+  }
+  /* From here on one is unsigned the other signed */
+  
+  /* Swap vars about so var_1 is signed, var_2 unsigned */
+  if(v1s == False && v2s == True){
+    var_sct *var_tmp;
+    var_tmp=var_1;var_1=var_2;var_2=var_tmp;
+    
+    /* refresh types */
+    typ_1=var_1->type;
+    typ_2=var_2->type;  
+  }
+
+  switch(typ_1){
+  case NC_BYTE: 
+  case NC_CHAR:
+    var_1=nco_var_cnf_typ(typ_2,var_1); 
+    break;
+  case NC_SHORT: 
+    if( typ_2 < NC_USHORT )  
+      var_2=nco_var_cnf_typ(typ_1,var_2);
+    else   
+      var_1=nco_var_cnf_typ(typ_2,var_1); 
+    break;
+  case NC_INT:
+    if( typ_2 < NC_UINT )  
+      var_2=nco_var_cnf_typ(typ_1,var_2);
+    else   
+      var_1=nco_var_cnf_typ(typ_2,var_1); 
+    break;
+  case NC_INT64:
+    if( typ_2 < NC_UINT64 )  
+      var_2=nco_var_cnf_typ(typ_1,var_2);
+    else   
+      var_1=nco_var_cnf_typ(typ_2,var_1); 
+    break; 
+  default: nco_dfl_case_nc_type_err(); break;
+    
+  } /* end switch */
+  return typ_1;
 } /* end ncap_var_retype */
 
 nc_type /* O [enm] Highest precision of arguments */
@@ -953,7 +1076,7 @@ ncap_var_scv_cnf_typ_hgh_prc /* [fnc] Promote arguments to higher precision if n
 {
   /* fxm: TODO nco616: netCDF4 breaks assumption that range/precision increases with nc_type enum */
   /* Purpose: If types of variable and scalar value differ, convert argument with 
-     lower precision to type of argument with higher precision.
+     lower precision to higher precision type.
      Otherwise do nothing. 
      fxm: Assumes nc_type increases monotonically with precision */
 
