@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.h,v 1.24 2007/02/23 21:59:28 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.h,v 1.29 2009/05/22 16:22:51 zender Exp $ */
 
 /* Purpose: Conform variable types */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -85,6 +85,10 @@ ncap_var_scv_cnf_typ_hgh_prc /* [fnc] Promote arguments to higher precision if n
 (var_sct ** const var, /* I/O [sct] Variable */
  scv_sct * const scv); /* I/O [sct] Scalar value */
 
+nco_bool /* O [flg] Input is signed type */
+nco_typ_sgn /* [fnc] Identify signed types */
+(nc_type typ_in); /* I [enm] Type to check for signedness */
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif /* __cplusplus */
