@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnv_arm.h,v 1.15 2007/02/23 21:59:28 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnv_arm.h,v 1.23 2009/05/29 20:12:36 zender Exp $ */
 
 /* Purpose: ARM conventions */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -41,7 +41,8 @@ nco_cnv_arm_time_mk /* [fnc] Return time corresponding to current time offset */
 void
 nco_cnv_arm_time_install /* [fnc] Add time variable to concatenated ARM files */
 (const int nc_id, /* I [id] netCDF file ID */
- const nco_int base_time_srt); /* I [s] base_time of first input file */
+ const nco_int base_time_srt, /* I [s] base_time of first input file */
+ const int dfl_lvl); /* I [enm] Deflate level [0..9] */
 
 nco_int /* O [s] Value of base_time variable */
 nco_cnv_arm_base_time_get /* [fnc] Get base_time variable from ARM file */
