@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnv_csm.c,v 1.36 2007/04/18 17:06:15 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnv_csm.c,v 1.42 2009/04/19 23:17:04 zender Exp $ */
 
 /* Purpose: CCM/CCSM/CF conventions */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_cnv_csm.h" /* CCM/CCSM/CF conventions */
@@ -183,7 +183,9 @@ nco_cnv_cf_crd_add /* [fnc] Add coordinates defined by CF convention */
 	/* NUL-terminate attribute */
 	att_val[att_sz]='\0';
 	/* Split list into separate coordinate names */
-	crd_lst=nco_lst_prs_2D(att_val,dlm_sng,&nbr_crd);
+        /* using nco_lst_prs_sgl_2D() and not nco_lst_prs_2D */
+        /* see TODO 944   */
+        crd_lst=nco_lst_prs_sgl_2D(att_val,dlm_sng,&nbr_crd);
 	/* ...for each coordinate in "coordinates" attribute... */
 	for(idx_crd=0;idx_crd<nbr_crd;idx_crd++){
 	  /* Verify "coordinate" exists in input file */
