@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_ctl.h,v 1.26 2007/02/23 21:59:28 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_ctl.h,v 1.32 2009/04/19 23:17:04 zender Exp $ */
 
 /* Purpose: Program flow control functions */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -33,6 +33,7 @@
 /* Personal headers */
 #include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_mmr.h" /* Memory management */
+#include "nco_sng_utl.h" /* String utilities */
 
 #ifdef __cplusplus
 extern "C" {
@@ -65,20 +66,23 @@ nco_ddra /* [fnc] Count operations */
 void 
 nco_dfl_case_tmr_typ_err(void); /* [fnc] Print error and exit for illegal switch(tmr_typ) case */
 
+void
+nco_cnf_prn(void); /* [fnc] Print NCO configuration and help text */
+
 void
 nco_lbr_vrs_prn(void); /* [fnc] Print netCDF library version */
 
 const char * /* O [sng] MPI implementation */
 nco_mpi_get(void); /* [fnc] Return MPI implementation */
 
+const char * /* O [sng] Mnemonic that describes current NCO version */
+nco_nmn_get(void); /* [fnc] Return mnemonic that describes current NCO version */
+
 char * /* O [sng] nm_in stripped of any path (i.e., program name stub) */ 
 prg_prs /* [fnc] Strip program name to stub and return program ID */
 (const char * const nm_in, /* I [sng] Name of program, i.e., argv[0] (may include path prefix) */
  int * const prg_lcl); /* O [enm] Enumerated number corresponding to nm_in */
 
-const char * /* O [sng] Mnemonic that describes current NCO version */
-nco_nmn_get(void); /* [fnc] Return mnemonic that describes current NCO version */
-
 void 
 nco_usg_prn(void); /* [fnc] Print correct program usage */
 
