@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.c,v 1.27 2007/02/23 21:59:28 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.c,v 1.34 2009/05/25 18:12:45 zender Exp $ */
 
 /* Purpose: Dimension utilities */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_dmn_utl.h" /* Dimension utilities */
@@ -98,6 +98,7 @@ nco_dmn_fll /* [fnc] Create and return completed dmn_sct */
     (void)nco_inq_vartype(dmn->nc_id,dmn->cid,&dmn->type);
   } /* end if */
   
+  dmn->cnk_sz=0L;
   dmn->cnt=dmn->sz;
   dmn->srt=0L;
   dmn->end=dmn->sz-1L;
@@ -119,6 +120,29 @@ nco_dmn_free /* [fnc] Free all memory associated with dimension structure */
   return NULL;
 } /* end nco_dmn_free() */
 
+void
+nco_dmn_cnk_mrg /* [fnc] Merge chunking information into dimension structures */
+(dmn_sct ** const dmn, /* I [sct] Dimension structures to modify */
+ const int nbr_dmn, /* I [nbr] Number of dimension structures in structure list */
+ CST_X_PTR_CST_PTR_CST_Y(cnk_sct,cnk), /* I [sct] Chunking information */
+ const int cnk_nbr) /* I [nbr] Number of dimensions with user-specified chunking */
+{
+  /* Purpose: Merge chunking information into dimension structures */
+  int dmn_idx;
+  int cnk_idx;
+
+  for(dmn_idx=0;dmn_idx<nbr_dmn;dmn_idx++){
+    /* Does this dimension have user-specified chunking? */
+    for(cnk_idx=0;cnk_idx<cnk_nbr;cnk_idx++){
+      /* Match on name not ID so nco_dmn_cnk_mrg() works on any input file */
+      if(!strcmp(cnk[cnk_idx]->nm,dmn[dmn_idx]->nm)){
+	dmn[dmn_idx]->cnk_sz=cnk[cnk_idx]->sz;
+	break;
+      } /* end if */
+    } /* end loop over cnk_idx */
+  } /* end loop over dmn */
+} /* end nco_dmn_cnk_mrg() */
+
 void
 nco_dmn_lmt_mrg /* [fnc] Merge limit structure information into dimension structures */
 (dmn_sct ** const dmn, /* I [sct] Dimension structures to modify */
@@ -128,19 +152,19 @@ nco_dmn_lmt_mrg /* [fnc] Merge limit structure information into dimension struct
 {
   /* Purpose: Merge limit structure information into dimension structures */
 
-  int idx;
+  int dmn_idx;
   int lmt_idx;
 
-  for(idx=0;idx<nbr_dmn;idx++){
+  for(dmn_idx=0;dmn_idx<nbr_dmn;dmn_idx++){
     /* Does this dimension have user-specified limits? */
     for(lmt_idx=0;lmt_idx<lmt_nbr;lmt_idx++){
       /* 20050707: Match on name not ID so nco_dmn_lmt_mrg() works with single
 	 limit list applied to any input file */
-      if(!strcmp(lmt[lmt_idx]->nm,dmn[idx]->nm)){
-	dmn[idx]->cnt=lmt[lmt_idx]->cnt;
-	dmn[idx]->srt=lmt[lmt_idx]->srt;
-	dmn[idx]->end=lmt[lmt_idx]->end;
-	dmn[idx]->srd=lmt[lmt_idx]->srd;
+      if(!strcmp(lmt[lmt_idx]->nm,dmn[dmn_idx]->nm)){
+	dmn[dmn_idx]->cnt=lmt[lmt_idx]->cnt;
+	dmn[dmn_idx]->srt=lmt[lmt_idx]->srt;
+	dmn[dmn_idx]->end=lmt[lmt_idx]->end;
+	dmn[dmn_idx]->srd=lmt[lmt_idx]->srd;
 	break;
       } /* end if */
     } /* end loop over lmt_idx */
