@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_fl_utl.c,v 1.88 2007/05/15 05:48:49 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_fl_utl.c,v 1.114 2009/05/05 05:34:31 zender Exp $ */
 
 /* Purpose: File manipulation */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_fl_utl.h" /* File manipulation */
@@ -44,8 +44,8 @@ nco_create_mode_mrg /* [fnc] Merge clobber mode with user-specified file format
 
 int /* [rcd] Return code */
 nco_create_mode_prs /* [fnc] Parse user-specified file format */
-(const char * const fl_fmt_sng, /* [sng] User-specified file format string */
- int * const fl_fmt_enm) /* I [enm] Output file format */
+(const char * const fl_fmt_sng, /* I [sng] User-specified file format string */
+ int * const fl_fmt_enm) /* O [enm] Output file format */
 {
   int rcd=NC_NOERR; /* [rcd] Return code */
 
@@ -99,11 +99,11 @@ nco_fl_cp /* [fnc] Copy first file to second */
   const char cp_cmd_fmt[]="cp %s %s";
 
   int rcd;
-  const int nbr_fmt_char=4;
+  const int fmt_chr_nbr=4;
 
   /* Construct and execute copy command */
-  cp_cmd=(char *)nco_malloc((strlen(cp_cmd_fmt)+strlen(fl_src)+strlen(fl_dst)-nbr_fmt_char+1UL)*sizeof(char));
-  if(dbg_lvl_get() > 0) (void)fprintf(stderr,"Copying %s to %s...",fl_src,fl_dst);
+  cp_cmd=(char *)nco_malloc((strlen(cp_cmd_fmt)+strlen(fl_src)+strlen(fl_dst)-fmt_chr_nbr+1UL)*sizeof(char));
+  if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"Copying %s to %s...",fl_src,fl_dst);
   (void)sprintf(cp_cmd,cp_cmd_fmt,fl_src,fl_dst);
   rcd=system(cp_cmd);
   if(rcd == -1){
@@ -111,7 +111,7 @@ nco_fl_cp /* [fnc] Copy first file to second */
     nco_exit(EXIT_FAILURE);
   } /* end if */
   cp_cmd=(char *)nco_free(cp_cmd);
-  if(dbg_lvl_get() > 0) (void)fprintf(stderr,"done\n");
+  if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"done\n");
 } /* end nco_fl_cp() */
 
 char * /* O [sng] Canonical file name*/
@@ -317,7 +317,7 @@ nco_fl_lst_mk /* [fnc] Create file list from command line positional arguments *
 	} /* endif err */
 
 	if(dbg_lvl_get() > 2) (void)fprintf(stderr,"%s: DEBUG Read %d filenames in %li characters from stdin\n",prg_nm_get(),*fl_nbr,(long)fl_lst_in_lng);
-	if(*fl_nbr > 0) *FL_LST_IN_FROM_STDIN=True; else (void)fprintf(stderr,"%s: WARNING Tried but failed to get input filenames from stdin\n",prg_nm_get());
+	if(*fl_nbr > 0) *FL_LST_IN_FROM_STDIN=True; else (void)fprintf(stderr,"%s: WARNING Tried and failed to get input filenames from stdin\n",prg_nm_get());
 
       } /* endif multi-file operator without positional arguments for fl_in */
 
@@ -364,15 +364,22 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
      return name of file on local system */
 
   FILE *fp_in;
-  nco_bool FTP_URL=False;
-  nco_bool FTP_NETRC=False;
-  nco_bool SFTP_URL=False;
-  nco_bool FTP_OR_SFTP_URL;
+  nco_bool DAP_URL=False; /* DAP handles netCDF API, no retrieval necessary */
+  nco_bool FTP_URL=False; /* Retrieve remote file via FTP */
+  nco_bool FTP_NETRC=False; /* Retrieve remote file via FTP with .netrc file */
+  nco_bool FTP_OR_SFTP_URL; /* FTP or SFTP */
+  nco_bool HTTP_URL=False; /* Retrieve remote file via wget */
+  nco_bool SFTP_URL=False; /* Retrieve remote file via SFTP */
+  nco_bool PTH_LCL_DRV=False; /* Local path was derived (not user-specified) */
   char *cln_ptr; /* [ptr] Colon pointer */
   char *fl_nm_lcl;
+  char *fl_nm_lcl_tmp;
   char *fl_nm_stub;
+  char *fl_pth_lcl_tmp=NULL;
+
   const char fnc_nm[]="nco_fl_mk_lcl"; /* [sng] Function name */
   const char ftp_url_sng[]="ftp://";
+  const char http_url_sng[]="http://";
   const char sftp_url_sng[]="sftp://";
   int rcd;
   size_t url_sng_lng=0L; /* CEWI */
@@ -393,23 +400,79 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
   } /* !ftp */
   FTP_OR_SFTP_URL=FTP_URL || SFTP_URL;
 
-//printf("\n\nprg_id=%d and ncatted=%d!\n", prg_id, ncatted);
-
   if(FTP_OR_SFTP_URL){
-    char *fl_nm_lcl_tmp;
-    char *fl_pth_lcl_tmp;
-
-    /* If FTP rearrange fl_nm_lcl to get rid of ftp://hostname part, and
-       if SFTP rearrange fl_nm_lcl to get rid of sftp://hostname: part,
+    /* If FTP rearrange fl_nm_lcl to remove ftp://hostname part, and
+       if SFTP rearrange fl_nm_lcl to remove sftp://hostname: part,
        before searching for file on local disk */
     fl_pth_lcl_tmp=strchr(fl_nm_lcl+url_sng_lng,'/');
     fl_nm_lcl_tmp=fl_nm_lcl;
     fl_nm_lcl=(char *)nco_malloc(strlen(fl_pth_lcl_tmp)+1UL);
     (void)strcpy(fl_nm_lcl,fl_pth_lcl_tmp);
     fl_nm_lcl_tmp=(char *)nco_free(fl_nm_lcl_tmp);
-  }else if(strstr(fl_nm_lcl,"http://") == fl_nm_lcl){
+  }else if(strstr(fl_nm_lcl,http_url_sng) == fl_nm_lcl){
+    /* Filename starts with "http://": Try DAP first (if available), then wget. */
 
-    /* If file indicates http protocol then pass unadulterated file name and let DAP deal with it */
+#ifdef ENABLE_DAP
+    /* Filename has http:// prefix so try DAP access to unadulterated filename */
+    int in_id; /* [id] Temporary input file ID */
+
+    /* Attempt nc_open() on HTTP protocol files. Success means DAP found file. */
+    rcd=nco_open_flg(fl_nm_lcl,NC_NOWRITE,&in_id);
+    
+    if(rcd == NC_NOERR){
+      /* Fail gracefully if ncatted or ncrename try to use DAP */
+      switch(prg_id){
+      case ncatted:
+      case ncrename:
+	(void)fprintf(stderr,"%s: ERROR nco_fl_mk_lcl() reminds you that ncatted and ncrename must process truly local (i.e., not read via DAP) files (fxm TODO nco664)\n",prg_nm_get());
+	nco_exit(EXIT_FAILURE);
+	break;
+      default:
+	/* All other operators work with DAP correctly */
+	break;
+      } /* end switch */
+      
+      /* Close file to prevent accumulating dangling open files on DAP server */
+      rcd=nco_close(in_id);
+
+      /* Great! DAP worked so file has earned DAP identification */
+      DAP_URL=True; 
+
+      /* Set rcd=0 to agree with successful stat() so rest of function treats file as local
+	 (DAP treats HTTP protocol files as local files) */
+      rcd=0;
+
+    }else{ /* DAP-access failed */
+      (void)fprintf(stderr,"%s: INFO DAP access to %s failed: Server does not respond, file does not exist, or user does not have read permission\n",prg_nm_get(),fl_nm_lcl);
+    } /* DAP-access failed */
+
+#else /* !ENABLE_DAP */
+    (void)fprintf(stderr,"%s: INFO Access to %s may (because filename starts with \"http://\") require DAP, but NCO was not enabled with DAP. Instead NCO will try to search for the file locally, and then will try wget.\n",prg_nm_get(),fl_nm_lcl); 
+    (void)fprintf(stderr,"%s: HINT: Obtain or build DAP-enabled NCO, e.g., with configure --enable-dap-netcdf ...\n",prg_nm_get());
+#endif /* !ENABLE_DAP */
+
+    if(DAP_URL == False){ /* DAP access to http:// file failed */
+      /* Attempt to retrieve URLs directly when DAP access fails. Tests:
+	 ncks -D 2 -M http://dust.ess.uci.edu/nco/in.nc # wget
+	 ncks -D 2 -M -l . http://dust.ess.uci.edu/nco/in.nc # wget
+	 ncks -D 2 -M -l . -p http://dust.ess.uci.edu/nco in.nc # wget
+	 ncks -D 2 -M -p http://dust.ess.uci.edu/cgi-bin/dods/nph-dods/dodsdata in.nc # DAP */
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"%s: INFO Will first attempt to find on local disk and, if unsuccessful, will then attempt retrieve remote file to local client using wget\n",prg_nm_get());
+      
+      /* DAP cannot open file so leave DAP_URL=FALSE and set HTTP_URL=True
+	 Later we will attempt to wget file to local system */
+      HTTP_URL=True;
+      url_sng_lng=strlen(http_url_sng);
+
+      /* If HTTP then rearrange fl_nm_lcl to remove http://hostname part
+	 before searching for file on local disk */
+      fl_pth_lcl_tmp=strchr(fl_nm_lcl+url_sng_lng,'/');
+      fl_nm_lcl_tmp=fl_nm_lcl;
+      fl_nm_lcl=(char *)nco_malloc(strlen(fl_pth_lcl_tmp)+1UL);
+      (void)strcpy(fl_nm_lcl,fl_pth_lcl_tmp);
+      fl_nm_lcl_tmp=(char *)nco_free(fl_nm_lcl_tmp);
+
+    } /* !DAP_URL, HTTP_URL */
 
   }else if((cln_ptr=strchr(fl_nm_lcl,':'))){
     /* 19990804
@@ -421,10 +484,7 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
        it must be preceded by period within three or four spaces, e.g., uci.edu: */
     if(((cln_ptr-4 >= fl_nm_lcl) && *(cln_ptr-4) == '.') ||
        ((cln_ptr-3 >= fl_nm_lcl) && *(cln_ptr-3) == '.')){
-      char *fl_nm_lcl_tmp;
-      char *fl_pth_lcl_tmp;
-
-      /* Rearrange fl_nm_lcl to get rid of hostname: part */
+      /* Rearrange fl_nm_lcl to remove hostname: part */
       fl_pth_lcl_tmp=strchr(fl_nm_lcl+url_sng_lng,'/');
       fl_nm_lcl_tmp=fl_nm_lcl;
       fl_nm_lcl=(char *)nco_malloc(strlen(fl_pth_lcl_tmp)+1UL);
@@ -434,23 +494,19 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
   } /* end if */
 
   /* Does file exist on local system? */
-  rcd=stat(fl_nm_lcl,&stat_sct);
-
-  /* One exception: let DAP try to access remote HTTP protocol files as local files */
-  if(strstr(fl_nm_lcl,"http://") == fl_nm_lcl) rcd=0;
+  if(!DAP_URL) rcd=stat(fl_nm_lcl,&stat_sct);
+  if(rcd == -1 && (dbg_lvl_get() >= nco_dbg_fl)) (void)fprintf(stderr,"%s: INFO stat() #1 failed: %s does not exist\n",prg_nm_get(),fl_nm_lcl);
 
   /* If not, check if file exists on local system under same path interpreted relative to current working directory */
   if(rcd == -1){
     if(fl_nm_lcl[0] == '/'){
       rcd=stat(fl_nm_lcl+1UL,&stat_sct);
+      if(rcd == -1 && (dbg_lvl_get() >= nco_dbg_fl)) (void)fprintf(stderr,"%s: INFO stat() #2 failed: %s does not exist\n",prg_nm_get(),fl_nm_lcl+1UL);
     } /* end if */
     if(rcd == 0){
-      char *fl_nm_lcl_tmp;
-
-      /* NB: simply adding one to filename pointer is like deleting
-	 the initial slash on the filename. Without copying the new name
-	 into its own memory space, free(fl_nm_lcl) would not be able to free
-	 the initial byte. */
+      /* NB: Adding one to filename pointer is like deleting initial slash on filename
+	 Then free(fl_nm_lcl) would miss this initial byte (memory is lost)
+	 Hence must copy new name into its own memory space */
       fl_nm_lcl_tmp=(char *)strdup(fl_nm_lcl+1UL);
       fl_nm_lcl=(char *)nco_free(fl_nm_lcl);
       fl_nm_lcl=fl_nm_lcl_tmp;
@@ -458,47 +514,46 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
     } /* end if */
   } /* end if */
 
-  /* Finally, check to see if file exists on local system in directory specified for storage of remotely retrieved files
-     This would be the case if some files had already been retrieved in a previous invocation of the program */
+  /* Finally, check if file exists locally in directory for remotely retrieved files
+     This occurs when previous program invocations have already retrieved some files */
   if(rcd == -1){
-    /* Where does filename stub begin? NB: Assume local filename has a slash (because remote file system always has a slash) */
+    /* Where does filename stub begin? 
+       NB: Assume local filename has a slash (because remote file system always has a slash) */
     fl_nm_stub=strrchr(fl_nm_lcl,'/')+1UL;
 
-    /* Construct local filename from user-supplied local file path along with existing file stub */
+    /* Construct local filename from user-supplied local file path and existing file stub */
     if(fl_pth_lcl != NULL){
-      char *fl_nm_lcl_tmp;
-
       fl_nm_lcl_tmp=fl_nm_lcl;
       /* Allocate enough room for the joining slash '/' and the terminating NUL */
       fl_nm_lcl=(char *)nco_malloc((strlen(fl_pth_lcl)+strlen(fl_nm_stub)+2)*sizeof(char));
       (void)strcpy(fl_nm_lcl,fl_pth_lcl);
       (void)strcat(fl_nm_lcl,"/");
       (void)strcat(fl_nm_lcl,fl_nm_stub);
-      /* Free the old filename space */
+      /* Free old filename space */
       fl_nm_lcl_tmp=(char *)nco_free(fl_nm_lcl_tmp);
     } /* end if */
 
-    /* At last, check for file in the local storage directory */
+    /* At last, check for file in local storage directory */
     rcd=stat(fl_nm_lcl,&stat_sct);
-    if (rcd != -1) (void)fprintf(stderr,"%s: WARNING not searching for %s on remote filesystem, using local file %s instead\n",prg_nm_get(),fl_nm,fl_nm_lcl);
+    if(rcd != -1) (void)fprintf(stderr,"%s: WARNING not searching for %s on remote filesystem, using local file %s instead\n",prg_nm_get(),fl_nm,fl_nm_lcl);
+    if(rcd == -1 && (dbg_lvl_get() >= nco_dbg_fl)) (void)fprintf(stderr,"%s: INFO stat() #3 failed: %s does not exist\n",prg_nm_get(),fl_nm_lcl);
   } /* end if */
 
-  /* File was not found locally, try to fetch it from remote file system */
+  /* File was not found locally and is not DAP-accessible, try to fetch file from remote filesystem */
   if(rcd == -1){
 
     typedef struct{ /* [enm] Remote fetch command structure */
       const char *fmt; /* [] Format */
-      const int nbr_fmt_char; /* [nbr] Number of formatting characters */
+      const int fmt_chr_nbr; /* [nbr] Number of formatting characters */
       const int transfer_mode; /* [enm] Transfer mode */
       const int file_order; /* [enm] File order */
     } rmt_fch_cmd_sct;
 
     char *cmd_sys;
     char *fl_nm_rmt;
-    char *fl_pth_lcl_tmp=NULL;
     char *fmt_ftp=NULL; /* [sng] Declare outside FTP block scope for easier freeing */
 
-    /* fxm: autoconf this */
+    /* fxm: use autoconf HAVE_MKDIR_M rather than SUN4 token TODO nco293 */
 #ifndef SUN4
     const char cmd_mkdir[]="mkdir -m 777 -p";
 #else /* SUN4 */
@@ -513,7 +568,7 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
       lcl_rmt, /* Local file argument before remote file argument */
       rmt_lcl}; /* Remote file argument before local file argument */
 
-    size_t fl_pth_lcl_lng;
+    size_t fl_pth_lcl_lng=size_t_CEWI; /* CEWI */
 
     rmt_fch_cmd_sct *rmt_cmd=NULL;
     /* fxm: Initialize structure contents as const */
@@ -521,6 +576,8 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
     rmt_fch_cmd_sct msrcp={"msrcp mss:%s %s",4,synchronous,rmt_lcl};
     rmt_fch_cmd_sct nrnet={"nrnet msget %s r flnm=%s l mail=FAIL",4,asynchronous,lcl_rmt};
     /* rmt_fch_cmd_sct rcp={"rcp -p %s %s",4,synchronous,rmt_lcl};*/
+    /* wget -r: re-download file (clobber existing file of same name, if any) */
+    rmt_fch_cmd_sct http={"wget -r --output-document=%s %s",4,synchronous,lcl_rmt};
     rmt_fch_cmd_sct scp={"scp -p %s %s",4,synchronous,rmt_lcl};
     rmt_fch_cmd_sct sftp={"sftp %s %s",4,synchronous,rmt_lcl};
     /* Fill in ftp structure fmt element dynamically later */
@@ -535,7 +592,7 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
     /* URL specifier in filename unambiguously signals to use FTP */
     if(rmt_cmd == NULL){
       if(FTP_URL){
-	/* fxm: use autoconf HAVE_XXX rather than WIN32 to handle this */
+	/* fxm: use autoconf HAVE_XXX rather than WIN32 token TODO nco292 */
 #ifdef WIN32
 /* #ifndef HAVE_NETWORK fxm */
 	/* I have no idea how networking calls work in MS Windows, so just exit */
@@ -612,7 +669,7 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
 	  host_nm_rmt_psn=strstr(fl_netrc_bfr,host_nm_rmt);
 	  if(host_nm_rmt_psn){
 	    FTP_NETRC=True;
-	    if(dbg_lvl_get() > 0) (void)fprintf(stderr,"%s: INFO %s() will use .netrc file at %s instead of anonymous FTP\n",prg_nm_get(),fnc_nm,fl_nm_netrc);
+	    if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"%s: INFO %s() will use .netrc file at %s instead of anonymous FTP\n",prg_nm_get(),fnc_nm,fl_nm_netrc);
 	  } /* endif host_nm_rmt_psn */
 	  fl_netrc_bfr=(char *)nco_free(fl_netrc_bfr);
 	} /* endif rcd */
@@ -676,10 +733,18 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
       } /* end if SFTP */
     } /* end if rmt_cmd */
 
-    /* Otherwise, single colon preceded by period in filename unambiguously signals to use rcp or scp
+    /* Attempt wget on files that contain http:// prefix and are not accessible via DAP */
+    if(rmt_cmd == NULL){
+      if(HTTP_URL){
+	rmt_cmd=&http;
+	(void)fprintf(stderr,"%s: INFO Will now attempt wget on the full filepath. wget will likely fail if the file is truly behind a DAP server. Unfortunately, even a failed wget attempt creates a rather long pathname in the current directory. fxm TODO nco970, nco971. On the other hand, wget should succeed if the file is stored in any publicly-accessible web location.\n",prg_nm_get());
+      } /* end if HTTP */
+    } /* end if rmt_cmd */
+
+    /* Otherwise, single colon preceded by period in filename signals rcp or scp
        Determining whether to try scp instead of rcp is difficult
        Ideally, NCO would test remote machine for rcp/scp priveleges with system command like, e.g., "ssh echo ok"
-       To start with, we use scp which has its own fall through to rcp */
+       To start we use scp which has its own fall-through to rcp */
     if(rmt_cmd == NULL){
       if((cln_ptr=strchr(fl_nm_rmt,':'))){
 	if(((cln_ptr-4 >= fl_nm_rmt) && *(cln_ptr-4) == '.') ||
@@ -720,23 +785,36 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
     } /* end if */
 
     if(rmt_cmd == NULL){
-      (void)fprintf(stderr,"%s: ERROR unable to determine method for remote retrieval of %s\n",prg_nm_get(),fl_nm_rmt);
+      (void)fprintf(stderr,"%s: ERROR %s neither exists locally nor matches remote filename patterns\n",prg_nm_get(),fl_nm_rmt);
       nco_exit(EXIT_FAILURE);
     } /* end if */
 
-    /* Find path for storing local file */
-    fl_nm_stub=strrchr(fl_nm_lcl,'/')+1UL;
-    /* Construct local storage filepath name */
-    fl_pth_lcl_lng=strlen(fl_nm_lcl)-strlen(fl_nm_stub)-1UL;
-    /* Allocate enough room for terminating NUL */
-    fl_pth_lcl_tmp=(char *)nco_malloc((fl_pth_lcl_lng+1UL)*sizeof(char));
-    (void)strncpy(fl_pth_lcl_tmp,fl_nm_lcl,fl_pth_lcl_lng);
-    fl_pth_lcl_tmp[fl_pth_lcl_lng]='\0';
-
-    /* Warn user when local filepath was machine-derived from remote name */
-    if(fl_pth_lcl == NULL) (void)fprintf(stderr,"%s: WARNING deriving local filepath from remote filename, using %s\n",prg_nm_get(),fl_pth_lcl_tmp);
-
-    /* Does local filepath already exist on local system? */
+    if(fl_pth_lcl == NULL){
+      /* Derive path for storing local file from remote filename */
+      (void)fprintf(stderr,"%s: INFO deriving local filepath from remote filename\n",prg_nm_get());
+      fl_nm_stub=strrchr(fl_nm_lcl,'/')+1UL;
+      if(HTTP_URL){
+	/* Strip leading slash from fl_nm_lcl for HTTP files so, e.g., 
+	   http://dust.ess.uci.edu/nco/in.nc produces local path "nco" not "/nco" */
+	fl_nm_lcl_tmp=(char *)strdup(fl_nm_lcl+1UL);
+	fl_nm_lcl=(char *)nco_free(fl_nm_lcl);
+	fl_nm_lcl=fl_nm_lcl_tmp;
+      } /* !HTTP_URL */
+      /* Construct local storage filepath name */
+      fl_pth_lcl_lng=strlen(fl_nm_lcl)-strlen(fl_nm_stub)-1UL;
+      /* Allocate enough room for terminating NUL */
+      fl_pth_lcl_tmp=(char *)nco_malloc((fl_pth_lcl_lng+1UL)*sizeof(char));
+      (void)strncpy(fl_pth_lcl_tmp,fl_nm_lcl,fl_pth_lcl_lng);
+      fl_pth_lcl_tmp[fl_pth_lcl_lng]='\0';
+      PTH_LCL_DRV=True; /* Local path was derived (not user-specified) */
+      /* Tell user what local filepath was derived */
+      (void)fprintf(stderr,"%s: INFO Retrieved files will be stored in directory ./%s\n",prg_nm_get(),fl_pth_lcl_tmp);
+    }else{
+      /* Copy user-specified local path to unite following code in terms of fl_pth_lcl_tmp */
+      fl_pth_lcl_tmp=(char *)strdup(fl_pth_lcl);
+    } /* fl_pth_lcl */
+    
+    /* Does local filepath exist already on local system? */
     rcd=stat(fl_pth_lcl_tmp,&stat_sct);
     /* If not, then create local filepath */
     if(rcd != 0){
@@ -745,13 +823,13 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
       (void)strcpy(cmd_sys,cmd_mkdir);
       (void)strcat(cmd_sys," ");
       (void)strcat(cmd_sys,fl_pth_lcl_tmp);
-      if(dbg_lvl_get() > 0) (void)fprintf(stderr,"%s: Creating local directory %s with %s\n",prg_nm_get(),fl_pth_lcl_tmp,cmd_sys);
       rcd=system(cmd_sys);
       if(rcd != 0){
 	(void)fprintf(stderr,"%s: ERROR Unable to create local directory %s\n",prg_nm_get(),fl_pth_lcl_tmp);
 	if(fl_pth_lcl == NULL) (void)fprintf(stderr,"%s: HINT Use -l option\n",prg_nm_get());
 	nco_exit(EXIT_FAILURE);
       } /* end if */
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"%s: INFO Created local directory ./%s\n",prg_nm_get(),fl_pth_lcl_tmp);
       /* Free local command space */
       cmd_sys=(char *)nco_free(cmd_sys);
     } /* end if */
@@ -760,13 +838,13 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
     fl_pth_lcl_tmp=(char *)nco_free(fl_pth_lcl_tmp);
 
     /* Allocate enough room for joining space ' ' and terminating NUL */
-    cmd_sys=(char *)nco_malloc((strlen(rmt_cmd->fmt)-rmt_cmd->nbr_fmt_char+strlen(fl_nm_lcl)+strlen(fl_nm_rmt)+2)*sizeof(char));
+    cmd_sys=(char *)nco_malloc((strlen(rmt_cmd->fmt)-rmt_cmd->fmt_chr_nbr+strlen(fl_nm_lcl)+strlen(fl_nm_rmt)+2)*sizeof(char));
     if(rmt_cmd->file_order == lcl_rmt){
       (void)sprintf(cmd_sys,rmt_cmd->fmt,fl_nm_lcl,fl_nm_rmt);
     }else{
       (void)sprintf(cmd_sys,rmt_cmd->fmt,fl_nm_rmt,fl_nm_lcl);
     } /* end else */
-    if(dbg_lvl_get() > 0) (void)fprintf(stderr,"%s: Retrieving file from remote location:\n%s",prg_nm_get(),cmd_sys);
+    if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"%s: Retrieving file from remote location with command:\n%s\n",prg_nm_get(),cmd_sys);
     (void)fflush(stderr);
     /* Fetch file from remote file system */
     rcd=system(cmd_sys);
@@ -777,7 +855,6 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
     if(rmt_cmd == &ftp) fmt_ftp=(char *)nco_free(fmt_ftp);
 
     if(rmt_cmd->transfer_mode == synchronous){
-      if(dbg_lvl_get() > 0) (void)fprintf(stderr,"\n");
       if(rcd != 0){
 	(void)fprintf(stderr,"%s: ERROR Synchronous fetch command failed\n",prg_nm_get());
 	nco_exit(EXIT_FAILURE);
@@ -807,62 +884,44 @@ nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
 	} /* end if */
 	/* Sleep for specified time */
 	(void)sleep((unsigned)tm_sleep_scn);
-	if(dbg_lvl_get() > 0) (void)fprintf(stderr,".");
+	if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,".");
 	(void)fflush(stderr);
       } /* end for */
       if(tm_idx == tm_nbr){
 	(void)fprintf(stderr,"%s: ERROR Maximum time (%d seconds = %.1f minutes) for asynchronous file retrieval exceeded.\n",prg_nm_get(),tm_nbr*tm_sleep_scn,tm_nbr*tm_sleep_scn/60.0);
 	nco_exit(EXIT_FAILURE);
       } /* end if */
-      if(dbg_lvl_get() > 0) (void)fprintf(stderr,"\n%s Retrieval successful after %d sleeps of %d seconds each = %.1f minutes\n",prg_nm_get(),tm_idx,tm_sleep_scn,tm_idx*tm_sleep_scn/60.0);
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"\n%s Retrieval successful after %d sleeps of %d seconds each = %.1f minutes\n",prg_nm_get(),tm_idx,tm_sleep_scn,tm_idx*tm_sleep_scn/60.0);
     } /* end else transfer mode is asynchronous */
     *FILE_RETRIEVED_FROM_REMOTE_LOCATION=True;
   }else{ /* end if input file did not exist locally */
     *FILE_RETRIEVED_FROM_REMOTE_LOCATION=False;
   } /* end if file was already on the local system */
 
-  /* Make sure we have read permission on local file */
-  if(strstr(fl_nm_lcl,"http://") == fl_nm_lcl){
-    /* Attempt nc_open() on HTTP protocol files. Success means DAP found file. */
-    int in_id; /* [id] Temporary input file ID */
+  if(dbg_lvl_get() >= nco_dbg_fl)
+    if(DAP_URL && fl_pth_lcl)
+      (void)fprintf(stderr,"%s: INFO User-specified option \"-l %s\" was not used since input file was not retrieved from remote location\n",prg_nm_get(),fl_pth_lcl);
 
-    /* Following is the graceful failure if ncatted or ncrename try to use DAP to retrieve a file. */
-  switch(prg_id){
-    case ncatted:  case ncrename:
-    (void)fprintf(stderr,"%s: ERROR - in nco_fl_mk_lcl(), ncatted and ncrename cannot currently retrieve entire files via DAP or http.\n",prg_nm_get());
-    nco_exit(EXIT_FAILURE);
-    break;
-    default:
-      /* ALl the others work with DAP correctly */
-    break;
-  }
-
-    rcd=nco_open(fl_nm_lcl,NC_NOWRITE,&in_id);
-
-    if(rcd != NC_NOERR){
-      (void)fprintf(stderr,"%s: ERROR Attempted HTTP access protocol failed: DAP server is not responding, %s does not exist, or user does not have read permission\n",prg_nm_get(),fl_nm_lcl);
-      nco_exit(EXIT_FAILURE);
-    } /* end if err */
-
-  }else{
+  if(!DAP_URL){
+    /* File is (now, anyway) truly local---does local system have read permission? */
     if((fp_in=fopen(fl_nm_lcl,"r")) == NULL){
       (void)fprintf(stderr,"%s: ERROR User does not have read permission for %s, or file does not exist\n",prg_nm_get(),fl_nm_lcl);
       nco_exit(EXIT_FAILURE);
     }else{
       (void)fclose(fp_in);
     } /* end else */
-
-    /* For local files, perform optional file diagnostics */
-    if(dbg_lvl_get() > 0){
+    
+      /* For local files, perform optional file diagnostics */
+    if(dbg_lvl_get() >= nco_dbg_std){
       char *fl_nm_cnc=NULL; /* [sng] Canonical file name */
       /* Determine canonical filename and properties */
       fl_nm_cnc=nco_fl_info_get(fl_nm_lcl);
       if(fl_nm_cnc != NULL) fl_nm_cnc=(char *)nco_free(fl_nm_cnc);
     } /* endif dbg */
-
-  } /* end if really a local file */
-
-  /* Free input filename space */
+    
+  } /* end if file is truly local */
+  
+    /* Free input filename space */
   fl_nm=(char *)nco_free(fl_nm);
 
   /* Return local filename */
@@ -880,11 +939,11 @@ nco_fl_mv /* [fnc] Move first file to second */
   const char mv_cmd_fmt[]="mv -f %s %s";
 
   int rcd;
-  const int nbr_fmt_char=4;
+  const int fmt_chr_nbr=4;
 
   /* Construct and execute copy command */
-  mv_cmd=(char *)nco_malloc((strlen(mv_cmd_fmt)+strlen(fl_src)+strlen(fl_dst)-nbr_fmt_char+1UL)*sizeof(char));
-  if(dbg_lvl_get() > 0) (void)fprintf(stderr,"%s: INFO Moving %s to %s...",prg_nm_get(),fl_src,fl_dst);
+  mv_cmd=(char *)nco_malloc((strlen(mv_cmd_fmt)+strlen(fl_src)+strlen(fl_dst)-fmt_chr_nbr+1UL)*sizeof(char));
+  if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"%s: INFO Moving %s to %s...",prg_nm_get(),fl_src,fl_dst);
   (void)sprintf(mv_cmd,mv_cmd_fmt,fl_src,fl_dst);
   rcd=system(mv_cmd);
   if(rcd == -1){
@@ -892,7 +951,7 @@ nco_fl_mv /* [fnc] Move first file to second */
     nco_exit(EXIT_FAILURE);
   } /* end if */
   mv_cmd=(char *)nco_free(mv_cmd);
-  if(dbg_lvl_get() > 0) (void)fprintf(stderr,"done\n");
+  if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"done\n");
 } /* end nco_fl_mv() */
 
 char * /* O [sng] Name of file to retrieve */
@@ -1047,6 +1106,15 @@ nco_fl_out_open /* [fnc] Open output file subject to availability and user input
 
   struct stat stat_sct;
 
+  /* Make sure output is possible */
+#ifndef ENABLE_NETCDF4
+  if(fl_out_fmt == NC_FORMAT_NETCDF4 || fl_out_fmt == NC_FORMAT_NETCDF4_CLASSIC){
+    (void)fprintf(stdout,"%s: ERROR Requested netCDF4-format output file but NCO was not built with netCDF4 support\n",prg_nm_get());
+    (void)fprintf(stdout,"%s: HINT: Obtain or build a netCDF4-enabled version of NCO.  Try, e.g., ./configure --enable-netcdf4 ...;make;make install\n",prg_nm_get());
+    nco_exit(EXIT_FAILURE);
+  } /* netCDF4 */
+#endif /* ENABLE_NETCDF4 */
+
   /* Set default clobber mode (clobber) then modify for specified file format */
   nccreate_mode=NC_CLOBBER; /* [enm] Mode flag for nco_create() call */
 
@@ -1170,7 +1238,7 @@ nco_fl_out_open /* [fnc] Open output file subject to availability and user input
 	nco_exit(EXIT_FAILURE);
       } /* end if */
       if(nbr_itr > 1) (void)fprintf(stdout,"%s: ERROR Invalid response.\n",prg_nm_get());
-      (void)fprintf(stdout,"%s: %s exists---`e'xit, `o'verwrite (i.e., delete existing file), or `a'ppend (i.e., replace duplicate and add new variables to existing file) (e/o/a)? ",prg_nm_get(),fl_out);
+      (void)fprintf(stdout,"%s: %s exists---`e'xit, `o'verwrite (i.e., delete existing file), or `a'ppend (i.e., replace duplicate variables in and add new variables to existing file) (e/o/a)? ",prg_nm_get(),fl_out);
       (void)fflush(stdout);
       /*       fgets() reads (at most one less than USR_RPL_MAX_LNG) to first newline or EOF */
       rcd_fgets=fgets(usr_rpl,USR_RPL_MAX_LNG,stdin);
@@ -1254,7 +1322,7 @@ nco_fl_rm /* [fnc] Remove file */
   rm_cmd=(char *)nco_malloc((strlen(rm_cmd_sys_dep)+1UL+strlen(fl_nm)+1UL)*sizeof(char));
   (void)sprintf(rm_cmd,"%s %s",rm_cmd_sys_dep,fl_nm);
 
-  if(dbg_lvl_get() > 0) (void)fprintf(stderr,"%s: DEBUG Removing %s with %s\n",prg_nm_get(),fl_nm,rm_cmd);
+  if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"%s: DEBUG Removing %s with %s\n",prg_nm_get(),fl_nm,rm_cmd);
   rcd=system(rm_cmd);
   if(rcd == -1) (void)fprintf(stderr,"%s: WARNING unable to remove %s, continuing anyway...\n",prg_nm_get(),fl_nm);
 
