@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_lmt.c,v 1.57 2007/05/25 06:05:29 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_lmt.c,v 1.82 2009/07/14 12:39:04 hmb Exp $ */
 
 /* Purpose: Hyperslab limits */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_lmt.h" /* Hyperslab limits */
@@ -18,6 +18,7 @@ nco_lmt_free /* [fnc] Free memory associated with limit structure */
   lmt->min_sng=(char *)nco_free(lmt->min_sng);
   lmt->max_sng=(char *)nco_free(lmt->max_sng);
   lmt->srd_sng=(char *)nco_free(lmt->srd_sng);
+  lmt->re_bs_sng=(char*)nco_free(lmt->re_bs_sng);   
 
   lmt=(lmt_sct *)nco_free(lmt);
 
@@ -212,7 +213,9 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
   long cnt_rmn_crr=-1L; /* Records to extract from current file */
   long cnt_rmn_ttl=-1L; /* Total records remaining to be read from this and all remaining files */
   long rec_skp_vld_prv_dgn=-1L; /* Records skipped at end of previous valid file (diagnostic only) */
-  
+    
+  char *fl_udu_sng=NULL_CEWI;   /* store units attribute of co-ordinate dim */
+          
   lmt=*lmt_ptr;
   
   prg_id=prg_get(); /* Program ID */
@@ -289,7 +292,21 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
     nco_exit(EXIT_FAILURE);
   } /* end if */
   lmt.lmt_typ=min_lmt_typ;
-  
+
+  /* co-ordinate rebasing code */
+  lmt.origin=0.0;
+  /* Get variable ID of coordinate */
+  rcd=nco_inq_varid_flg(nc_id,lmt.nm,&dim.cid);
+  if(rcd==NC_NOERR)
+    fl_udu_sng=nco_lmt_get_udu_att(nc_id,dim.cid); /* units attribute of co-ordinate var */
+
+  if(rec_dmn_and_mlt_fl_opr && fl_udu_sng && lmt.re_bs_sng){ 
+#ifdef ENABLE_UDUNITS
+      /* Re-base and reset origin to 0.0 if re-basing fails */
+      if(nco_lmt_clc_org(fl_udu_sng,lmt.re_bs_sng,&lmt.origin)!=EXIT_SUCCESS) lmt.origin=0.0;
+#endif /* !ENABLE_UDUNITS */
+    } /* endif */
+
   if((lmt.lmt_typ == lmt_crd_val) || (lmt.lmt_typ == lmt_udu_sng)){
     double *dmn_val_dp;
     
@@ -300,9 +317,10 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
     long min_idx;
     long tmp_idx;
     long dmn_srt=0L;
-    
-    /* Get variable ID of coordinate */
-    rcd=nco_inq_varid(nc_id,lmt.nm,&dim.cid);
+
+
+    /* Get variable ID of coordinate 
+       rcd=nco_inq_varid(nc_id,lmt.nm,&dim.cid); */
     
     /* Get coordinate type */
     (void)nco_inq_vartype(nc_id,dim.cid,&dim.type);
@@ -384,15 +402,29 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
     
     /* Convert UDUnits strings if necessary */
     if(lmt.lmt_typ == lmt_udu_sng){
-      if(nco_lmt_udu_cnv(nc_id,dim.cid,lmt.min_sng,&lmt.min_val)) nco_exit(EXIT_FAILURE);
-      if(nco_lmt_udu_cnv(nc_id,dim.cid,lmt.max_sng,&lmt.max_val)) nco_exit(EXIT_FAILURE);
+      
+      if(!fl_udu_sng){ 
+        (void)fprintf(stdout,"%s: ERROR attempting to read units attribute from variable \"%s\" \n",prg_nm_get(),dim.nm);
+        nco_exit(EXIT_FAILURE);
+      } /* end if */
+      
+      if(nco_lmt_udu_cnv(dim.cid,fl_udu_sng,lmt.min_sng,&lmt.min_val)) nco_exit(EXIT_FAILURE);
+      if(nco_lmt_udu_cnv(dim.cid,fl_udu_sng,lmt.max_sng,&lmt.max_val)) nco_exit(EXIT_FAILURE);
+ 
     }else{ /* end UDUnits conversion */
       /* Convert user-specified limits into double precision numeric values, or supply defaults */
       if(lmt.min_sng == NULL) lmt.min_val=dmn_val_dp[min_idx]; else lmt.min_val=strtod(lmt.min_sng,(char **)NULL);
       if(lmt.max_sng == NULL) lmt.max_val=dmn_val_dp[max_idx]; else lmt.max_val=strtod(lmt.max_sng,(char **)NULL);
+      
+      /* re-base co-ordinates as necessary in multi-file operatators 
+         lmt.origin has been calculated earlier in file */
+      if(rec_dmn_and_mlt_fl_opr){ 
+        lmt.min_val-=lmt.origin;
+        lmt.max_val-=lmt.origin;     
+      }
     }
     /* Warn when min_val > max_val (i.e., wrapped coordinate)*/
-    if(dbg_lvl_get() > 0 && lmt.min_val > lmt.max_val) (void)fprintf(stderr,"%s: INFO Interpreting hyperslab specifications as wrapped coordinates [%s <= %g] and [%s >= %g]\n",prg_nm_get(),lmt.nm,lmt.max_val,lmt.nm,lmt.min_val);
+    if(dbg_lvl_get() > nco_dbg_std && lmt.min_val > lmt.max_val) (void)fprintf(stderr,"%s: INFO Interpreting hyperslab specifications as wrapped coordinates [%s <= %g] and [%s >= %g]\n",prg_nm_get(),lmt.nm,lmt.max_val,lmt.nm,lmt.min_val);
     
     /* Fail when... */
     if(
@@ -509,7 +541,7 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
     
     /* Un-typecast pointer to values after access */
     (void)cast_nctype_void((nc_type)NC_DOUBLE,&dim.val);
-    
+  
     /* Free space allocated for dimension */
     dim.val.vp=nco_free(dim.val.vp);
     
@@ -552,7 +584,7 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
       lmt.min_idx--;
       lmt.max_idx--;
     } /* end if */
-    
+  
     /* Exit if requested indices are always invalid for all operators... */
     if(lmt.min_idx < 0 || lmt.max_idx < 0 || 
        /* ...or are invalid for non-record dimensions or single file operators */
@@ -583,6 +615,8 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
 	 No records were skipped in previous files */
       if(cnt_crr == 0L && lmt.rec_skp_nsh_spf == 0L) lmt.rec_skp_vld_prv=0L;
       
+    
+
       /* This if statement is required to avoid an ugly goto statment */
       if(!flg_no_data){
 	
@@ -806,7 +840,7 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
   /* Place contents of working structure in location of returned structure */
   *lmt_ptr=lmt;
   
-  if(dbg_lvl_get() == 5){
+  if(dbg_lvl_get() >= nco_dbg_io){
     (void)fprintf(stderr,"Dimension hyperslabber nco_lmt_evl() diagnostics:\n");
     (void)fprintf(stderr,"Dimension name = %s\n",lmt.nm);
     (void)fprintf(stderr,"Limit type is %s\n",(min_lmt_typ == lmt_crd_val) ? "coordinate value" : "zero-based dimension index");
@@ -836,6 +870,8 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
     if(prg_id != ncks) (void)fprintf(stderr,"WARNING: Possible instance of Schweitzer data hole requiring better diagnostics TODO #148\n");
     if(prg_id != ncks) (void)fprintf(stderr,"HINT: If operation fails, try multislabbing (http://nco.sf.net/nco.html#msa) wrapped dimension using ncks first, and then apply %s to the resulting file\n",prg_nm_get());
   } /* end dbg */
+
+  (void*)nco_free(fl_udu_sng);
   
 } /* end nco_lmt_evl() */
 
@@ -866,7 +902,6 @@ nco_lmt_prs /* [fnc] Create limit structures with name, min_sng, max_sng element
   if(lmt_nbr > 0) lmt=(lmt_sct **)nco_malloc(lmt_nbr*sizeof(lmt_sct *));
   for(idx=0;idx<lmt_nbr;idx++){
     /* Process hyperslab specifications as normal text list */
-    /* fxm: probably need to free arg_lst sometime... */
     arg_lst=nco_lst_prs_2D(lmt_arg[idx],dlm_sng,&arg_nbr);
     
     /* Check syntax */
@@ -878,12 +913,12 @@ nco_lmt_prs /* [fnc] Create limit structures with name, min_sng, max_sng element
        (arg_nbr == 3 && arg_lst[1] == NULL && arg_lst[2] == NULL) || /* No min or max when stride not specified */
        (arg_nbr == 4 && arg_lst[3] == NULL) || /* Stride should be specified */
        False){
-      (void)fprintf(stdout,"%s: ERROR in hyperslab specification for dimension %s\n%s: HINT Conform request to hyperslab documentation at http://nco.sf.net#hyp\n",prg_nm_get(),lmt_arg[idx],prg_nm_get());
+      (void)fprintf(stdout,"%s: ERROR in hyperslab specification for dimension %s\n%s: HINT Conform request to hyperslab documentation at http://nco.sf.net/nco.html#hyp\n",prg_nm_get(),lmt_arg[idx],prg_nm_get());
       nco_exit(EXIT_FAILURE);
     } /* end if */
     
     /* Initialize structure */
-    /* lmt strings which are not explicitly set by user remain NULL, i.e., 
+    /* lmt strings that are not explicitly set by user remain NULL, i.e., 
        specifying default setting will appear as if nothing at all was set.
        Hopefully, in routines that follow, branch followed when dimension has
        all default settings specified (e.g.,"-d foo,,,,") yields same answer
@@ -907,10 +942,14 @@ nco_lmt_prs /* [fnc] Create limit structures with name, min_sng, max_sng element
     
     if(lmt[idx]->max_sng == NULL) lmt[idx]->is_usr_spc_max=False; else lmt[idx]->is_usr_spc_max=True;
     if(lmt[idx]->min_sng == NULL) lmt[idx]->is_usr_spc_min=False; else lmt[idx]->is_usr_spc_min=True;
-    
+
+    /* Initialize types used to re-base coordinate variables */
+    lmt[idx]->origin=0.0;
+    lmt[idx]->re_bs_sng=NULL_CEWI;
+
     /* Free current pointer array to strings
-       Strings themselves are untouched and will be free()'d with limit structures 
-       in nco_lmt_lst_free() */
+       Strings themselves are untouched 
+       They will be free()'d with limit structures in nco_lmt_lst_free() */
     arg_lst=(char **)nco_free(arg_lst);
 
   } /* End loop over lmt structure list */
@@ -919,100 +958,321 @@ nco_lmt_prs /* [fnc] Create limit structures with name, min_sng, max_sng element
   
 } /* end nco_lmt_prs() */
 
+void
+nco_prs_time /* Parse time limits into years, months, day, etc. */
+(const char * const in_sng, /* I [sng] String to parse */
+ int * const dt, /* O [sct] Time decomposed into longs */
+ double * const out_sec) /* O [s] Double precision seconds */
+{
+  /* Parse limits into years, months, day, etc. */
+  char **arg_lst; /* Array of date/time strings after parsing */
+  int arg_nbr; /* [nbr] Number of arguments parsed */
+  int year,month,day,hour,min;
+  double sec;
+    
+  /* Clear date and time */
+  year=0L;
+  month=0L;
+  day=0L;
+  hour=0L;
+  min=0L;
+  sec=(double)0;
+  /* Break up date string copy delimited by dashes */
+  arg_lst=nco_lst_prs_2D(in_sng,"-",&arg_nbr);
+  /* Fill in year, month, day */
+  if((arg_nbr > 0) && (arg_lst[0] != NULL)) year=strtol(arg_lst[0],(char **)NULL,10);
+  if((arg_nbr > 1) && (arg_lst[1] != NULL)) month=strtol(arg_lst[1],(char **)NULL,10);
+  if((arg_nbr > 2) && (arg_lst[2] != NULL)) day=strtol(arg_lst[2],(char **)NULL,10);
+  /* Advance pointer past date/time delimiting space */
+  if(arg_lst[2] != NULL) arg_lst[2]=strrchr(arg_lst[2],' ');
+  /* Step past last space */
+  if(arg_lst[2] != NULL) arg_lst[arg_nbr-1]++;
+  /* Break up remaining time string delimited by colons */
+  if(arg_lst[2] != NULL){
+    arg_lst=nco_lst_prs_2D(arg_lst[2],":",&arg_nbr);
+    /* Fill in hour, minute, second */
+    if((arg_nbr > 0) && strlen(arg_lst[0]) > 2){ /* HHMM spec */
+      hour=strtol(arg_lst[0],(char **)NULL,10);
+      hour=(long)(hour/100);
+      min=strtol(arg_lst[0],(char **)NULL,10);
+      min=(long)(min%100);
+    }else{ /* HH:MM:S.S spec */
+      if((arg_nbr > 0) && (arg_lst[0] != NULL)) hour=strtol(arg_lst[0],(char **)NULL,10);
+      if((arg_nbr > 1) && (arg_lst[1] != NULL)) min=strtol(arg_lst[1],(char **)NULL,10);
+      if((arg_nbr > 2) && (arg_lst[2] != NULL)) sec=strtod(arg_lst[2],(char **)NULL);
+    } /* end else */
+  } /* if arg_lst[2] == NULL */
+
+  arg_lst=nco_sng_lst_free(arg_lst,arg_nbr);
+
+  dt[0]=year;
+  dt[1]=month;
+  dt[2]=day;
+  dt[3]=hour;
+  dt[4]=min;
+  *out_sec=sec;
+
+} /* end nco_prs_time */
+
+#ifdef ENABLE_UDUNITS
+# ifdef HAVE_UDUNITS2_H
+
+
 int /* [rcd] Successful conversion returns 0 */
 nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
-(const int nc_id, /* I [idx] netCDF file ID */
- const int dmn_id, /* I [idx] netCDF dimension ID */
+(const int dmn_id, /* I [idx] netCDF dimension ID */
+ const char* fl_unt_sng, /* string with disk units attribute */
  char *lmt_sng, /* I [ptr] Limit string */
  double *lmt_val) /* O [val] Limit coordinate value */ 
 {
-  int rcd; /* Return code */
-#ifdef ENABLE_UDUNITS
-  static const char* att_nm="units";
-  char *fl_unt_sng=NULL;  /* [sng] Unit string in file */
+  int ut_rcd; /* [enm] UDUnits2 status */
   char *usr_unt_sng; /* [sng] User-specified unit string */
-  long att_sz; /* [nbr] "units" attribute size */
-  nc_type att_typ; /* [enm] Atttribute type, probably NC_CHAR */
-  utUnit udu_sct_in; /* Unidata units structure */
-  utUnit udu_sct_out; /* Unidata units structure */
+  double crr_val; 
+   
+  cv_converter *ut_cnv; /* UDUnits converter */
+  ut_system *ut_sys;
+  ut_unit *ut_sct_in; /* UDUnits structure, input units */
+  ut_unit *ut_sct_out; /* UDUnits structure, output units */
   
-#ifdef UDUNITS_PATH
+
+  /* When empty, ut_read_xml() uses environment variable UDUNITS2_XML_PATH, if any
+     Otherwise it uses default initial location hardcoded when library was built */
+  if(dbg_lvl_get() >= nco_dbg_vrb) ut_set_error_message_handler(ut_write_to_stderr); else ut_set_error_message_handler(ut_ignore);
+  ut_sys=ut_read_xml(NULL);
+  if(ut_sys == NULL){
+    (void)fprintf(stdout,"%s: nco_udu_lmt_cnv() failed to initialize UDUnits2 library\n",prg_nm_get());
+    return EXIT_FAILURE;
+  } /* end if err */ 
+
+  
+  /* Convert on-disk 'units' attribute into ut_unit structure */
+  ut_sct_out=ut_parse(ut_sys,fl_unt_sng,UT_ASCII); 
+  if(ut_sct_out == NULL){ /* Problem with 'units' attribute */
+    ut_rcd=ut_get_status(); /* [enm] UDUnits2 status */
+    if(ut_rcd == UT_BAD_ARG) (void)fprintf(stderr,"ERROR: empty attribute or units system for dimension ID %d\n",dmn_id);
+    if(ut_rcd == UT_SYNTAX) (void)fprintf(stderr,"ERROR: units  attribute \"%s\" for dimension ID %d has a syntax error\n",fl_unt_sng,dmn_id);
+    if(ut_rcd == UT_UNKNOWN) (void)fprintf(stderr,"ERROR: units attribute \"%s\" for dimension ID %d is not listed in UDUnits2 SI system database\n",fl_unt_sng,dmn_id);
+
+    return EXIT_FAILURE;
+  } /* endif coordinate on disk has no units attribute */
+
+
+
+  /* check if disk units is a timestamp if so */
+  if( strstr(fl_unt_sng," from ") || strstr(fl_unt_sng," since ") || strstr(fl_unt_sng," after ") ){
+
+  // prefix output units with s@ so that ut_parse works
+    if(strncmp("s@", lmt_sng,2)){
+      usr_unt_sng=(char*)nco_malloc( (strlen(lmt_sng)+3) *sizeof(char) );
+      strcpy(usr_unt_sng,"s@");
+      strcat(usr_unt_sng,lmt_sng);
+    }else{
+      usr_unt_sng=strdup(lmt_sng);
+    }
+
+
+    crr_val=0.0;
+
+  /* assume we are dealing with regular output units */
+  }else{
+    usr_unt_sng=(char*)nco_calloc( strlen(lmt_sng)+1, sizeof(char));
+    /* number space units */
+    /* this should be gaurranted by calling function */
+    sscanf(lmt_sng,"%lg %s", &crr_val,usr_unt_sng);    
+  }
+
+
+
+  /* Convert user-specified unit into utUnit structure */
+  ut_sct_in=ut_parse(ut_sys,usr_unt_sng,UT_ASCII); 
+  if(ut_sct_in == NULL){ /* Problem with 'units' attribute */
+    ut_rcd=ut_get_status(); /* [enm] UDUnits2 status */
+    if(ut_rcd == UT_BAD_ARG) (void)fprintf(stderr,"ERROR: Empty attribute or units system\n");
+    if(ut_rcd == UT_SYNTAX) (void)fprintf(stderr,"ERROR: User-specified unit \"%s\" has a syntax error\n",usr_unt_sng);
+    if(ut_rcd == UT_UNKNOWN) (void)fprintf(stderr,"ERROR: User-specified unit \"%s\" is not listed in UDUnits2 SI system database\n",usr_unt_sng);
+
+    return EXIT_FAILURE;
+  } /* endif */
+
+
+  /* Create a converter */
+  ut_cnv=ut_get_converter(ut_sct_in,ut_sct_out); /* UDUnits converter */
+  if(ut_cnv == NULL){
+    ut_rcd=ut_get_status(); /* [enm] UDUnits2 status */
+    if(ut_rcd == UT_BAD_ARG) (void)fprintf(stderr,"ERROR: One of units is NULL\n");
+    if(ut_rcd == UT_NOT_SAME_SYSTEM) (void)fprintf(stderr,"ERROR: Units belong to different unit systems\n");
+    if(ut_rcd == UT_MEANINGLESS) (void)fprintf(stderr,"ERROR: Conversion between user specified unit \"%s\" and file units \"%s\" is  is meaningless\n",usr_unt_sng,fl_unt_sng);
+
+    return EXIT_FAILURE;
+  } /* endif */
+
+
+  /* Finally do the conversion  */
+  *lmt_val=cv_convert_double(ut_cnv,crr_val);
+
+  nco_free(usr_unt_sng); 
+
+  ut_free_system(ut_sys); /* Free memory taken by UDUnits library */
+  ut_free(ut_sct_in);
+  ut_free(ut_sct_out);
+  cv_free(ut_cnv);
+
+  return EXIT_SUCCESS; /* Successful conversion */
+
+}  /* end nco_lmt_udu_cnv() */
+
+
+
+int   /* O  difference between two co-ordinate units */      
+nco_lmt_clc_org(
+const char* fl_unt_sng, /* I [ptr] units attribute string from disk  */     
+const char* fl_bs_sng,  /* I [ptr] units attribute string from disk  */     
+double *og_val)         /* O difference between two units string */
+{
+  int ut_rcd; /* [enm] UDUnits2 status */
+  double in_val=0.0;
+      
+  cv_converter *ut_cnv; /* UDUnits converter */
+  ut_system *ut_sys;
+  ut_unit *ut_sct_in; /* UDUnits structure, input units */
+  ut_unit *ut_sct_out; /* UDUnits structure, output units */
+
+
+  /* quick return if units identical */
+  if(!strcmp(fl_unt_sng,fl_bs_sng) ){
+    *og_val=0.0;  
+    return EXIT_SUCCESS;
+  }
+  /* When empty, ut_read_xml() uses environment variable UDUNITS2_XML_PATH, if any
+     Otherwise it uses default initial location hardcoded when library was built */
+  if(dbg_lvl_get() >= nco_dbg_vrb) ut_set_error_message_handler(ut_write_to_stderr); else ut_set_error_message_handler(ut_ignore);
+  ut_sys=ut_read_xml(NULL);
+  if(ut_sys == NULL){
+    (void)fprintf(stdout,"%s: nco_udu_lmt_cnv() failed to initialize UDUnits2 library\n",prg_nm_get());
+    return EXIT_FAILURE; /* Failure */
+  } /* end if err */ 
+
+  
+  /* units string to convert from */
+  ut_sct_in=ut_parse(ut_sys,fl_unt_sng,UT_ASCII); 
+  if(ut_sct_in == NULL){ /* Problem with 'units' attribute */
+    ut_rcd=ut_get_status(); /* [enm] UDUnits2 status */
+    if(ut_rcd == UT_BAD_ARG) (void)fprintf(stderr,"ERROR: empty units attribute string\n");
+    if(ut_rcd == UT_SYNTAX) (void)fprintf(stderr,"ERROR:  units attribute \"%s\" has a syntax error\n",fl_unt_sng);
+    if(ut_rcd == UT_UNKNOWN) (void)fprintf(stderr,"ERROR: units attribute \"%s\" is not listed in UDUnits2 SI system database\n",fl_unt_sng);
+
+    return EXIT_FAILURE; /* Failure */
+  } /* endif coordinate on disk has no units attribute */
+
+
+
+
+  /* units string to convert to */
+  ut_sct_out=ut_parse(ut_sys,fl_bs_sng,UT_ASCII); 
+  if(ut_sct_out == NULL){ /* Problem with 'units' attribute */
+    ut_rcd=ut_get_status(); /* [enm] UDUnits2 status */
+    if(ut_rcd == UT_BAD_ARG) (void)fprintf(stderr,"ERROR: Empty units attribute string\n");
+    if(ut_rcd == UT_SYNTAX) (void)fprintf(stderr,"ERROR: units attribute  \"%s\" has a syntax error\n",fl_bs_sng);
+    if(ut_rcd == UT_UNKNOWN) (void)fprintf(stderr,"ERROR: units attribute \"%s\" is not listed in UDUnits2 SI system database\n",fl_bs_sng);
+
+      return EXIT_FAILURE; /* Failure */
+  } /* endif */
+
+
+  /* Create a converter */
+  ut_cnv=ut_get_converter(ut_sct_in,ut_sct_out); /* UDUnits converter */
+  if(ut_cnv == NULL){
+    ut_rcd=ut_get_status(); /* [enm] UDUnits2 status */
+    if(ut_rcd == UT_BAD_ARG) (void)fprintf(stderr,"ERROR: One of units is NULL\n");
+    if(ut_rcd == UT_NOT_SAME_SYSTEM) (void)fprintf(stderr,"ERROR: Units belong to different unit systems\n");
+    if(ut_rcd == UT_MEANINGLESS) (void)fprintf(stderr,"ERROR: Conversion between user specified unit \"%s\" and file units \"%s\" is  is meaningless\n",fl_bs_sng,fl_unt_sng);
+    return EXIT_FAILURE; /* Failure */
+  } /* endif */
+
+
+  /* Finally do the conversion  */
+  *og_val=cv_convert_double(ut_cnv,in_val);
+  
+  if(dbg_lvl_get() >nco_dbg_std) {
+    fprintf(stderr, "%s : nco_lmt_clc_org: difference between systems \"%s\" and \"%s\" is %f\n",prg_nm_get(),fl_unt_sng,fl_bs_sng, *og_val);
+  }
+
+
+  ut_free_system(ut_sys); /* Free memory taken by UDUnits library */
+  ut_free(ut_sct_in);
+  ut_free(ut_sct_out);
+  cv_free(ut_cnv);
+
+  return EXIT_SUCCESS;
+
+}  /* end nco_lmt_clc_org() */
+
+
+
+# else /* !HAVE_UDUNITS2_H */
+
+int /* [rcd] Successful conversion returns 0 */
+nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
+(const int dmn_id, /* I [idx] netCDF dimension ID */
+ const char* fl_unt_sng, /* string with disk units attribute */
+ char *lmt_sng, /* I [ptr] Limit string */
+ double *lmt_val) /* O [val] Limit coordinate value */ 
+{
+  int rcd;
+  char *usr_unt_sng; /* [sng] User-specified unit string */
+
+  utUnit udu_sct_in; /* UDUnits structure, input units */
+  utUnit udu_sct_out; /* UDUnits structure, output units */
+
+# ifdef UDUNITS_PATH
   /* UDUNITS_PATH macro expands to where autoconf found database file */
   rcd=utInit(UDUNITS_PATH);
-#else /* !UDUNITS_PATH */
+# else /* !UDUNITS_PATH */
   /* When empty, utInit() uses environment variable UDUNITS_PATH, if any
      Otherwise it uses default initial location hardcoded when library was built */
   rcd=utInit("");
-#endif /* !UDUNITS_PATH */
+# endif /* !UDUNITS_PATH */
+
   if(rcd != 0){
-    (void)fprintf(stdout,"Failed to initialize UDUnits library\n");
-    return 1;
+    (void)fprintf(stdout,"%s: nco_udu_lmt_cnv() failed to initialize UDUnits library\n",prg_nm_get());
+
+    return EXIT_FAILURE;
+
   } /* end if err */ 
-  
-  /* Does 'units' attribute exist? */
-  if(nco_inq_att(nc_id,dmn_id,att_nm,&att_typ,&att_sz) == NC_NOERR){
-    /* Allocate memory for attribute */
-    fl_unt_sng=(char *)nco_malloc((att_sz+1)*sizeof(char));
-    /* Get 'units' attribute */
-    (void)nco_get_att(nc_id,dmn_id,att_nm,fl_unt_sng,att_typ);
-    fl_unt_sng[att_sz]='\0';
-  } /* end if 'units' attribute */
-  
-  /* Convert 'units' attribute into utUnit structure */
+
+  /* Convert on-disk 'units' attribute into utUnit structure */
   rcd=utScan(fl_unt_sng,&udu_sct_out); 
-  if(rcd == UT_EINVALID){ /* 'units' attribute is absent*/
+  if(rcd == UT_EINVALID){ /* 'units' attribute is absent */
     (void)fprintf(stderr,"ERROR: no units attribute available for dimension %d\n",dmn_id);
     return 1; 
-  } /* endif no units attribute */
+  } /* endif coordinate on disk has no units attribute */
   if(rcd == UT_EUNKNOWN || rcd == UT_ESYNTAX){
     (void)fprintf(stderr,"ERROR units attribute \"%s\" (in input file) is not a valid UDUnits string",fl_unt_sng);
     (void)utTerm(); /* Free memory taken by UDUnits library */
     return 1;
   } /* endif unkown type */
+
   
-  /* Assume if unit attribute is 'time since...', then user specified same */
+
   if(utIsTime(&udu_sct_out) && utHasOrigin(&udu_sct_out)){
-    /* Parse limits into years, months, day, etc. */
-    char **arg_lst; /* Array of date/time strings after parsing */
-    int arg_nbr; /* [nbr] Number of arguments parsed */
-    int year,month,day,hour,min;
+ 
+
+    int dt[5]; /* 0-year, 1-month, 2-day, 3-hour, 4-minute */
     double sec;
     double *crr_val=lmt_val; /* Pointer to set min/max value */
+          
+    /* Parse limits into years, months, day, etc. */
+    (void)nco_prs_time(lmt_sng,dt,&sec);
     
-    /* Clear date and time */
-    year=0L; month=0L; day=0L; hour=0L; min=0L; sec=(double)0;
-    /* Break up date string copy delimited by dashes */
-    arg_lst=nco_lst_prs_2D(lmt_sng,"-",&arg_nbr);
-    /* Fill in year, month, day */
-    if((arg_nbr > 0) && (arg_lst[0] != NULL)) year=strtol(arg_lst[0],(char **)NULL,10);
-    if((arg_nbr > 1) && (arg_lst[1] != NULL)) month=strtol(arg_lst[1],(char **)NULL,10);
-    if((arg_nbr > 2) && (arg_lst[2] != NULL)) day=strtol(arg_lst[2],(char **)NULL,10);
-    /* Advance pointer past date/time delimiting space */
-    if(arg_lst[2] != NULL) arg_lst[2]=strrchr(arg_lst[2],' ');
-    /* Step past last space */
-    if(arg_lst[2] != NULL) arg_lst[arg_nbr-1]++;
-    /* Break up remaining time string delimited by colons */
-    if(arg_lst[2] != NULL){
-      arg_lst=nco_lst_prs_2D(arg_lst[2],":",&arg_nbr);
-      /* Fill in hour, minute, second */
-      if((arg_nbr > 0) && strlen(arg_lst[0]) > 2){ /* HHMM spec */
-	hour=strtol(arg_lst[0],(char **)NULL,10);
-	hour=(long)(hour/100);
-	min=strtol(arg_lst[0],(char **)NULL,10);
-	min=(long)(min%100);
-      }else{ /* HH:MM:S.S spec */
-	if((arg_nbr > 0) && (arg_lst[0] != NULL)) hour=strtol(arg_lst[0],(char **)NULL,10);
-	if((arg_nbr > 1) && (arg_lst[1] != NULL)) min=strtol(arg_lst[1],(char **)NULL,10);
-	if((arg_nbr > 2) && (arg_lst[2] != NULL)) sec=strtod(arg_lst[2],(char **)NULL);
-      } /* end else */
-    } /* if arg_lst[2] == NULL */
-    
-      /* Convert dates into "time since ..." value */
-    rcd=utInvCalendar(year,month,day,hour,min,sec,&udu_sct_out,crr_val);
+
+    rcd=utInvCalendar(dt[0],dt[1],dt[2],dt[3],dt[4],sec,&udu_sct_out,crr_val);
     
   }else{ /* if utIsTime() && utHasOrigin */
-    
-    double slp,incpt,crr_val; /* Slope and intercept */ 
+
+    double slp; /* Slope */
+    double incpt; /* Intercept */
+    double crr_val; /* Current value */ 
+
     /* Parse user-specified unit */
     usr_unt_sng=strchr(lmt_sng,' ');
     /* Advance past space delimiter
@@ -1023,41 +1283,172 @@ nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
     if(strcmp(usr_unt_sng,fl_unt_sng) == 0){
       *lmt_val=strtod(lmt_sng,&usr_unt_sng); /* Convert to double */
       (void)utTerm();
-      return 0; /* Success */
+      return EXIT_SUCCESS;
     } /* endif */
-    
-    /* Convert user-specified unit into utUnit structure */
+
     rcd=utScan(usr_unt_sng,&udu_sct_in);
     if(rcd == UT_EUNKNOWN){
       (void)fprintf(stderr,"ERROR: User-specified unit \"%s\" is not in UDUnits database.\n",usr_unt_sng);
       (void)utTerm();
-      return 1;
+      return EXIT_FAILURE;
     } /* endif */
-    
-    /* Get unit transformation information */
+
     rcd=utConvert(&udu_sct_in,&udu_sct_out,&slp,&incpt);
     if(rcd == UT_ECONVERT){
       (void)fprintf(stderr,"ERROR: user specified unit \"%s\" cannot be converted into file units \"%s\"\n",usr_unt_sng,fl_unt_sng);
       (void)utTerm();
-      return 1; /* Failure */
+
+      return EXIT_FAILURE;
     } /* endif */
-    
     /* Convert to disk-based units */
     crr_val=strtod(lmt_sng,&usr_unt_sng);
     crr_val=slp*crr_val+incpt;
     *lmt_val=crr_val;
+    
   } /* !utIsTime() ... */
-  
+
   (void)utTerm(); /* Free memory taken by UDUnits library */
-  return 0; /* Successful conversion */
+
+  return EXIT_SUCCESS; /* Successful conversion */
+}  /* end nco_lmt_udu_cnv() */
+
+
+
+int                 /* [rcd] Successful conversion returns 0 */
+nco_lmt_clc_org(    /* [fnc] Difference between two co-ordinate units */      
+const char* fl_unt_sng, /* I [ptr] units attribute string from disk  */     
+const char* fl_bs_sng,  /* I [ptr] units attribute string from disk  */     
+double *og_val){        /* O [ptr]                                   */
+
+  int rcd;
+
+  double slp;
+  double incpt;
+
+  utUnit udu_sct_in; /* UDUnits structure, input units */
+  utUnit udu_sct_out; /* UDUnits structure, output units */
+
+
+  /* quick return if units identical */
+  if(!strcmp(fl_unt_sng,fl_bs_sng) ){
+    *og_val=0.0;  
+    return EXIT_SUCCESS;
+  }
+
+# ifdef UDUNITS_PATH
+  /* UDUNITS_PATH macro expands to where autoconf found database file */
+  rcd=utInit(UDUNITS_PATH);
+# else /* !UDUNITS_PATH */
+  /* When empty, utInit() uses environment variable UDUNITS_PATH, if any
+     Otherwise it uses default initial location hardcoded when library was built */
+  rcd=utInit("");
+# endif /* !UDUNITS_PATH */
+
+  if(rcd != 0){
+    (void)fprintf(stdout,"%s: nco_lmt_cls_org() failed to initialize UDUnits library\n",prg_nm_get());
+
+    return EXIT_FAILURE;
+
+  } /* end if err */ 
+
+
+  /* units string to convert from */
+  rcd=utScan(fl_unt_sng,&udu_sct_in); 
+  if(rcd !=0){
+    if(rcd == UT_EINVALID)
+      (void)fprintf(stderr,"ERROR: units attribute \"%s\" is invalid \n",fl_unt_sng);
+
+    if(rcd == UT_ESYNTAX)
+      (void)fprintf(stderr,"ERROR units attribute \"%s\" contains a syntax error",fl_unt_sng);
+
+    if(rcd == UT_EUNKNOWN)
+      (void)fprintf(stderr,"ERROR units attribute \"%s\" is not in udunits database",fl_unt_sng);
+
+    (void)utTerm(); /* Free memory taken by UDUnits library */
+    return EXIT_FAILURE;
+  } /* endif unkown type */
+
+
+  /* units string to convert to */
+  rcd=utScan(fl_bs_sng,&udu_sct_out); 
+  if(rcd !=0){
+    if(rcd == UT_EINVALID)
+      (void)fprintf(stderr,"ERROR: units attribute \"%s\" is invalid \n",fl_bs_sng);
+
+    if(rcd == UT_ESYNTAX)
+      (void)fprintf(stderr,"ERROR units attribute \"%s\" contains a syntax error",fl_bs_sng);
+
+    if(rcd == UT_EUNKNOWN)
+      (void)fprintf(stderr,"ERROR units attribute \"%s\" is not in udunits database",fl_bs_sng);
+
+    (void)utTerm(); /* Free memory taken by UDUnits library */
+    return EXIT_FAILURE;
+  } /* endif unkown type */
+
+
+  rcd=utConvert(&udu_sct_in,&udu_sct_out,&slp,&incpt);
+  if(rcd == UT_ECONVERT){
+    (void)fprintf(stderr,"ERROR: user specified unit \"%s\" cannot be converted to units \"%s\"\n",fl_unt_sng,fl_bs_sng);
+    (void)utTerm();
+    return EXIT_FAILURE;
+  } /* endif */
+
+
+           
+  *og_val=incpt; 
+
+  /* debug stuff */
+  if(dbg_lvl_get() >nco_dbg_std) 
+    fprintf(stderr, "%s : nco_lmt_clc_org: difference between systems \"%s\" and \"%s\" is %f\n",prg_nm_get(),fl_unt_sng,fl_bs_sng, *og_val);
+  
+
+  (void)utTerm();
+
+  return EXIT_SUCCESS;   
+
+}
+
+
+
+# endif /*!HAVE_UDUNITS2 */
 #else /* !ENABLE_UDUNITS */
-  rcd=(int)(1+0*nc_id+0*dmn_id+0*(*lmt_val)); /* CEWI removes unused parameter warnings */
+
+int /* [rcd] Successful conversion returns 0 */
+nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
+(const int dmn_id, /* I [idx] netCDF dimension ID */
+ const char* fl_unt_sng, /* string with disk units attribute */
+ char *lmt_sng, /* I [ptr] Limit string */
+ double *lmt_val) /* O [val] Limit coordinate value */ 
+{
+  int rcd; /* [enm] Return code */
+
+  rcd=(int)(1+0*dmn_id+0*(*lmt_val)); /* CEWI removes unused parameter warnings */
   (void)fprintf(stdout,"UDUnits limit detected in \"%s\" but UDUnits library is unavailable to perform conversion.\nHINT: Re-compile and re-install NCO with UDUnits enabled. http://nco.sf.net/nco.html#udunits",lmt_sng);
   nco_exit(EXIT_FAILURE);
-  return 1; /* Conversion failed */
-#endif /* !ENABLE_UDUNITS */
+  return EXIT_FAILURE; /* Conversion failed */
+
 }  /* end nco_lmt_udu_cnv() */
 
+
+
+int                      /* O  difference between two co-ordinate units */      
+nco_lmt_clc_org(
+const char* fl_unt_sng, /* I [ptr] units attribute string from disk  */     
+const char* fl_bs_sng,  /* I [ptr] units attribute string from disk  */     
+double *og_val)         /* O difference between two units string */
+{
+  int rcd; /* [enm] Return code */
+
+  
+  (void)fprintf(stdout,"UDUnits limit detected in \"%s\" but UDUnits library is unavailable to perform conversion.\nHINT: Re-compile and re-install NCO with UDUnits enabled. http://nco.sf.net/nco.html#udunits",fl_unt_sng);
+  nco_exit(EXIT_FAILURE);
+  return EXIT_FAILURE; /* Conversion failed */
+} /* end nco_lmt_clc_org() */
+
+
+
+#endif /* !ENABLE_UDUNITS */
+
 int /* O [enm] Limit type */
 nco_lmt_typ /* [fnc] Determine limit type */
 (char *sng) /* I [ptr] Pointer to limit string */
@@ -1082,3 +1473,27 @@ nco_lmt_typ /* [fnc] Determine limit type */
 
 } /* end nco_lmt_typ() */
 
+char * /* O [sng] Units string */
+nco_lmt_get_udu_att /* Successful conversion returns units attribute otherwise null */
+(const int nc_id, /* I [idx] netCDF file ID */
+ const int var_id) /* I [id] Variable ID whose attribute to read */
+{
+ /* Grab units attribute from disk */
+ const char *att_nm="units"; 
+ nc_type att_typ; 
+ long att_sz;
+ char *fl_udu_sng=NULL_CEWI;
+
+ if(nco_inq_att_flg(nc_id,var_id,att_nm,&att_typ,&att_sz) == NC_NOERR){
+   /* Allocate memory for attribute */
+   if(att_typ == NC_CHAR){
+     fl_udu_sng=(char *)nco_malloc((att_sz+1UL)*sizeof(char));
+     /* Get 'units' attribute */
+     (void)nco_get_att(nc_id,var_id,att_nm,fl_udu_sng,att_typ);
+     fl_udu_sng[att_sz]='\0';
+   } /* !NC_CHAR */
+ } /* endif */
+ return fl_udu_sng;
+} /* end nco_lmt_get_udu_att() */
+
+
