@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_lmt.h,v 1.28 2007/02/23 21:59:30 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_lmt.h,v 1.39 2009/07/14 12:30:17 hmb Exp $ */
 
 /* Purpose: Hyperslab limits */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -26,7 +26,11 @@
 #include <netcdf.h> /* netCDF definitions and C library */
 #include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 #ifdef ENABLE_UDUNITS
-#include <udunits.h> /* Unidata units library */
+# ifdef HAVE_UDUNITS2_H
+#  include <udunits2.h> /* Unidata units library */
+# else
+#  include <udunits.h> /* Unidata units library */
+# endif /* !HAVE_UDUNITS2_H */
 #endif /* !ENABLE_UDUNITS */
 
 /* Personal headers */
@@ -35,6 +39,7 @@
 #include "nco_ctl.h" /* Program flow control functions */
 #include "nco_lst_utl.h" /* List utilities */
 #include "nco_mmr.h" /* Memory management */
+#include "nco_sng_utl.h" /* String utilities */
 
 #ifdef __cplusplus
 extern "C" {
@@ -57,6 +62,11 @@ lmt_sct * /* O [sct] Pointer to free'd structure */
 nco_lmt_free /* [fnc] Free memory associated with limit structure */
 (lmt_sct *lmt); /* I/O [sct] Limit structure to free */
 
+char * /* O [sng] Units string */
+nco_lmt_get_udu_att /* Successful conversion returns units attribute otherwise null */
+(const int nc_id, /* I [idx] netCDF file ID */
+ const int var_id); /* I [id] Variable ID whose attribute to read */
+
 lmt_sct ** /* O [sct] Pointer to free'd structure list */
 nco_lmt_lst_free /* [fnc] Free memory associated with limit structure list */
 (lmt_sct **lmt_lst, /* I/O [sct] Limit structure list to free */
@@ -90,11 +100,24 @@ nco_lmt_typ /* [fnc] Determine limit type */
 
 int /* [rcd] Successful conversion returns 0 */
 nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
-(const int nc_id, /* I [idx] netCDF file ID */
- const int dmn_id, /* I [idx] netCDF dimension ID */
+(const int dmn_id, /* I [idx] netCDF dimension ID */
+ const char* fl_unt_sng, /* I [ptr] string with disk units attribute */ 
  char *lmt_sng, /* I [ptr] Limit string */
  double *lmt_val); /* O [val] Limit coordinate value */ 
 
+void
+nco_prs_time /* Parse time limits into years, months, day, etc. */
+(const char * const in_sng, /* I [sng] String to parse */
+ int * const dt, /* O [sct] Time decomposed into longs */
+ double * const out_sec); /* O [s] Double precision seconds */
+
+int                 /* [rcd] Successful conversion returns 0 */
+nco_lmt_clc_org(    /* [fnc] Difference between two co-ordinate units */      
+const char* fl_unt_sng, /* I [ptr] units attribute string from disk  */     
+const char* fl_bs_sng,  /* I [ptr] units attribute string from disk  */     
+double *og_val);        /* O [ptr]                                   */
+
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif /* __cplusplus */
