@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_lst_utl.c,v 1.46 2007/05/19 00:11:35 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_lst_utl.c,v 1.53 2009/04/19 23:17:04 zender Exp $ */
 
 /* Purpose: List utilities */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_lst_utl.h" /* List utilities */
@@ -42,7 +42,7 @@ nco_srt_lph /* [fnc] Sort array of strings */
 nm_id_sct * /* O [sct] Sorted output list */
 lst_heapsort /* [fnc] Heapsort input lists numerically or alphabetically */
 (nm_id_sct *lst, /* I/O [sct] Current list (destroyed) */
- const int nbr_lst, /* I [nbr] number of members in list */
+ const int nbr_lst, /* I [nbr] Number of members in list */
  const nco_bool ALPHABETIZE_OUTPUT) /* I [flg] Alphabetize extraction list */
 {
   /* Purpose: Sort extraction lists numerically or alphabetically
@@ -105,7 +105,7 @@ char ** /* O [sng] Array of list elements */
 nco_lst_prs_1D /* [fnc] Create 1D array of strings from given string and delimiter */
 (char * const sng_in, /* I/O [sng] Delimited argument list (delimiters are changed to NULL on output */
  const char * const dlm_sng, /* I [sng] Delimiter string */
- int * const nbr_lst) /* O [nbr] number of elements in list */
+ int * const nbr_lst) /* O [nbr] Number of elements in list */
 {
   /* Purpose: Create list of strings from given string and arbitrary delimiter
      Routine is often called with system memory, e.g., with strings from
@@ -191,7 +191,7 @@ char ** /* O [sng] List of strings */
 nco_lst_prs_2D /* [fnc] Create list of strings from given string and delimiter */
 (const char * const sng_in, /* I [sng] Delimited argument list */
  const char * const dlm_sng, /* I [sng] Delimiter string */
- int * const nbr_lst) /* O [nbr] number of elements in list */
+ int * const nbr_lst) /* O [nbr] Number of elements in list */
 {
   /* Purpose: Create list of strings from given string and arbitrary delimiter
      Algorithm recursively copies all text up to delimiter into new string and
@@ -280,6 +280,79 @@ nco_lst_prs_2D /* [fnc] Create list of strings from given string and delimiter *
   return sng_lst_out;
 } /* end nco_lst_prs_2D() */
 
+
+char ** /* O [sng] List of strings */
+nco_lst_prs_sgl_2D /* [fnc] Create list of strings from given string and delimiter */
+(const char * const sng_in, /* I [sng] Delimited argument list */
+ const char * const dlm_sng, /* I [sng] Delimiter string */
+ int * const nbr_lst) /* O [nbr] Number of elements in list */
+{
+  /* Similar to nco_lst_prs_2D() except for following            */
+  /* Only first char of dlm_sng is used and                      */
+  /* empty substrings are ignored and NOT returned as nulls      */
+  /* eg with dlm_sng=' ' and sng_in=" one two  three    five   " */
+  /* the following is returned "one"/"two"/"three"/"five"        */
+              
+                                    
+  int sng_len;
+  int nbr_sng;
+  char ch_dlm;
+  char *cp_sng;
+  char *cp_end;
+  char *cp_ptr;
+  char **lst=NULL;;
+
+  sng_len=strlen(sng_in);
+  
+  if(sng_len==0){
+    *nbr_lst=0;
+    return lst;
+  }
+    
+  /* use copy of string */
+  cp_sng=strdup(sng_in);
+
+  /* use only first char of delimiter string */
+  ch_dlm=dlm_sng[0];
+
+  cp_end=cp_sng+sng_len;
+  cp_ptr=cp_sng;
+
+
+  /* replace delimiters with nulls */
+  while( cp_ptr<cp_end){
+    if(*cp_ptr==ch_dlm)
+      *cp_ptr='\0';
+    cp_ptr++; 
+  }
+
+  nbr_sng=0;
+  cp_ptr=cp_sng;  
+  
+  /* loop thru string finding null delimited substrings */
+  while(cp_ptr < cp_end){
+    sng_len=strlen(cp_ptr);
+    if(sng_len >0 ){ 
+      lst=(char**)nco_realloc(lst, ++nbr_sng*sizeof(char*));
+      lst[nbr_sng-1]=strdup(cp_ptr); 
+      cp_ptr+=(sng_len+1);
+    }else
+      cp_ptr++;
+  }
+   
+  (void)nco_free(cp_sng);
+
+  *nbr_lst=nbr_sng;
+
+  return lst;
+
+} /* end nco_lst_prs_sgl_2D() */
+
+
+
+
+
+
 int /* O [enm] Comparison result [<,=,>] 0 iff val_1 [<,==,>] val_2 */
 nco_cmp_chr /* [fnc] Compare two characters */
 (const void * val_1, /* I [chr] Character to compare */
