@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_netcdf.c,v 1.78 2007/05/24 00:50:28 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_netcdf.c,v 1.116 2009/07/15 05:39:59 zender Exp $ */
 
 /* Purpose: NCO wrappers for netCDF C library */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
@@ -33,6 +33,28 @@
    To ensure this is the case, it is only safe to print diagnostics on
    variables which are supposed to be valid on input. */
 
+/* NB: nco_netcdf.c does not #include nco.h which #defines forward-compatibility tokens
+   This barrier helps segregate NCO from wrappers
+   Re-define minimal sub-set of tokens for nco_netcdf.c as necessary
+   This is small exception to the barrier */
+#ifndef NC_FORMAT_CLASSIC
+# define NC_FORMAT_CLASSIC (1)
+#endif
+#ifndef NC_FORMAT_64BIT
+# define NC_FORMAT_64BIT   (2)
+#endif
+#ifndef NC_FORMAT_NETCDF4
+# define NC_FORMAT_NETCDF4 (3)
+#endif
+#ifndef NC_FORMAT_NETCDF4_CLASSIC
+# define NC_FORMAT_NETCDF4_CLASSIC  (4) /* create netcdf-4 files, with NC_CLASSIC_MODEL. */
+#endif
+#ifndef NC_CHUNKED
+# define NC_CHUNKED (0)
+#endif
+#ifndef NC_CONTIGUOUS
+# define NC_CONTIGUOUS (1)
+#endif
 
 /* Utility routines not defined by netCDF library, but useful in working with it */
 void
@@ -58,7 +80,8 @@ nco_err_exit /* [fnc] Print netCDF error message, routine name, then exit */
 #endif /* !NCO_ABORT_ON_ERROR */
 
   switch(rcd){
-  case NC_ERANGE: (void)fprintf(stdout,"ERROR Result not representable in output file\nHINT: This may occur when an arithmetic operation results in a value not representible by the output variable type and NCO attempts to write that variable to an output file, with, e.g., nc_put_var*(). For more details, see\nhttp://nco.sf.net/nco.html#typ_cnv\n\nPossible workaround: Permanently promote the variable before attempting the arithmetic operation. For example,\nncap -O -s \'foo=double(foo);\' in.nc in.nc\n"); break;
+  case NC_ERANGE: (void)fprintf(stdout,"ERROR NC_ERANGE Result not representable in output file\nHINT: NC_ERANGE errors typically occur after an arithmetic operation results in a value not representible by the output variable type when NCO attempts to write those values to an output file.  Possible workaround: Promote the variable to higher precision before attempting arithmetic.  For example,\nncap2 -O -s \'foo=double(foo);\' in.nc in.nc\nFor more details, see http://nco.sf.net/nco.html#typ_cnv\n"); break;
+  case NC_ENOTNC: (void)fprintf(stdout,"ERROR NC_ENOTNC Not a netCDF file\nHINT: NC_ENOTNC errors will occur when NCO operators linked to the netCDF3 library attempt to read netCDF4 files.  Are your input files netCDF4 format?  (http://nco.sf.net/nco.html#fmt_inq shows how to tell.) If so then installing or re-building a netCDF4-compatible version of NCO should solve this problem.\n"); break;
   } /* end switch */
 
   /* Print NCO-generated error message, if any */
@@ -290,16 +313,6 @@ nco_fmt_sng /* [fnc] Convert netCDF file format enum to string */
     return "NC_FORMAT_CLASSIC";
   case NC_FORMAT_64BIT:
     return "NC_FORMAT_64BIT";
-    /* NB: nco_netcdf.c does not #include nco.h which #defines forward-compatibility tokens
-       This barrier helps segregate NCO from wrappers
-       Re-define minimal sub-set of tokens in nco_netcdf.c as necessary
-       This small exception to the barrier */
-#ifndef NC_FORMAT_NETCDF4
-# define NC_FORMAT_NETCDF4 (3)
-#endif
-#ifndef NC_FORMAT_NETCDF4_CLASSIC
-# define NC_FORMAT_NETCDF4_CLASSIC  (4) /* create netcdf-4 files, with NC_CLASSIC_MODEL. */
-#endif
   case NC_FORMAT_NETCDF4:
     return "NC_FORMAT_NETCDF4";
   case NC_FORMAT_NETCDF4_CLASSIC:
@@ -371,8 +384,18 @@ nco_open(const char * const fl_nm,const int mode,int * const nc_id)
   return rcd;
 } /* end nco_open */
 
+int
+nco_open_flg(const char * const fl_nm,const int mode,int * const nc_id)
+{
+  /* Purpose: Error-tolerant wrapper for nc_open(). Tolerates all errors.
+     Currently used only by nco_fl_mk_lcl() to test file accessibility via DAP */
+  int rcd;
+  rcd=nc_open(fl_nm,mode,nc_id);
+  return rcd;
+} /* end nco_open */
+
 #if 0
-#ifdef ENABLE_NETCDF4
+#ifdef HAVE_NETCDF4_H
 /* netCDF4 routines defined by Unidata netCDF4 Library libnetcdf.a 
    20051129: nc_*_par() routines require certain netCDF4 configuration options */
 int 
@@ -385,7 +408,7 @@ nco_open_par(const char * const fl_nm,const int mode,MPI_Comm mpi_cmm,MPI_Info m
   if(rcd != NC_NOERR) nco_err_exit(rcd,fnc_nm);
   return rcd;
 } /* end nco_open_par */
-#endif /* !ENABLE_NETCDF4 */
+#endif /* !HAVE_NETCDF4_H */
 #endif /* !0 */
 
 #ifdef ENABLE_PNETCDF
@@ -490,15 +513,15 @@ nco_inq(const int nc_id,int * const dmn_nbr_fl,int * const var_nbr_fl,int * cons
   return rcd;
 } /* end nco_inq() */
 
-#if NEED_NC_INQ_FORMAT
+#ifdef NEED_NC_INQ_FORMAT
 int nc_inq_format(int nc_id, int * const fl_fmt)
 {
-  /* Purpose: Stub for nc_inq_format(), which appeared in netCDF 3.6.1 or 3.6.2 */
+  /* Purpose: Stub for nc_inq_format(), which appeared in netCDF 3.6.1 or 3.6.2
+     20070901 Current OPeNDAP does not have nc_inq_format() and thus requires this stub */
   *fl_fmt=NC_FORMAT_CLASSIC; /* [enm] Output file format */
   return NC_NOERR+0*nc_id; /* CEWI */
-} /* end nco_inq_format() */
+} /* end nc_inq_format() */
 #endif /* !NEED_NC_INQ_FORMAT */
-
 int
 nco_inq_format(const int nc_id,int * const fl_fmt)
 {
@@ -581,7 +604,7 @@ nco_inq_dimid(const int nc_id,const char * const dmn_nm,int * const dmn_id)
 int
 nco_inq_dimid_flg(const int nc_id,const char * const dmn_nm,int * const dmn_id)
 {
-  /* Purpose: Wrapper for nc_inq_dimid() which does not require success */
+  /* Purpose: Error-tolerant wrapper for nc_inq_dimid(). Tolerates NC_EBADDIM. */
   int rcd;
   rcd=nc_inq_dimid(nc_id,dmn_nm,dmn_id);
   if(rcd == NC_EBADDIM) return rcd;
@@ -602,7 +625,7 @@ nco_inq_dim(const int nc_id,const int dmn_id,char *dmn_nm,long *dmn_sz)
 int
 nco_inq_dim_flg(const int nc_id,const int dmn_id,char *dmn_nm,long *dmn_sz)
 {
-  /* Purpose: Wrapper for nc_inq_dim_flg() which does not require success */
+  /* Purpose: Error-tolerant wrapper for nc_inq_dim_flg(). Tolerates NC_EBADDIM. */
   int rcd;
   rcd=nc_inq_dim(nc_id,dmn_id,dmn_nm,(size_t *)dmn_sz);
   if(rcd == NC_EBADDIM) return rcd;
@@ -642,6 +665,16 @@ nco_rename_dim(const int nc_id,const int dmn_id,const char * const dmn_nm)
 /* End Dimension routines */
 
 /* Begin Variable routines (_var) */
+int
+nco_copy_var(const int nc_in_id,const int var_id,const int nc_out_id)
+{
+  /* Purpose: Wrapper for nc_copy_var() */
+  int rcd;
+  rcd=nc_copy_var(nc_in_id,var_id,nc_out_id);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_copy_var()");
+  return rcd;
+} /* end nco_copy_var */
+
 int
 nco_def_var(const int nc_id,const char * const var_nm,const nc_type var_typ,const int dmn_nbr,const int * const dmn_id,int * const var_id)
 {
@@ -652,6 +685,43 @@ nco_def_var(const int nc_id,const char * const var_nm,const nc_type var_typ,cons
   return rcd;
 } /* end nco_def_var */
 
+int nco_def_var_chunking
+(const int nc_id, /* [ID] netCDF ID */
+ const int var_id, /* [ID] Variable ID */
+ const int srg_typ, /* [enm] Storage type */
+ const size_t * const cnk_sz) /* [nbr] Chunk sizes */
+{
+  /* Purpose: Wrapper for nc_def_var_chunking() */
+  int rcd;
+  /* NB: 20090713: netCDF4 API for nc_def_var_chunking() changed ~200906 
+     Before that a weak netCDF4 prototype did not make cnk_sz const
+     After I notified them of this, Unidata changed prototype to 
+     const size_t * const cnk_sz
+     This API change may cause confusion/differences when compiling
+     NCO with netCDF4 versions pre- and post-200906, e.g., 
+     netcdf-4.1-beta1-snapshot2009050200 has old behavior while
+     netcdf-4.1-beta1-snapshot2009071200 has new behavior.
+     Finding one size fits all method may be difficult! */
+  /*  rcd=nc_def_var_chunking(nc_id,var_id,srg_typ,cnk_sz);*/
+  rcd=nc_def_var_chunking(nc_id,var_id,srg_typ,(size_t *)cnk_sz);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_def_var_chunking()");
+  return rcd;
+} /* end nco_def_var_chunking() */
+
+int nco_def_var_deflate
+(const int nc_id, /* [ID] netCDF ID */
+ const int var_id, /* [ID] Variable ID */
+ const int shuffle, /* [flg] Turn on shuffle filter */
+ const int deflate, /* [flg] Turn on deflate filter */
+ const int dfl_lvl) /* [enm] Deflate level [0..9] */
+{
+  /* Purpose: Wrapper for nc_def_var_deflate() */
+  int rcd;
+  rcd=nc_def_var_deflate(nc_id,var_id,shuffle,deflate,dfl_lvl);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_def_var_deflate()");
+  return rcd;
+} /* end nco_def_var_deflate() */
+
 int
 nco_inq_var(const int nc_id,const int var_id,char * const var_nm,nc_type *var_typ,int * const dmn_nbr,int * const dmn_id,int * const nbr_att)
 {
@@ -662,6 +732,53 @@ nco_inq_var(const int nc_id,const int var_id,char * const var_nm,nc_type *var_ty
   return rcd;
 } /* end nco_inq_var */
 
+int nco_inq_var_chunking
+(const int nc_id, /* [ID] netCDF ID */
+ const int var_id, /* [ID] Variable ID */
+ int * const srg_typ, /* [enm] Storage type */
+ size_t * const cnk_sz) /* [nbr] Chunk sizes */
+{
+  /* Purpose: Wrapper for nc_inq_var_chunking() */
+  /* NB: netCDF chunking inquire function only works on netCDF4 files
+     NCO wrapper works on netCDF3 and netCDF4 files */
+  int rcd;
+  int fl_fmt; /* [enm] Input file format */
+  rcd=nco_inq_format(nc_id,&fl_fmt);
+  if(fl_fmt == NC_FORMAT_NETCDF4 || fl_fmt == NC_FORMAT_NETCDF4_CLASSIC){
+    rcd=nc_inq_var_chunking(nc_id,var_id,srg_typ,cnk_sz);
+  }else{ /* !netCDF4 */
+    *srg_typ=NC_CONTIGUOUS;
+    *cnk_sz=(size_t)NULL;
+  } /* !netCDF4 */
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_var_chunking()");
+  return rcd;
+} /* end nco_inq_var_chunking() */
+
+int
+nco_inq_var_deflate
+(const int nc_id, /* I [ID] netCDF ID */
+ const int var_id, /* I [ID] Variable ID */
+ int * const shuffle, /* O [flg] Turn on shuffle filter */
+ int * const deflate, /* O [flg] Turn on deflate filter */
+ int * const dfl_lvl) /* O [enm] Deflate level [0..9] */
+{
+  /* Purpose: Wrapper for nc_inq_var_deflate() */
+  /* NB: netCDF deflate inquire function only works on netCDF4 files
+     NCO wrapper works on netCDF3 and netCDF4 files */
+  int rcd;
+  int fl_fmt; /* [enm] Input file format */
+  rcd=nco_inq_format(nc_id,&fl_fmt);
+  if(fl_fmt == NC_FORMAT_NETCDF4 || fl_fmt == NC_FORMAT_NETCDF4_CLASSIC){
+    rcd=nc_inq_var_deflate(nc_id,var_id,shuffle,deflate,dfl_lvl);
+  }else{ /* !netCDF4 */
+    *shuffle=0;
+    *deflate=0;
+    *dfl_lvl=0;
+  } /* !netCDF4 */
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_var_deflate()");
+  return rcd;
+} /* end nco_inq_var_deflate */
+
 int
 nco_inq_varid(const int nc_id,const char * const var_nm,int * const var_id)
 {
@@ -677,7 +794,7 @@ nco_inq_varid(const int nc_id,const char * const var_nm,int * const var_id)
 int
 nco_inq_varid_flg(const int nc_id,const char * const var_nm,int * const var_id)
 {
-  /* Purpose: Wrapper for nc_inq_varid_flg() which does not require success */
+  /* Purpose: Error-tolerant wrapper for nc_inq_varid_flg(). Tolerates NC_ENOTVAR. */
   int rcd;
   rcd=nc_inq_varid(nc_id,var_nm,var_id);
   if(rcd == NC_ENOTVAR) return rcd;
@@ -745,16 +862,6 @@ nco_rename_var(const int nc_id,const int var_id,const char * const var_nm)
   return rcd;
 } /* end nco_rename_var */
 
-int
-nco_copy_var(const int nc_in_id,const int var_id,const int nc_out_id)
-{
-  /* Purpose: Wrapper for nc_copy_var() */
-  int rcd;
-  rcd=nc_copy_var(nc_in_id,var_id,nc_out_id);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_copy_var()");
-  return rcd;
-} /* end nco_copy_var */
-
 /* End _var */
 
 /* Start _get _put _var */
@@ -776,9 +883,7 @@ nco_get_var1(const int nc_id,const int var_id,const long * const srt,void * cons
   case NC_UINT: rcd=NCO_GET_VAR1_UINT(nc_id,var_id,(const size_t *)srt,(nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_GET_VAR1_INT64(nc_id,var_id,(const size_t *)srt,(nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_GET_VAR1_UINT64(nc_id,var_id,(const size_t *)srt,(nco_uint64 *)vp); break;
-#if 0
   case NC_STRING: rcd=NCO_GET_VAR1_STRING(nc_id,var_id,(const size_t *)srt,(nco_string *)vp); break;
-#endif /* !0 */
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
@@ -804,9 +909,18 @@ nco_put_var1(const int nc_id,const int var_id,const long * const srt,const void
   case NC_UINT: rcd=NCO_PUT_VAR1_UINT(nc_id,var_id,(const size_t *)srt,(const nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_PUT_VAR1_INT64(nc_id,var_id,(const size_t *)srt,(const nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_PUT_VAR1_UINT64(nc_id,var_id,(const size_t *)srt,(const nco_uint64 *)vp); break;
-#if 0
-  case NC_STRING: rcd=NCO_PUT_VAR1_STRING(nc_id,var_id,(const size_t *)srt,(const nco_string *)vp); break;
-#endif /* !0 */
+    /* Next line produces GCC warning:
+       attention : passing argument 4 of ‘nc_put_var1_string’ from incompatible pointer type 
+       I think this warning occurs because this routine receives the vp input 
+       as "const void * const vp", a singly indirect pointer, and then
+       casts vp to a doubly indirect pointer on output, i.e., to something like
+       "(const char **)vp" or "(const nco_string * const)vp".
+       Is casting a singly-indirect pointer to a doubly-indirect ever allowed? */
+    /*  case NC_STRING: rcd=NCO_PUT_VAR1_STRING(nc_id,var_id,(const size_t *)srt,(const char **)vp); break;*/
+  case NC_STRING: rcd=NCO_PUT_VAR1_STRING(nc_id,var_id,(const size_t *)srt,(const nco_string * const)vp); break;
+    /* Next line produces GCC warning:
+       attention : le transtypage annule des qualificateurs du type pointeur ciblé */
+    /*  case NC_STRING: rcd=NCO_PUT_VAR1_STRING(nc_id,var_id,(const size_t *)srt,(const char **)vp); break;*/
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
@@ -832,9 +946,7 @@ nco_get_vara(const int nc_id,const int var_id,const long * const srt,const long
   case NC_UINT: rcd=NCO_GET_VARA_UINT(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_GET_VARA_INT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_GET_VARA_UINT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(nco_uint64 *)vp); break;
-#if 0
   case NC_STRING: rcd=NCO_GET_VARA_STRING(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(nco_string *)vp); break;
-#endif /* !0 */
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
@@ -860,9 +972,7 @@ nco_put_vara(const int nc_id,const int var_id,const long * const srt,const long
   case NC_UINT: rcd=NCO_PUT_VARA_UINT(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_PUT_VARA_INT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_PUT_VARA_UINT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const nco_uint64 *)vp); break;
-#if 0
   case NC_STRING: rcd=NCO_PUT_VARA_STRING(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const nco_string *)vp); break;
-#endif /* !0 */
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
@@ -888,9 +998,7 @@ nco_get_vars(const int nc_id,const int var_id,const long * const srt,const long
   case NC_UINT: rcd=NCO_GET_VARS_UINT(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_GET_VARS_INT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_GET_VARS_UINT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(nco_uint64 *)vp); break;
-#if 0
   case NC_STRING: rcd=NCO_GET_VARS_STRING(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(nco_string *)vp); break;
-#endif /* !0 */
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
@@ -916,9 +1024,7 @@ nco_put_vars(const int nc_id,const int var_id,const long * const srt,const long
   case NC_UINT: rcd=NCO_PUT_VARS_UINT(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd, (const nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_PUT_VARS_INT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd, (const nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_PUT_VARS_UINT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const nco_uint64 *)vp); break;
-#if 0
   case NC_STRING: rcd=NCO_PUT_VARS_STRING(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const nco_string *)vp); break;
-#endif /* !0 */
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
@@ -945,9 +1051,7 @@ nco_get_varm(const int nc_id,const int var_id,const long * const srt,const long
   case NC_UINT: rcd=NCO_GET_VARM_UINT(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_GET_VARM_INT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_GET_VARM_UINT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(nco_uint64 *)vp); break;
-#if 0
   case NC_STRING: rcd=NCO_GET_VARM_STRING(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(nco_string *)vp); break;
-#endif /* !0 */
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
@@ -973,9 +1077,7 @@ nco_put_varm(const int nc_id,const int var_id,const long * const srt,const long
   case NC_UINT: rcd=NCO_PUT_VARM_UINT(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(const nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_PUT_VARM_INT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(const nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_PUT_VARM_UINT64(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(const nco_uint64 *)vp); break;
-#if 0
   case NC_STRING: rcd=NCO_PUT_VARM_STRING(nc_id,var_id,(const size_t *)srt,(const size_t *)cnt,(const ptrdiff_t *)srd,(const ptrdiff_t *)map,(const nco_string *)vp); break;
-#endif /* !0 */
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
@@ -1002,7 +1104,7 @@ nco_inq_att(const int nc_id,const int var_id,const char * const att_nm,nc_type *
 int
 nco_inq_att_flg(const int nc_id,const int var_id,const char * const att_nm,nc_type * const att_typ,long * const att_sz)
 {
-  /* Purpose: Wrapper for nc_inq_att() */
+  /* Purpose: Error-tolerant wrapper for nc_inq_att(). Tolerates ENOTATT. */
   const char fnc_nm[]="nco_inq_att_flg()";
   int rcd;
   rcd=nc_inq_att(nc_id,var_id,att_nm,att_typ,(size_t *)att_sz);
@@ -1027,7 +1129,7 @@ nco_inq_attid(const int nc_id,const int var_id,const char * const att_nm,int * c
 int
 nco_inq_attid_flg(const int nc_id,const int var_id,const char * const att_nm,int * const att_id)
 {
-  /* Purpose: Wrapper for nc_inq_attid() */
+  /* Purpose: Error-tolerant wrapper for nc_inq_attid(): Tolerates NC_ENOTATT. */
   const char fnc_nm[]="nco_inq_attid_flg()";
   int rcd;
   rcd=nc_inq_attid(nc_id,var_id,att_nm,att_id);
@@ -1059,6 +1161,18 @@ nco_inq_attlen(const int nc_id,const int var_id,const char * const att_nm,long *
   return rcd;
 } /* end nco_inq_attlen */
 
+int
+nco_inq_attlen_flg(const int nc_id,const int var_id,const char * const att_nm,long * const att_sz)
+{
+  /* Purpose: Error-tolerant wrapper for nc_inq_attlen(). Tolerates NC_ENOTATT. */
+  const char fnc_nm[]="nco_inq_attlen_flg()";
+  int rcd;
+  rcd=nc_inq_attlen(nc_id,var_id,att_nm,(size_t *)att_sz);
+  if(rcd == NC_ENOTATT) return rcd;
+  if(rcd != NC_NOERR) nco_err_exit(rcd,fnc_nm);
+  return rcd;
+} /* end nco_inq_attlen */
+
 int
 nco_inq_attname(const int nc_id,const int var_id,const int att_id,char * const att_nm)
 {
@@ -1075,7 +1189,7 @@ nco_copy_att(const int nc_id_in,const int var_id_in,const char * const att_nm,co
   /* Purpose: Wrapper for nc_copy_att() */
   int rcd;
   rcd=nc_copy_att(nc_id_in,var_id_in,att_nm,nc_id_out,var_id_out);
-  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_copy_att()");
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_copy_att()");
   return rcd;
 }  /* end nco_copy_att */
 
@@ -1112,15 +1226,13 @@ nco_put_att(const int nc_id,const int var_id,const char * const att_nm,const nc_
   case NC_CHAR: rcd=NCO_PUT_ATT_CHAR(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_char *)vp); break;
   case NC_BYTE: rcd=NCO_PUT_ATT_BYTE(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_byte *)vp); break;
 #ifdef ENABLE_NETCDF4
-    /* 20051119: netcdf4 library did not support these until alpha10, still does not support nco_put/get_att_ubyte() */
-    /*  case NC_UBYTE: rcd=NCO_PUT_ATT_UBYTE(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_ubyte *)vp); break;*/
+    /* 20051119: netCDF4 library did not support these until alpha10, still does not support nco_put/get_att_ubyte() */
+  case NC_UBYTE: rcd=NCO_PUT_ATT_UBYTE(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_ubyte *)vp); break;
   case NC_USHORT: rcd=NCO_PUT_ATT_USHORT(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_ushort *)vp); break;
   case NC_UINT: rcd=NCO_PUT_ATT_UINT(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_PUT_ATT_INT64(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_PUT_ATT_UINT64(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_uint64 *)vp); break;
-#if 0
   case NC_STRING: rcd=NCO_PUT_ATT_STRING(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(const nco_string *)vp); break;
-#endif /* !0 */
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
@@ -1142,14 +1254,12 @@ nco_get_att(const int nc_id,const int var_id,const char * const att_nm,void * co
   case NC_BYTE: rcd=NCO_GET_ATT_BYTE(nc_id,var_id,att_nm,(nco_byte *)vp); break;
 #ifdef ENABLE_NETCDF4
     /* 20051119: netcdf4 library did not support these until alpha10, still does not support nco_put/get_att_ubyte() */
-    /*  case NC_UBYTE: rcd=NCO_GET_ATT_UBYTE(nc_id,var_id,att_nm,(nco_ubyte *)vp); break;*/
+  case NC_UBYTE: rcd=NCO_GET_ATT_UBYTE(nc_id,var_id,att_nm,(nco_ubyte *)vp); break;
   case NC_USHORT: rcd=NCO_GET_ATT_USHORT(nc_id,var_id,att_nm,(nco_ushort *)vp); break;
   case NC_UINT: rcd=NCO_GET_ATT_UINT(nc_id,var_id,att_nm,(nco_uint *)vp); break;
   case NC_INT64: rcd=NCO_GET_ATT_INT64(nc_id,var_id,att_nm,(nco_int64 *)vp); break;
   case NC_UINT64: rcd=NCO_GET_ATT_UINT64(nc_id,var_id,att_nm,(nco_uint64 *)vp); break;
-#if 0
   case NC_STRING: rcd=NCO_GET_ATT_STRING(nc_id,var_id,att_nm,(nco_string *)vp); break;
-#endif /* !0 */
 #endif /* !ENABLE_NETCDF4 */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
@@ -1159,15 +1269,26 @@ nco_get_att(const int nc_id,const int var_id,const char * const att_nm,void * co
 /* End Attribute routines */
 
 /* Begin netCDF4 stubs */
+#ifndef HAVE_NETCDF4_H
+/* NB: netCDF chunking/deflate define/inquire functions work only on netCDF4 files
+   NCO stubs perform no-ops on netCDF3 files */
+/* Newer, post-200906 netCDF4 API */
+int nc_def_var_chunking(const int nc_id,const int var_id,const int srg_typ,const size_t * const cnk_sz){return 1;}
+# if 0
+  /* Older, pre-200906 netCDF4 API */
+int nc_def_var_chunking(const int nc_id,const int var_id,const int srg_typ,size_t * const cnk_sz){return 1;}
+# endif /* !0 */
+int nc_inq_var_chunking(const int nc_id,const int var_id,int * const srg_typ,size_t *const cnk_sz){*srg_typ=(size_t)NC_CONTIGUOUS;*cnk_sz=(size_t)NULL;return 1;}
+int nc_def_var_deflate(const int nc_id,const int var_id,const int shuffle,const int deflate,const int dfl_lvl){return 1;}
+int nc_inq_var_deflate(const int nc_id,const int var_id,int * const shuffle, int * const deflate,int * const dfl_lvl){*shuffle=0;*deflate=0;*dfl_lvl=0;return 1;}
+#endif /* HAVE_NETCDF4_H */
 #ifndef ENABLE_NETCDF4
 int NCO_GET_VAR1_UBYTE(const int nc_id,const int var_id,const size_t *srt,nco_ubyte *ubp){return 1;}
-int NCO_GET_VAR1_USHORT(const int nc_id,const int var_id,const size_t *srt,nco_ubyte *usp){return 1;}
+int NCO_GET_VAR1_USHORT(const int nc_id,const int var_id,const size_t *srt,nco_ushort *usp){return 1;}
 int NCO_GET_VAR1_UINT(const int nc_id,const int var_id,const size_t *srt,nco_uint *uip){return 1;}
 int NCO_GET_VAR1_INT64(const int nc_id,const int var_id,const size_t *srt,nco_int64 *i64p){return 1;}
 int NCO_GET_VAR1_UINT64(const int nc_id,const int var_id,const size_t *srt,nco_uint64 *ui64p){return 1;}
-#if 0
 int NCO_GET_VAR1_STRING(const int nc_id,const int var_id,const size_t *srt,nco_string *sngp){return 1;}
-#endif /* !0 */
 #endif /* ENABLE_NETCDF4 */
 #ifndef ENABLE_NETCDF4
 int NCO_PUT_VAR1_UBYTE(const int nc_id,const int var_id,const size_t *srt,const nco_ubyte *ubp){return 1;}
@@ -1175,19 +1296,15 @@ int NCO_PUT_VAR1_USHORT(const int nc_id,const int var_id,const size_t *srt,const
 int NCO_PUT_VAR1_UINT(const int nc_id,const int var_id,const size_t *srt,const nco_uint *uip){return 1;}
 int NCO_PUT_VAR1_INT64(const int nc_id,const int var_id,const size_t *srt,const nco_int64 *i64p){return 1;}
 int NCO_PUT_VAR1_UINT64(const int nc_id,const int var_id,const size_t *srt,const nco_uint64 *ui64p){return 1;}
-#if 0
-int NCO_PUT_VAR1_STRING(const int nc_id,const int var_id,const size_t *srt,const nco_string *sngp){return 1;}
-#endif /* !0 */
+int NCO_PUT_VAR1_STRING(const int nc_id,const int var_id,const size_t *srt,const char **sngp){return 1;}
 #endif /* ENABLE_NETCDF4 */
 #ifndef ENABLE_NETCDF4
 int NCO_GET_VARA_UBYTE(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,nco_ubyte *ubp){return 1;}
-int NCO_GET_VARA_USHORT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,nco_ubyte *usp){return 1;}
+int NCO_GET_VARA_USHORT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,nco_ushort *usp){return 1;}
 int NCO_GET_VARA_UINT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,nco_uint *uip){return 1;}
 int NCO_GET_VARA_INT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,nco_int64 *i64p){return 1;}
 int NCO_GET_VARA_UINT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,nco_uint64 *ui64p){return 1;}
-#if 0
 int NCO_GET_VARA_STRING(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,nco_string *sngp){return 1;}
-#endif /* !0 */
 #endif /* ENABLE_NETCDF4 */
 #ifndef ENABLE_NETCDF4
 int NCO_PUT_VARA_UBYTE(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const nco_ubyte *ubp){return 1;}
@@ -1195,19 +1312,15 @@ int NCO_PUT_VARA_USHORT(const int nc_id,const int var_id,const size_t *srt,const
 int NCO_PUT_VARA_UINT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const nco_uint *uip){return 1;}
 int NCO_PUT_VARA_INT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const nco_int64 *i64p){return 1;}
 int NCO_PUT_VARA_UINT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const nco_uint64 *ui64p){return 1;}
-#if 0
-int NCO_PUT_VARA_STRING(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const nco_string *sngp){return 1;}
-#endif /* !0 */
+int NCO_PUT_VARA_STRING(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const char **sngp){return 1;}
 #endif /* ENABLE_NETCDF4 */
 #ifndef ENABLE_NETCDF4
 int NCO_GET_VARS_UBYTE(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,nco_ubyte *ubp){return 1;}
-int NCO_GET_VARS_USHORT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,nco_ubyte *usp){return 1;}
+int NCO_GET_VARS_USHORT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,nco_ushort *usp){return 1;}
 int NCO_GET_VARS_UINT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,nco_uint *uip){return 1;}
 int NCO_GET_VARS_INT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,nco_int64 *i64p){return 1;}
 int NCO_GET_VARS_UINT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,nco_uint64 *ui64p){return 1;}
-#if 0
 int NCO_GET_VARS_STRING(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,nco_string *sngp){return 1;}
-#endif /* !0 */
 #endif /* ENABLE_NETCDF4 */
 #ifndef ENABLE_NETCDF4
 int NCO_PUT_VARS_UBYTE(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const nco_ubyte *ubp){return 1;}
@@ -1215,19 +1328,15 @@ int NCO_PUT_VARS_USHORT(const int nc_id,const int var_id,const size_t *srt,const
 int NCO_PUT_VARS_UINT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd, const nco_uint *uip){return 1;}
 int NCO_PUT_VARS_INT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd, const nco_int64 *i64p){return 1;}
 int NCO_PUT_VARS_UINT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const nco_uint64 *ui64p){return 1;}
-#if 0
-int NCO_PUT_VARS_STRING(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const nco_string *sngp){return 1;}
-#endif /* !0 */
+int NCO_PUT_VARS_STRING(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const char **sngp){return 1;}
 #endif /* ENABLE_NETCDF4 */
 #ifndef ENABLE_NETCDF4
 int NCO_GET_VARM_UBYTE(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,nco_ubyte *ubp){return 1;}
-int NCO_GET_VARM_USHORT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,nco_ubyte *usp){return 1;}
+int NCO_GET_VARM_USHORT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,nco_ushort *usp){return 1;}
 int NCO_GET_VARM_UINT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,nco_uint *uip){return 1;}
 int NCO_GET_VARM_INT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,nco_int64 *i64p){return 1;}
 int NCO_GET_VARM_UINT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,nco_uint64 *ui64p){return 1;}
-#if 0
 int NCO_GET_VARM_STRING(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,nco_string *sngp){return 1;}
-#endif /* !0 */
 #endif /* ENABLE_NETCDF4 */
 #ifndef ENABLE_NETCDF4
 int NCO_PUT_VARM_UBYTE(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,const nco_ubyte *ubp){return 1;}
@@ -1235,9 +1344,7 @@ int NCO_PUT_VARM_USHORT(const int nc_id,const int var_id,const size_t *srt,const
 int NCO_PUT_VARM_UINT(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,const nco_uint *uip){return 1;}
 int NCO_PUT_VARM_INT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,const nco_int64 *i64p){return 1;}
 int NCO_PUT_VARM_UINT64(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,const nco_uint64 *ui64p){return 1;}
-#if 0
-int NCO_PUT_VARM_STRING(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,const nco_string *sngp){return 1;}
-#endif /* !0 */
+int NCO_PUT_VARM_STRING(const int nc_id,const int var_id,const size_t *srt,const size_t *cnt,const ptrdiff_t *srd,const ptrdiff_t *map,const char **sngp){return 1;}
 #endif /* ENABLE_NETCDF4 */
 #ifndef ENABLE_NETCDF4
 /* 20051119: netcdf4 library did not support these until alpha10, still does not support nco_put/get_att_ubyte() */
@@ -1246,19 +1353,15 @@ int NCO_PUT_ATT_USHORT(const int nc_id,const int var_id,const char *att_nm,const
 int NCO_PUT_ATT_UINT(const int nc_id,const int var_id,const char *att_nm,const nc_type att_typ,size_t att_len,const nco_uint *uip){return 1;}
 int NCO_PUT_ATT_INT64(const int nc_id,const int var_id,const char *att_nm,const nc_type att_typ,size_t att_len,const nco_int64 *i64p){return 1;}
 int NCO_PUT_ATT_UINT64(const int nc_id,const int var_id,const char *att_nm,const nc_type att_typ,size_t att_len,const nco_uint64 *ui64p){return 1;}
-#if 0
-int NCO_PUT_ATT_STRING(const int nc_id,const int var_id,const char *att_nm,const nc_type att_typ,size_t att_len,const nco_string *sngp){return 1;}
-#endif /* !0 */
+int NCO_PUT_ATT_STRING(const int nc_id,const int var_id,const char *att_nm,const nc_type att_typ,size_t att_len,const char **sngp){return 1;}
 #endif /* ENABLE_NETCDF4 */
 #ifndef ENABLE_NETCDF4
 /* 20051119: netcdf4 library did not support these until alpha10, still does not support nco_put/get_att_ubyte() */
 int NCO_GET_ATT_UBYTE(const int nc_id,const int var_id,const char *att_nm,nco_ubyte *ubp){return 1;}
-int NCO_GET_ATT_USHORT(const int nc_id,const int var_id,const char *att_nm,nco_ubyte *usp){return 1;}
+int NCO_GET_ATT_USHORT(const int nc_id,const int var_id,const char *att_nm,nco_ushort *usp){return 1;}
 int NCO_GET_ATT_UINT(const int nc_id,const int var_id,const char *att_nm,nco_uint *uip){return 1;}
 int NCO_GET_ATT_INT64(const int nc_id,const int var_id,const char *att_nm,nco_int64 *i64p){return 1;}
 int NCO_GET_ATT_UINT64(const int nc_id,const int var_id,const char *att_nm,nco_uint64 *ui64p){return 1;}
-#if 0
 int NCO_GET_ATT_STRING(const int nc_id,const int var_id,const char *att_nm,nco_string *sngp){return 1;}
-#endif /* !0 */
 #endif /* ENABLE_NETCDF4 */
 /* end netCDF4 stubs */
