@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_prn.c,v 1.41 2007/05/25 17:36:47 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_prn.c,v 1.56 2009/05/26 05:29:04 zender Exp $ */
 
 /* Purpose: Printing variables, attributes, metadata */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_prn.h" /* Printing variables, attributes, metadata */
@@ -64,16 +64,16 @@ nco_prn_att /* [fnc] Print all attributes of single variable */
     (void)sprintf(att_sng,"%s%%s",nco_typ_fmt_sng(att[idx].type));
     switch(att[idx].type){
     case NC_FLOAT:
-      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.fp[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.fp[att_lmn],(att_lmn != att_sz-1L) ? dlm_sng : "");
       break;
     case NC_DOUBLE:
-      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.dp[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.dp[att_lmn],(att_lmn != att_sz-1L) ? dlm_sng : "");
       break;
     case NC_SHORT:
-      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.sp[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.sp[att_lmn],(att_lmn != att_sz-1L) ? dlm_sng : "");
       break;
     case NC_INT:
-      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,(long)att[idx].val.lp[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,(long)att[idx].val.lp[att_lmn],(att_lmn != att_sz-1L) ? dlm_sng : "");
       break;
     case NC_CHAR:
       for(att_lmn=0;att_lmn<att_sz;att_lmn++){
@@ -83,25 +83,28 @@ nco_prn_att /* [fnc] Print all attributes of single variable */
       } /* end loop over element */
       break;
     case NC_BYTE:
-      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.bp[att_lmn]);
+      /*      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.bp[att_lmn]);*/
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++){
+	(void)fprintf(stdout,att_sng,((signed char *)att[idx].val.bp)[att_lmn]);
+      }
       break;
     case NC_UBYTE:
       for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.ubp[att_lmn]);
       break;
     case NC_USHORT:
-      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.usp[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.usp[att_lmn],(att_lmn != att_sz-1L) ? dlm_sng : "");
       break;
     case NC_UINT:
-      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.uip[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.uip[att_lmn],(att_lmn != att_sz-1L) ? dlm_sng : "");
       break;
     case NC_INT64:
-      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.i64p[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.i64p[att_lmn],(att_lmn != att_sz-1L) ? dlm_sng : "");
       break;
     case NC_UINT64:
-      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.ui64p[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.ui64p[att_lmn],(att_lmn != att_sz-1L) ? dlm_sng : "");
       break;
     case NC_STRING:
-      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.sngp[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.sngp[att_lmn],(att_lmn != att_sz-1L) ? dlm_sng : "");
       break;
     default: nco_dfl_case_nc_type_err();
       break;
@@ -134,10 +137,11 @@ nco_typ_fmt_sng /* [fnc] Provide sprintf() format string for specified type */
   static const char fmt_NC_SHORT[]="%hi";
   static const char fmt_NC_CHAR[]="%c";
   /* Formats useful in printing byte data as decimal notation */
-  static const char fmt_NC_BYTE[]="%i";
-  /* static const char fmt_NC_BYTE[]="%c"; */
+  /*  static const char fmt_NC_BYTE[]="%i";*/
+  /*  static const char fmt_NC_BYTE[]="%c"; */
+  static const char fmt_NC_BYTE[]="%d";
   /* NB: %hhi is GNU extension, not ANSI standard */
-  /* static const char fmt_NC_BYTE[]="%hhi";*/ /* Takes signed char as arg and prints 0..255 (unfortunately) */
+  /* static const char fmt_NC_BYTE[]="%hhi"; *//* Takes signed char as arg and prints 0..255 (unfortunately) */
   /* static const char fmt_NC_BYTE[]="%hhu";*/ /* Takes unsigned char as arg and prints 0..255 */
 
   static const char fmt_NC_UBYTE[]="%hhu"; /*  */
@@ -187,12 +191,17 @@ nco_prn_var_dfn /* [fnc] Print variable metadata */
   /* Purpose: Print variable metadata. This routine does not take into 
      account any user-specified limits, it just prints what it finds. */
   int *dmn_id=NULL_CEWI;
+  size_t *cnk_sz=NULL_CEWI; /* [nbr] Chunk sizes */
   int idx;
   int nbr_dim;
   int nbr_att;
   int rcd=NC_NOERR; /* [rcd] Return code */
   int rec_dmn_id;
   int var_id;
+  int srg_typ; /* [enm] Storage type */
+  int shuffle; /* [flg] Shuffling is on */
+  int deflate; /* [flg] Deflation is on */
+  int dfl_lvl; /* [enm] Deflate level [0..9] */
   
   nc_type var_typ;
   
@@ -204,20 +213,22 @@ nco_prn_var_dfn /* [fnc] Print variable metadata */
   /* Get number of dimensions, type, and number of attributes for variable */
   (void)nco_inq_var(in_id,var_id,(char *)NULL,&var_typ,&nbr_dim,(int *)NULL,&nbr_att);
 
-  /* Get the ID of the record dimension, if any */
+  /* Get record dimension ID, if any */
   (void)nco_inq(in_id,(int *)NULL,(int *)NULL,(int *)NULL,&rec_dmn_id);
 
-  /* Print the header for variable */
+  /* Print header for variable */
   (void)fprintf(stdout,"%s: # dim. = %i, %s, # att. = %i, ID = %i\n",var_nm,nbr_dim,nco_typ_sng(var_typ),nbr_att,var_id);
 
   if(nbr_dim > 0){
     /* Allocate space for dimension info */
+    cnk_sz=(size_t *)nco_malloc(nbr_dim*sizeof(size_t)); /* [nbr] Chunk sizes */
     dim=(dmn_sct *)nco_malloc(nbr_dim*sizeof(dmn_sct));
     dmn_id=(int *)nco_malloc(nbr_dim*sizeof(int));
   } /* end if nbr_dim > 0 */
   
   /* Get dimension IDs */
   (void)nco_inq_vardimid(in_id,var_id,dmn_id);
+  rcd=nco_inq_var_chunking(in_id,var_id,&srg_typ,cnk_sz);
   
   /* Get dimension sizes and names */
   for(idx=0;idx<nbr_dim;idx++){
@@ -229,18 +240,18 @@ nco_prn_var_dfn /* [fnc] Print variable metadata */
     /* Is dimension a coordinate, i.e., stored as a variable? */
     rcd=nco_inq_varid_flg(in_id,dim[idx].nm,&dim[idx].cid);
     if(rcd == NC_NOERR){
-      /* Find out what type of variable the coordinate is */
+      /* Which type of variable is the coordinate? */
       (void)nco_inq_vartype(in_id,dim[idx].cid,&dim[idx].type);
-      (void)fprintf(stdout,"%s dimension %i: %s, size = %li %s, dim. ID = %d (CRD)",var_nm,idx,dim[idx].nm,dim[idx].sz,nco_typ_sng(dim[idx].type),dim[idx].id);
+      if(srg_typ == NC_CHUNKED) (void)fprintf(stdout,"%s dimension %i: %s, size = %li %s, chunksize = %li, dim. ID = %d (CRD)",var_nm,idx,dim[idx].nm,dim[idx].sz,nco_typ_sng(dim[idx].type),cnk_sz[idx],dim[idx].id); else (void)fprintf(stdout,"%s dimension %i: %s, size = %li %s, dim. ID = %d (CRD)",var_nm,idx,dim[idx].nm,dim[idx].sz,nco_typ_sng(dim[idx].type),dim[idx].id);
     }else{
-      (void)fprintf(stdout,"%s dimension %i: %s, size = %li, dim. ID = %d",var_nm,idx,dim[idx].nm,dim[idx].sz,dim[idx].id);
+      if(srg_typ == NC_CHUNKED) (void)fprintf(stdout,"%s dimension %i: %s, size = %li, chunksize = %li, dim. ID = %d",var_nm,idx,dim[idx].nm,dim[idx].sz,cnk_sz[idx],dim[idx].id); else (void)fprintf(stdout,"%s dimension %i: %s, size = %li, dim. ID = %d",var_nm,idx,dim[idx].nm,dim[idx].sz,dim[idx].id);
     } /* end else */
     if(dim[idx].id == rec_dmn_id) (void)fprintf(stdout,"(REC)"); 
     (void)fprintf(stdout,"\n"); 
     
   } /* end loop over dim */
   
-  /* Find the total size of variable array */
+  /* Find total size of variable array */
   if(nbr_dim>0){
     long var_sz=1L;
     char sz_sng[100];
@@ -254,17 +265,23 @@ nco_prn_var_dfn /* [fnc] Print variable metadata */
     } /* end loop over dim */
     (void)sprintf(sng_foo,"%li*nco_typ_lng(%s)",dim[idx].sz,nco_typ_sng(var_typ));
     (void)strcat(sz_sng,sng_foo);
-    (void)fprintf(stdout,"%s memory size is %s = %li*%lu = %lu bytes\n",var_nm,sz_sng,var_sz,(unsigned long)nco_typ_lng(var_typ),(unsigned long)(var_sz*nco_typ_lng(var_typ)));
+    /* NB: netCDF chunking/deflate define/inquire functions only work with netCDF4
+       NCO wrappers perform no-ops on netCDF3 files */
+    rcd=nco_inq_var_deflate(in_id,var_id,&shuffle,&deflate,&dfl_lvl);
+    if(deflate) (void)fprintf(stdout,"%s stored compressed (Lempel-Ziv %s shuffling) at level = %d\n",var_nm,(shuffle) ? "with" : "without",dfl_lvl);
+    (void)fprintf(stdout,"%s RAM size is %s = %li*%lu = %lu bytes\n",var_nm,sz_sng,var_sz,(unsigned long)nco_typ_lng(var_typ),(unsigned long)(var_sz*nco_typ_lng(var_typ)));
   }else{
     long var_sz=1L;
 
-    (void)fprintf(stdout,"%s memory size is %ld*nco_typ_lng(%s) = %ld*%lu = %lu bytes\n",var_nm,var_sz,nco_typ_sng(var_typ),var_sz,(unsigned long)nco_typ_lng(var_typ),(unsigned long)(var_sz*nco_typ_lng(var_typ)));
+    (void)fprintf(stdout,"%s RAM size is %ld*nco_typ_lng(%s) = %ld*%lu = %lu bytes\n",var_nm,var_sz,nco_typ_sng(var_typ),var_sz,(unsigned long)nco_typ_lng(var_typ),(unsigned long)(var_sz*nco_typ_lng(var_typ)));
   } /* end if variable is scalar */
+  if((nc_type)var_typ == NC_STRING) (void)fprintf(stdout,"%s RAM size above is space required for pointers only, full size of strings is unknown until data are read\n",var_nm);
   (void)fflush(stdout);
   
   /* Free space allocated for dimension information */
   for(idx=0;idx<nbr_dim;idx++) dim[idx].nm=(char *)nco_free(dim[idx].nm);
   if(nbr_dim > 0){
+    cnk_sz=(size_t *)nco_free(cnk_sz); /* [nbr] Chunk sizes */
     dim=(dmn_sct *)nco_free(dim);
     dmn_id=(int *)nco_free(dmn_id);
   } /* end if nbr_dim > 0*/
