@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_flt.c,v 1.17 2007/02/23 21:59:31 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_flt.c,v 1.24 2009/04/19 23:17:04 zender Exp $ */
 
 /* Purpose: Float-precision arithmetic */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_rth_flt.h" /* Float-precision arithmetic */
@@ -36,6 +36,9 @@ float asinhf(float x){return (float)(asinh((double)x));}
 #ifdef NEED_ATANF
 float atanf(float x){return (float)(atan((double)x));}
 #endif /* !NEED_ATANF */ 
+#ifdef NEED_ATAN2F
+float atan2f(float x,float y){return (float)(atan2((double)x,(double)y));}
+#endif /* !NEED_ATAN2F */ 
 #ifdef NEED_ATANHF
 float atanhf(float x){return (float)(atanh((double)x));}
 #endif /* !NEED_ATANHF */ 
