@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_utl.c,v 1.30 2007/05/19 00:02:10 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_utl.c,v 1.35 2009/05/04 00:04:37 zender Exp $ */
 
 /* Purpose: Arithmetic controls and utilities */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_rth_utl.h" /* Arithmetic controls and utilities */
@@ -30,7 +30,7 @@ nco_rth_prc_rnk /* [fnc] Rank precision of arithmetic type */
   } /* end switch */
 
   /* Some compilers, e.g., SGI cc, need return statement to end non-void functions */
-  return 0;
+  return (nco_rth_prc_rnk_enm)0;
 } /* end nco_rth_prc_rnk() */
 
 void 
