@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_scm.h,v 1.12 2007/02/23 21:59:31 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_scm.h,v 1.18 2009/04/19 23:17:04 zender Exp $ */
 
 /* Purpose: Software configuration management */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -24,6 +24,7 @@
 
 /* Personal headers */
 #include "nco_mmr.h" /* Memory management */
+#include "nco_sng_utl.h" /* String utilities */
 
 #ifdef __cplusplus
 extern "C" {
@@ -33,10 +34,13 @@ char * /* O [sng] CVS version string */
 cvs_vrs_prs(void); /* [fnc] Return CVS version string */
 
 void
-copyright_prn /* [fnc] Print copyright notice */
+nco_vrs_prn /* [fnc] Print NCO version */
 (const char * const CVS_Id, /* I [sng] CVS identification string */
  const char * const CVS_Revision); /* I [sng] CVS revision string */
 
+void
+nco_cpy_prn(void); /* [fnc] Print copyright notice */
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif /* __cplusplus */
