@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_typ.h,v 1.16 2007/05/18 22:37:46 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_typ.h,v 1.25 2009/07/12 19:42:53 zender Exp $ */
 
 /* Purpose: Type definitions, opaque types */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
@@ -27,8 +27,8 @@
    NCO uses types nco_byte, nco_char, and nco_int to handle netCDF internal
    types NC_BYTE, NC_CHAR, and NC_INT, respectively
    Most routines need access to opaque type definitions, so nco.h is appropriate 
-   However, in future it may be wiser to put opaque types in, e.g., nco_typ.h 
-   so that certain files, e.g., nco_netcdf.c, do not depend on rest of nco.h */
+   However, isolating opaque types definitions in nco_typ.h ensures that 
+   wrappers (e.g., nco_netcdf.[ch]) need not depend on nco.h. */
 
 /* byte, char, and int/long types are the most confusing part of netCDF:
    
@@ -39,14 +39,16 @@ byte: 8-bit signed or unsigned integers
 unsigned (0 to 255). However, when reading byte data to be converted into other 
 numeric types, it is interpreted as signed."
 
-NCO treats NC_BYTE as C-type "signed char".
-NCO treats NC_CHAR as C-type "char".
-C-Type "char" equals C-type "unsigned char" on most compilers/OSs
-NCO uses an opaque type nco_char to ease code portability 
+By default (and subject to modification by manipulating the tokens below), 
+  NCO treats NC_BYTE as C-type "signed char".
+  NCO treats NC_CHAR as C-type "char".
+  C-Type "char" equals C-type "unsigned char" on most compilers/OSs
+  NCO uses an opaque type nco_char to ease code portability 
 
-NCO reads/writes NC_BYTE using nc_put/get_var*_schar() functions
-NCO reads/writes NC_CHAR using nc_put/get_var*_text() functions
-NCO does not use nc_put/get_var*_uchar() functions for anything
+Therefore, by default,
+  NCO reads/writes NC_BYTE using nc_put/get_var*_schar() functions
+  NCO reads/writes NC_CHAR using nc_put/get_var*_text() functions
+  NCO does not use nc_put/get_var*_uchar() functions for anything
 
 netCDF manual p. 102:
 "The byte type differs from the char type in that it is intended for eight-bit data 
@@ -57,11 +59,12 @@ C code." */
 /* NCO_TYP_IO_FNC_MRG(x,y) function generates appropriate netCDF-layer I/O function 
    call for given I/O operation (x) and (possibly opaque) type (y)
    fxm TODO nco549: Automagically generate function names when called with, e g., 
-   NCO_TYP_IO_FNC_MRG(nc_get_var1,NCO_TYP_BYTE_IO_SFX) */
+   NCO_TYP_IO_FNC_MRG(nc_get_var1,NCO_TYP_BYTE_IO_SFX)
+   Unfortunately this pre-precessor macro has never worked... */
 #define NCO_TYP_IO_FNC_MRG(x,y) x##y
 
-  /* Define compatibility tokens when user does not have netCDF4 */
-#ifndef ENABLE_NETCDF4
+/* Define compatibility tokens when user does not have netCDF4 */
+#ifndef HAVE_NETCDF4_H
 /* Datatypes referenced in nco_typ.h, nco_netcdf.c: */
 #define	NC_UBYTE 	7	/* unsigned 1 byte int */
 #define	NC_USHORT 	8	/* unsigned 2-byte int */
@@ -73,17 +76,14 @@ C code." */
 #define NC_FILL_UBYTE   (255)
 #define NC_FILL_USHORT  (65535)
 #define NC_FILL_UINT    (4294967295U)
-#define NC_FILL_INT64   ((long long)-92233720)
-#define NC_FILL_UINT64  ((unsigned long long)18446744)
-#endif /* !ENABLE_NETCDF4 */
+/* NB: These fill values are one shy of min(int64) and max(uint64) */
+#define NC_FILL_INT64   ((long long)-9223372036854775806LL)
+#define NC_FILL_UINT64  ((unsigned long long)18446744073709551614ULL)
+#define NC_FILL_STRING  ""
+#endif /* HAVE_NETCDF4_H */
 
-/* fxm: Move inside ENABLE_NETCDF4 once netCDF4 netcdf.h finally defines this */
-#ifndef NC_FILL_STRING
-#define NC_FILL_STRING NULL
-#endif /* NC_FILL_STRING */
-
-/* C pre-processor can compare integers not strings
-   Enumerate integer values corresponding to each type and compare those instead */
+/* C pre-processor compares integers not strings
+   Perform comparisons on enumerated integer values corresponding to each type */
 #define NCO_TYP_CHAR 0
 #define NCO_TYP_SCHAR 1
 #define NCO_TYP_UCHAR 2
@@ -152,7 +152,7 @@ typedef unsigned char nco_byte; /* [typ] NC_BYTE */
 # define NCO_PUT_VARS_BYTE nc_put_vars_uchar
 # define NCO_PUT_VARM_BYTE nc_put_varm_uchar
 #else
-#error "ERROR: Unrecognized NCO_BYTE token"
+# error "ERROR: Unrecognized NCO_BYTE token"
 #endif /* NCO_BYTE */
 
 /* NC_UBYTE handling */
@@ -177,7 +177,7 @@ typedef unsigned char nco_ubyte; /* [typ] NC_UBYTE */
 # define NCO_PUT_VARS_UBYTE nc_put_vars_ubyte
 # define NCO_PUT_VARM_UBYTE nc_put_varm_ubyte
 #else
-#error "ERROR: Unrecognized NCO_UBYTE token"
+# error "ERROR: Unrecognized NCO_UBYTE token"
 #endif /* NCO_UBYTE */
 
 /* NC_CHAR handling */
@@ -232,7 +232,7 @@ typedef unsigned char nco_char; /* [typ] NC_CHAR */
 # define NCO_PUT_VARS_CHAR nc_put_vars_uchar
 # define NCO_PUT_VARM_CHAR nc_put_varm_uchar
 #else
-#error "ERROR: Unrecognized NCO_CHAR token"
+# error "ERROR: Unrecognized NCO_CHAR token"
 #endif /* NCO_CHAR */
 
 /* NC_USHORT handling */
@@ -257,7 +257,7 @@ typedef unsigned short nco_ushort; /* [typ] NC_USHORT */
 # define NCO_PUT_VARS_USHORT nc_put_vars_ushort
 # define NCO_PUT_VARM_USHORT nc_put_varm_ushort
 #else
-#error "ERROR: Unrecognized NCO_USHORT token"
+# error "ERROR: Unrecognized NCO_USHORT token"
 #endif /* NCO_USHORT */
 
 /* NC_INT handling */
@@ -297,7 +297,7 @@ typedef long nco_int; /* [typ] NC_INT */
 # define NCO_PUT_VARS_INT nc_put_vars_long
 # define NCO_PUT_VARM_INT nc_put_varm_long
 #else
-#error "ERROR: Unrecognized NCO_INT token"
+# error "ERROR: Unrecognized NCO_INT token"
 #endif /* NCO_INT */
 
 /* NC_UINT handling */
@@ -322,7 +322,7 @@ typedef unsigned int nco_uint; /* [typ] NC_UINT */
 # define NCO_PUT_VARS_UINT nc_put_vars_uint
 # define NCO_PUT_VARM_UINT nc_put_varm_uint
 #else
-#error "ERROR: Unrecognized NCO_UINT token"
+# error "ERROR: Unrecognized NCO_UINT token"
 #endif /* NCO_UINT */
 
 /* NC_INT64 handling */
@@ -347,7 +347,7 @@ typedef long long nco_int64; /* [typ] NC_INT64 */
 # define NCO_PUT_VARS_INT64 nc_put_vars_longlong
 # define NCO_PUT_VARM_INT64 nc_put_varm_longlong
 #else
-#error "ERROR: Unrecognized NCO_INT64 token"
+# error "ERROR: Unrecognized NCO_INT64 token"
 #endif /* NCO_INT64 */
 
 /* NC_UINT64 handling */
@@ -372,7 +372,7 @@ typedef unsigned long long nco_uint64; /* [typ] NC_UINT64 */
 # define NCO_PUT_VARS_UINT64 nc_put_vars_ulonglong
 # define NCO_PUT_VARM_UINT64 nc_put_varm_ulonglong
 #else
-#error "ERROR: Unrecognized NCO_UINT64 token"
+# error "ERROR: Unrecognized NCO_UINT64 token"
 #endif /* NCO_UINT64 */
 
 /* NC_STRING handling */
@@ -399,7 +399,7 @@ typedef char * nco_string; /* [typ] NC_STRING */
 # define NCO_PUT_VARS_STRING nc_put_vars_string
 # define NCO_PUT_VARM_STRING nc_put_varm_string
 #else
-#error "ERROR: Unrecognized NCO_STRING token"
+# error "ERROR: Unrecognized NCO_STRING token"
 #endif /* NCO_STRING */
 
 #ifdef __cplusplus
