@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_lst.c,v 1.68 2007/05/25 06:02:20 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_lst.c,v 1.83 2009/04/19 23:17:04 zender Exp $ */
 
 /* Purpose: Variable list utilities */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_var_lst.h" /* Variable list utilities */
@@ -67,7 +67,7 @@ nco_var_lst_mk /* [fnc] Create variable extraction list using regular expression
       /* ...and regular expression library is present */
 #ifdef NCO_HAVE_REGEX_FUNCTIONALITY
       rx_mch_nbr=nco_var_meta_search(nbr_var,fl_in_var_lst,var_sng,var_xtr_rqs);
-      if(rx_mch_nbr == 0) (void)fprintf(stdout,"%s: WARNING: Regular expression \"%s\" does not match any variables\nHINT: http://nco.sf.net/nco.html#rx",prg_nm_get(),var_sng); 
+      if(rx_mch_nbr == 0) (void)fprintf(stdout,"%s: WARNING: Regular expression \"%s\" does not match any variables\nHINT: See regular expression syntax examples at http://nco.sf.net/nco.html#rx\n",prg_nm_get(),var_sng); 
       continue;
 #else
       (void)fprintf(stdout,"%s: ERROR: Sorry, wildcarding (extended regular expression matches to variables) was not built into this NCO executable, so unable to compile regular expression \"%s\".\nHINT: Make sure libregex.a is on path and re-build NCO.\n",prg_nm_get(),var_sng);
@@ -308,7 +308,7 @@ nco_var_lst_crd_add /* [fnc] Add all coordinates to extraction list */
        all variables in file, not just over current extraction list. */ 
     /* ...for each variable in file... */
     for(idx_var=0;idx_var<nbr_var;idx_var++){
-      /* This assumption is valid in netCDF2, netCDF3, and netCDF4 */
+      /* This assumption, praise the Lord, is valid in netCDF2, netCDF3, and netCDF4 */
       var_id=idx_var;
       /* fxm: Functionalize remainder and call from both 
 	 nco_var_lst_crd_ass_add() and nco_var_lst_crd_add()? */
@@ -328,8 +328,9 @@ nco_var_lst_crd_add /* [fnc] Add all coordinates to extraction list */
 	  if(att_sz > 0) (void)nco_get_att(nc_id,var_id,att_nm,(void *)att_val,NC_CHAR);	  
 	  /* NUL-terminate attribute */
 	  att_val[att_sz]='\0';
-	  /* Split list into separate coordinate names */
-	  crd_lst=nco_lst_prs_2D(att_val,dlm_sng,&nbr_crd);
+	  /* Split list into separate coordinate names
+             Use nco_lst_prs_sgl_2D() not nco_lst_prs_2D() to avert TODO nco944 */
+	  crd_lst=nco_lst_prs_sgl_2D(att_val,dlm_sng,&nbr_crd);
 	  /* ...for each coordinate in "coordinates" attribute... */
 	  for(idx_crd=0;idx_crd<nbr_crd;idx_crd++){
 	    /* Verify "coordinate" exists in input file */
@@ -459,10 +460,13 @@ nco_var_lst_crd_ass_add /* [fnc] Add coordinates associated extracted variables
 	  if(att_sz > 0) (void)nco_get_att(nc_id,var_id,att_nm,(void *)att_val,NC_CHAR);	  
 	  /* NUL-terminate attribute */
 	  att_val[att_sz]='\0';
-	  /* Split list into separate coordinate names */
-	  crd_lst=nco_lst_prs_2D(att_val,dlm_sng,&nbr_crd);
+	  /* Split list into separate coordinate names
+             Use nco_lst_prs_sgl_2D() not nco_lst_prs_2D() to avert TODO nco944 */
+	  crd_lst=nco_lst_prs_sgl_2D(att_val,dlm_sng,&nbr_crd);
 	  /* ...for each coordinate in "coordinates" attribute... */
 	  for(idx_crd=0;idx_crd<nbr_crd;idx_crd++){
+            if(crd_lst[idx_crd]==NULL)
+              continue;
 	    /* Verify "coordinate" exists in input file */
 	    rcd=nco_inq_varid_flg(nc_id,crd_lst[idx_crd],&crd_id);
 	    /* NB: Do not check that dimension by this name exists
@@ -612,7 +616,7 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
   int idx_xcl;
   int var_op_typ[NC_MAX_VARS];
 
-  nco_bool var_typ_fnk; /* [flg] Variable type is too funky for arithmetic */
+  nco_bool var_typ_fnk=False; /* [flg] Variable type is too funky for arithmetic */ /* CEWI */
 
   nc_type var_type=NC_NAT; /* NC_NAT present in netcdf.h version netCDF 3.5+ */
 
@@ -720,6 +724,13 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
       if(prg_id == ncbo && (!strcmp(var_nm,"hyam") || !strcmp(var_nm,"hybm") || !strcmp(var_nm,"hyai") || !strcmp(var_nm,"hybi") || !strcmp(var_nm,"gw") || !strcmp(var_nm,"lon_bnds") || !strcmp(var_nm,"lat_bnds") || !strcmp(var_nm,"area") || !strcmp(var_nm,"ORO") || !strcmp(var_nm,"date") || !strcmp(var_nm,"datesec") || (strstr(var_nm,"msk_") == var_nm))) var_op_typ[idx]=fix;
     } /* end if CNV_CCM_CCSM_CF */
 
+    /* Warn about any expected weird behavior */
+    if(var_op_typ[idx] == prc){
+      if(var_typ_fnk && ((prg_id != ncecat) && (prg_id != ncpdq) && (prg_id != ncrcat))){
+	if(dbg_lvl_get() > 0) (void)fprintf(stderr,"%s: INFO Variable %s is of type %s, for which requested processing (i.e., averaging, differencing) is ill-defined\n",prg_nm_get(),var[idx]->nm,nco_typ_sng(var[idx]->type));
+      } /* end if */
+    } /* end if prc */
+
   } /* end loop over var */
 
   /* Assign list pointers based on operation type for each variable */
@@ -735,9 +746,6 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
       var_prc[*nbr_var_prc]=var[idx];
       var_prc_out[*nbr_var_prc]=var_out[idx];
       ++*nbr_var_prc;
-      if(var_typ_fnk && ((prg_id != ncecat) && (prg_id != ncpdq) && (prg_id != ncrcat))){
-	if(dbg_lvl_get() > 0) (void)fprintf(stderr,"%s: INFO Variable %s is of type %s, for which requested processing (i.e., averaging, differencing) is ill-defined\n",prg_nm_get(),var[idx]->nm,nco_typ_sng(var[idx]->type));
-      } /* end if */
     } /* end else */
   } /* end loop over var */
   
@@ -805,11 +813,28 @@ nco_var_lst_mrg /* [fnc] Merge two variable lists into same order */
  int * const var_nbr_2) /* I/O [nbr] Number of variables in list 2 */
 {
   /* Purpose: Merge two variable lists into same order
-     NB: Routine design is open-ended with maximum flexibility
+
+     Routine design is open-ended with maximum flexibility
      Initial functionality will simply sort list two into list one order and
      destroy original (un-merged) list two on output
      Refinements could include changing number of variables in each list
-     This would allow symmetric list merges */
+     This would allow asymmetric list merges
+
+     Routine is only used by ncbo to synchronize processed variable list
+     Until 20070628, routine did not change var_nbr_2
+     As of 20070628 (NCO 3.9.1) routine shrinks var_nbr_2 to var_nbr_1
+     In effect this sets nbr_var_prc_2:=nbr_var_prc_1 in ncbo
+     This allows file_2 to contain variables not in file_1
+     Routine now warns about processed variable list "truncation" 
+     "asymmetric" list processing should now work iff lst_2 is superset of lst_1
+
+     Until 20070707 lst_2 had to be superset of lst_1
+     Next desired functionality is to copy variables only in lst_1 or lst_2 as
+     fixed variables to output file, i.e., do not subtract them.
+     May only be possible with lst_1 variables
+     At minimum, want to print which variables are _only_ in which file
+     This would make it easier for user to give -x -v var_lst argument which
+     would make files subtractable */
 
   const char fnc_nm[]="nco_var_lst_mrg()"; /* [sng] Function name */
 
@@ -842,6 +867,16 @@ nco_var_lst_mrg /* [fnc] Merge two variable lists into same order */
     var_out[idx_1]=var_2[idx_2];
   } /* end loop over idx_1 */
 
+  /* Asymmetric lists */
+  if(*var_nbr_2 > *var_nbr_1){
+    (void)fprintf(stderr,"%s: WARNING %s detects that file two has more variables than file one. The following variables, present only in file two, will not be present in the output file: %s",prg_nm_get(),fnc_nm,var_2[*var_nbr_1]->nm);
+    for(idx_2=*var_nbr_1+1;idx_2<*var_nbr_2;idx_2++){
+      (void)fprintf(stderr,", %s",var_2[idx_2]->nm);
+    } /* end loop over idx_2 */
+    (void)fprintf(stderr,"\n");
+    *var_nbr_2=*var_nbr_1;
+  } /* end if asymmetric */
+
   /* Free un-merged list before overwriting with merged list */
   var_2=(var_sct **)nco_free(var_2);
   *var_2_ptr=(var_sct **)nco_realloc(var_out,*var_nbr_2*sizeof(var_sct *));
