@@ -1,9 +1,9 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_utl.c,v 1.124 2007/05/25 05:24:23 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_utl.c,v 1.143 2009/05/29 20:36:32 zender Exp $ */
 
 /* Purpose: Variable utilities */
 
-/* Copyright (C) 1995--2007 Charlie Zender
-   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+/* Copyright (C) 1995--2009 Charlie Zender
+   License: GNU General Public License (GPL) Version 3
    See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_var_utl.h" /* Variable utilities */
@@ -13,13 +13,16 @@ nco_cpy_var_dfn /* [fnc] Copy variable metadata from input to output file */
 (const int in_id, /* I [id] netCDF input file ID */
  const int out_id, /* I [id] netCDF output file ID */
  const int rec_dmn_id, /* I [id] Input file record dimension ID  */
- const char * const var_nm) /* I [sng] Input variable name */
+ const char * const var_nm, /* I [sng] Input variable name */
+ const int dfl_lvl) /* I [enm] Deflate level [0..9] */
 {
   /* Purpose: Copy variable metadata from input netCDF file to output netCDF file
      Routine does not take into account any user-specified limits
      It just copies what it finds
      Routine copies_variable by variable, old-style, used only by ncks */
 
+  const char fnc_nm[]="nco_cpy_var_dfn()"; /* [sng] Function name */
+
   int *dmn_in_id;
   int *dmn_out_id;
   int idx;
@@ -70,12 +73,39 @@ nco_cpy_var_dfn /* [fnc] Copy variable metadata from input to output file */
 	(void)nco_def_dim(out_id,dmn_nm,NC_UNLIMITED,dmn_out_id+idx);
       } /* end else */
     } /* end if */
+
   } /* end loop over dim */
   
   /* Define variable in output file */
   (void)nco_def_var(out_id,var_nm,var_type,nbr_dim,dmn_out_id,&var_out_id);
-  
-  /* Free the space holding dimension IDs */
+
+  /* Deflation */
+  if(nbr_dim > 0){
+    int shuffle; /* [flg] Turn on shuffle filter */
+    int deflate; /* [flg] Turn on deflate filter */
+    int dfl_lvl_in; /* [enm] Deflate level [0..9] */
+    rcd=nco_inq_var_deflate(in_id,var_in_id,&shuffle,&deflate,&dfl_lvl_in);
+    /* Copy original deflation settings */
+    if(deflate || shuffle) (void)nco_def_var_deflate(out_id,var_out_id,deflate,shuffle,dfl_lvl_in);
+    /* Overwrite HDF Lempel-Ziv compression level, if requested */
+    if(dfl_lvl > 0) (void)nco_def_var_deflate(out_id,var_out_id,(int)True,(int)True,dfl_lvl);
+  } /* endif */
+
+  /* Chunking */
+  if(nbr_dim > 0){
+    int srg_typ; /* [enm] Storage type */
+    size_t *cnk_sz; /* [nbr] Chunksize list */
+    cnk_sz=(size_t *)nco_malloc(nbr_dim*sizeof(size_t));
+    rcd=nco_inq_var_chunking(in_id,var_in_id,&srg_typ,cnk_sz);
+    /* Copy original chunking settings */
+    if(srg_typ == NC_CHUNKED){
+      if(dbg_lvl_get() >= nco_dbg_var ) (void)fprintf(stdout,"%s: DEBUG %s copying input-to-ouput chunking information for %s\n",prg_nm_get(),fnc_nm,var_nm);
+      (void)nco_def_var_chunking(out_id,var_out_id,srg_typ,cnk_sz);
+    } /* endif */
+    cnk_sz=(size_t *)nco_free(cnk_sz);
+  } /* endif */
+
+  /* Free space holding dimension IDs */
   dmn_in_id=(int *)nco_free(dmn_in_id);
   dmn_out_id=(int *)nco_free(dmn_out_id);
   
@@ -89,7 +119,8 @@ nco_cpy_var_dfn_lmt /* Copy variable metadata from input to output file */
  const int rec_dmn_id, /* I [id] Input file record dimension ID  */
  const char * const var_nm, /* I [sng] Input variable name */
  CST_X_PTR_CST_PTR_CST_Y(lmt_all_sct,lmt_all_lst), /* I [sct] Hyperslab limits */
- const int lmt_all_lst_nbr) /* I [nbr] Number of hyperslab limits */
+ const int lmt_all_lst_nbr, /* I [nbr] Number of hyperslab limits */
+ const int dfl_lvl) /* I [enm] Deflate level [0..9] */
 {
   /* Purpose: Copy variable metadata from input netCDF file to output netCDF file
      This routine truncates dimensions in variable definition in output file according to user-specified limits.
@@ -142,7 +173,7 @@ nco_cpy_var_dfn_lmt /* Copy variable metadata from input to output file */
       if(dmn_in_id[idx] != rec_dmn_id){
 	int lmt_all_idx;
 
-	/* Decide whether this dimension has any user-specified limits */
+	/* Does dimension have any user-specified limits? */
 	for(lmt_all_idx=0;lmt_all_idx<lmt_all_lst_nbr;lmt_all_idx++){
 	  if(lmt_all_lst[lmt_all_idx]->lmt_dmn[0]->id == dmn_in_id[idx]){
 	    dmn_sz=lmt_all_lst[lmt_all_idx]->dmn_cnt;
@@ -159,6 +190,29 @@ nco_cpy_var_dfn_lmt /* Copy variable metadata from input to output file */
   /* Define variable in output file */
   (void)nco_def_var(out_id,var_nm,var_type,nbr_dim,dmn_out_id,&var_out_id);
   
+  /* Deflation */
+  if(nbr_dim > 0){
+    int shuffle; /* [flg] Turn on shuffle filter */
+    int deflate; /* [flg] Turn on deflate filter */
+    int dfl_lvl_in; /* [enm] Deflate level [0..9] */
+    rcd=nco_inq_var_deflate(in_id,var_in_id,&shuffle,&deflate,&dfl_lvl_in);
+    /* Copy original deflation settings */
+    if(deflate || shuffle) (void)nco_def_var_deflate(out_id,var_out_id,deflate,shuffle,dfl_lvl_in);
+    /* Overwrite HDF Lempel-Ziv compression level, if requested */
+    if(dfl_lvl > 0) (void)nco_def_var_deflate(out_id,var_out_id,(int)True,(int)True,dfl_lvl);
+  } /* endif */
+
+  /* Chunking */
+  if(nbr_dim > 0){
+    int srg_typ; /* [enm] Storage type */
+    size_t *cnk_sz; /* [nbr] Chunksize list */
+    cnk_sz=(size_t *)nco_malloc(nbr_dim*sizeof(size_t));
+    rcd=nco_inq_var_chunking(in_id,var_in_id,&srg_typ,cnk_sz);
+    /* Copy original chunking settings */
+    if(srg_typ == NC_CHUNKED) (void)nco_def_var_chunking(out_id,var_out_id,srg_typ,cnk_sz);
+    cnk_sz=(size_t *)nco_free(cnk_sz);
+  } /* endif */
+
   /* Free space holding dimension IDs */
   dmn_in_id=(int *)nco_free(dmn_in_id);
   dmn_out_id=(int *)nco_free(dmn_out_id);
@@ -168,11 +222,11 @@ nco_cpy_var_dfn_lmt /* Copy variable metadata from input to output file */
 
 void
 nco_cpy_var_val /* [fnc] Copy variable from input to output file, no limits */
-(int in_id, /* I [id] netCDF input file ID */
- int out_id, /* I [id] netCDF output file ID */
+(const int in_id, /* I [id] netCDF input file ID */
+ const int out_id, /* I [id] netCDF output file ID */
  FILE * const fp_bnr, /* I [fl] Unformatted binary output file handle */
  const nco_bool NCO_BNR_WRT, /* I [flg] Write binary file */
- char *var_nm) /* I [sng] Variable name */
+ const char *var_nm) /* I [sng] Variable name */
 {
   /* NB: nco_cpy_var_val() contains OpenMP critical region */
   /* Purpose: Copy single variable from input netCDF file to output netCDF file
@@ -536,30 +590,48 @@ nco_var_dpl /* [fnc] Duplicate input variable */
 {
   /* Threads: Routine is thread safe and calls no unsafe routines */
   /* Purpose: nco_malloc() and return duplicate of input var_sct
-     Duplicate is deep copy of original so original may always be free()'d */
+     Duplicate is deep-copy of original so original may always be free()'d */
 
   const char fnc_nm[]="nco_var_dpl()"; /* [sng] Function name */
   var_sct *var_cpy;
 
   var_cpy=(var_sct *)nco_malloc(sizeof(var_sct));
 
-  /* Shallow copy structure */
+  /* Shallow-copy structure itself */
   (void)memcpy((void *)var_cpy,(const void *)var,sizeof(var_sct));
 
   /* Make sure var_free() frees names when variable is destructed */
   if(var->nm != NULL) var_cpy->nm=(char *)strdup(var->nm);
 
-  /* Deep copy dyamically allocated arrays currently defined in original */
+  /* Deep-copy dyamically allocated arrays from original to copy */
   if(var->val.vp != NULL){
-    var_cpy->val.vp=(void *)nco_malloc_dbg(var_cpy->sz*nco_typ_lng(var_cpy->type),"Unable to malloc() value buffer in variable deep copy",fnc_nm);
+    var_cpy->val.vp=(void *)nco_malloc_dbg(var_cpy->sz*nco_typ_lng(var_cpy->type),"Unable to malloc() value buffer in variable deep-copy",fnc_nm);
     (void)memcpy((void *)(var_cpy->val.vp),(void *)(var->val.vp),var_cpy->sz*nco_typ_lng(var_cpy->type));
-  } /* end if */
+    /* Deep-copy string data, if any */
+    if(var->type == (nc_type)NC_STRING){
+      ptr_unn val_in;
+      ptr_unn val_out;
+      long idx;
+      long sz;
+      sz=var->sz;  
+      val_in=var->val;
+      val_out=var_cpy->val;
+      /* Typecast pointer to values before access 
+	 Use local copies of pointer unions to maintain const-ness of var */
+      (void)cast_void_nctype((nc_type)NC_STRING,&val_in); 
+      (void)cast_void_nctype((nc_type)NC_STRING,&val_out); 
+      for(idx=0;idx<sz;idx++) val_out.sngp[idx]=(char *)strdup(val_in.sngp[idx]);
+      /* NB: we operated on local copies of val_in and val_out
+	 Neither is used again in this routine
+	 Therefore not necessary to un-typecast pointer unions */
+    } /* endif type */
+  } /* end if val */
   if(var->mss_val.vp != NULL){
     var_cpy->mss_val.vp=(void *)nco_malloc(nco_typ_lng(var_cpy->type));
     (void)memcpy((void *)(var_cpy->mss_val.vp),(void *)(var->mss_val.vp),nco_typ_lng(var_cpy->type));
   } /* end if */
   if(var->tally != NULL){
-    var_cpy->tally=(long *)nco_malloc_dbg(var_cpy->sz*sizeof(long),"Unable to malloc() tally buffer in variable deep copy",fnc_nm);
+    var_cpy->tally=(long *)nco_malloc_dbg(var_cpy->sz*sizeof(long),"Unable to malloc() tally buffer in variable deep-copy",fnc_nm);
     (void)memcpy((void *)(var_cpy->tally),(void *)(var->tally),var_cpy->sz*sizeof(long));
   } /* end if */
   if(var->dim != NULL){
@@ -570,6 +642,10 @@ nco_var_dpl /* [fnc] Duplicate input variable */
     var_cpy->dmn_id=(int *)nco_malloc(var_cpy->nbr_dim*sizeof(int));
     (void)memcpy((void *)(var_cpy->dmn_id),(void *)(var->dmn_id),var_cpy->nbr_dim*sizeof(var->dmn_id[0]));
   } /* end if */
+  if(var->cnk_sz != NULL){
+    var_cpy->cnk_sz=(size_t *)nco_malloc(var_cpy->nbr_dim*sizeof(size_t));
+    (void)memcpy((void *)(var_cpy->cnk_sz),(void *)(var->cnk_sz),var_cpy->nbr_dim*sizeof(var->cnk_sz[0]));
+  } /* end if */
   if(var->cnt != NULL){
     var_cpy->cnt=(long *)nco_malloc(var_cpy->nbr_dim*sizeof(long));
     (void)memcpy((void *)(var_cpy->cnt),(void *)(var->cnt),var_cpy->nbr_dim*sizeof(var->cnt[0]));
@@ -613,7 +689,7 @@ nco_var_get /* [fnc] Allocate, retrieve variable hyperslab from disk to memory *
 
   var->val.vp=(void *)nco_malloc_dbg(var->sz*nco_typ_lng(var->typ_dsk),"Unable to malloc() value buffer when retrieving variable from disk",fnc_nm);
 
-  if(False) (void)fprintf(stdout,"%s: DEBUG: fxm TODO nco354. Calling nco_get_vara() for variable %s with nc_id=%d, var_id=%d, var_srt=%li, var_cnt = %li, var_val = %g, var_typ = %s\n",prg_nm_get(),var->nm,nc_id,var->id,var->srt[0],var->cnt[0],var->val.fp[0],nco_typ_sng(var->typ_dsk));
+  if(False) (void)fprintf(stdout,"%s: DEBUG: fxm TODO nco354. Calling nco_get_vara() for %s with nc_id=%d, var_id=%d, var_srt=%li, var_cnt = %li, var_val = %g, var_typ = %s\n",prg_nm_get(),var->nm,nc_id,var->id,var->srt[0],var->cnt[0],var->val.fp[0],nco_typ_sng(var->typ_dsk));
 
   /* 20051021: Removed this potentially critical region by parallelizing 
      over in_id's in calling code */
@@ -703,11 +779,13 @@ nco_var_free /* [fnc] Free all memory associated with variable structure */
   /* Threads: Routine is thread safe and calls no unsafe routines */
   /* Purpose: Free all memory associated with a dynamically allocated variable structure */
 
+  /* String values must be deep-free'd, everything else is a flat buffer */
+  if(var->type == (nc_type)NC_STRING && var->val.vp != (void *)NULL) var->val.vp=(void *)nco_sng_lst_free((char **)var->val.vp,var->sz); else var->val.vp=nco_free(var->val.vp);
   var->nm=(char *)nco_free(var->nm);
-  var->val.vp=nco_free(var->val.vp);
   var->mss_val.vp=nco_free(var->mss_val.vp);
   var->tally=(long *)nco_free(var->tally);
   var->dmn_id=(int *)nco_free(var->dmn_id);
+  var->cnk_sz=(size_t *)nco_free(var->cnk_sz);
   var->dim=(dmn_sct **)nco_free(var->dim);
   var->srt=(long *)nco_free(var->srt);
   var->end=(long *)nco_free(var->end);
@@ -772,6 +850,7 @@ var_dfl_set /* [fnc] Set defaults for each member of variable structure */
   var->nbr_att=-1;
   var->dim=(dmn_sct **)NULL;
   var->dmn_id=(int *)NULL;
+  var->cnk_sz=(size_t *)NULL;
   var->cnt=(long *)NULL;
   var->srt=(long *)NULL;
   var->end=(long *)NULL;
@@ -812,13 +891,14 @@ nco_var_dfn /* [fnc] Define variables and write their attributes to output file
  CST_X_PTR_CST_PTR_CST_Y(dmn_sct,dmn_ncl), /* I [sct] Dimensions included in output file */
  const int nbr_dmn_ncl, /* I [nbr] Number of dimensions in list */
  const int nco_pck_map, /* I [enm] Packing map */
- const int nco_pck_plc) /* I [enm] Packing policy */
+ const int nco_pck_plc, /* I [enm] Packing policy */
+ const int dfl_lvl) /* I [enm] Deflate level [0..9] */
 {
   /* Purpose: Define variables in output file, copy their attributes */
 
   /* This function is unusual (for me) in that dimension arguments are only intended
      to be used by certain programs, those that alter the rank of input variables. 
-     If program does not alter input variable rank (dimensionality) then it should
+     Programs that do not alter input variable rank (dimensionality) should
      call this function with NULL dimension list and nbr_dmn_ncl=0. 
      Otherwise, this routine attempts to define variable correctly in output file 
      (allowing variable to be defined with only those dimensions that are in dimension inclusion list) 
@@ -926,6 +1006,9 @@ nco_var_dfn /* [fnc] Define variables and write their attributes to output file
       /* The all-important variable definition call itself... */
       (void)nco_def_var(out_id,var[idx]->nm,typ_out,dmn_nbr,dmn_id_vec,&var[idx]->id);
       
+      /* Set HDF Lempel-Ziv compression level, if requested */
+      if(dfl_lvl > 0 && dmn_nbr > 0) (void)nco_def_var_deflate(out_id,var[idx]->id,(int)True,(int)True,dfl_lvl);
+
       if(dbg_lvl_get() > 3 && prg_id != ncwa){
 	/* fxm TODO nco374 diagnostic information fails for ncwa since var[idx]->dim[dmn_idx]->nm
 	   contains _wrong name_ when variables will be averaged.
@@ -1095,6 +1178,7 @@ nco_var_fll /* [fnc] Allocate variable structure and fill with metadata */
   /* Allocate space for dimension information */
   if(var->nbr_dim > 0) var->dim=(dmn_sct **)nco_malloc(var->nbr_dim*sizeof(dmn_sct *)); else var->dim=(dmn_sct **)NULL;
   if(var->nbr_dim > 0) var->dmn_id=(int *)nco_malloc(var->nbr_dim*sizeof(int)); else var->dmn_id=(int *)NULL;
+  if(var->nbr_dim > 0) var->cnk_sz=(size_t *)nco_malloc(var->nbr_dim*sizeof(size_t)); else var->cnk_sz=(size_t *)NULL;
   if(var->nbr_dim > 0) var->cnt=(long *)nco_malloc(var->nbr_dim*sizeof(long)); else var->cnt=(long *)NULL;
   if(var->nbr_dim > 0) var->srt=(long *)nco_malloc(var->nbr_dim*sizeof(long)); else var->srt=(long *)NULL;
   if(var->nbr_dim > 0) var->end=(long *)nco_malloc(var->nbr_dim*sizeof(long)); else var->end=(long *)NULL;
